
/*
 * +==========================================================
 * |	'id_ca.h':
 * |	Resource cache manager defs.
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#ifndef __ID_CA_H__
#define __ID_CA_H__

/* ======================================================================== */

#define NUMMAPS		60
#define MAPPLANES	2

typedef	struct
{
	int planestart[3];
	int planelength[3];
	int width, height;
	char name[16];
} maptype;

/* ======================================================================== */

extern	word	*mapsegs[MAPPLANES];
extern	maptype	*mapheaderseg[NUMMAPS];

extern char extension[5];

// Locate game resource
char *resource_name (const char *base_name, boolean global);

/* ======================================================================== */

boolean CA_LoadFile(char *filename, byte **ptr);
boolean CA_WriteFile(char *filename, void *ptr, long length);

void CA_RLEWexpand(word *source, word *dest, long length, word rlewtag);

void CA_Startup();
void CA_Shutdown();

void CA_CacheAudioChunk(int chunk);
void CA_UnCacheAudioChunk(int chunk);
byte *CA_GetAudioChunk(int chunk);

void CA_CacheMusicChunk(int chunk);
void CA_UnCacheMusicChunk(int chunk);
byte *CA_GetMusicChunk (int chunk);

void CA_CacheGrChunk(int chunk);
void CA_UnCacheGrChunk(int chunk);

void CA_CacheMap(int mapnum);

void CacheGrLump (int lumpstart,int lumpend);
void UnCacheGrLump (int lumpstart,int lumpend);

byte *CA_GetGrChunk(int chunk);

void GetPicDims (int ichunk, int *width, int *height);

int GetGrLogical (int p_chunk);

/* ======================================================================= */

void MM_Startup();
void MM_Shutdown();

void *MM_Alloc(unsigned long size, char *tag);
void MM_Free(void *block);

#define PMPageSize	4096

#define	PMPageNo(size)	(((size) + (PMPageSize - 1)) / PMPageSize)

typedef	struct {
	int offset;	/* Offset of chunk into file */
	int length;	/* Length of the chunk */
	byte *addr;
} PageListStruct;

// Get overall information about sounds
void GetAudioTotals (unsigned *p_count, unsigned *p_npages);

// Get information about `sound`
void GetAudioInfo (unsigned sound, unsigned *p_page, unsigned *p_length);

// Locate sound `page`
int LocateSoundPage (unsigned page);

extern int ChunksInFile, PMSpriteStart, PMSoundStart, PMSoundInfo;

extern PageListStruct *PMPages;

#define	PM_GetSoundPage(v)	PM_GetPage(PMSoundStart + (v))
byte *PM_GetPage(int pagenum);
void PM_FreePage(int pagenum);

void PM_Startup();
void PM_Shutdown();

// Bounding box (for sprites)
typedef struct {
	// top, left (incl);
	// bottom, right (excl)
	byte t, l, b, r;
	} BBox;

void DeCompileSprite(byte *ptr, byte *buf, BBox *area);

byte *GetSprite(int shapenum, BBox **p_box);

#endif
