
/*
 * +==========================================================
 * |	'misc.c':
 * |	Miscellaneous low-level routines
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
              
#include "wl_def.h"

#include "_mingw.h"

#ifndef USE_clock
#include "SDL.h"
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif

long filelength(int handle)
{
	struct stat buf;
	
	if (fstat(handle, &buf) == -1) {
		perror("filelength");
		exit(EXIT_FAILURE);
	}
	
	return buf.st_size;
}

char *strlwr(char *s)
{
	char *p = s;
	
	while (*p) {
		*p = tolower(*p);
		p++;
	}
	
	return s;
}
	
uint16_t get_16(byte *data) {
return *data | ((uint16_t) data[1] << 8);
}	// get_16

uint32_t get_32(byte *data) {
return get_16(data) | ((uint32_t) get_16(data+2) << 16);
}	// get_32

void put_16(byte *data, uint16_t v) {
*data++ = v;
*data = v >> 8;
}	// put_16

void put_32(byte *data, uint32_t v) {
put_16 (data, v);
put_16 (data+2, v >> 16);
}	// put_32

int OpenWrite(char *fn)
{
	return open(fn, O_CREAT|O_WRONLY|O_TRUNC|O_BINARY,
		S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
}

int OpenWriteAppend(char *fn)
{
	return open(fn, O_CREAT|O_WRONLY|O_BINARY,
		S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
}

void CloseWrite(int fp)
{
	close(fp);
}

int WriteSeek(int fp, int offset, int whence)
{
	return lseek(fp, offset, whence);
}

int WritePos(int fp)
{
	return lseek(fp, 0, SEEK_CUR);
}

int WriteInt8(int fp, int8_t d)
{
	return write(fp, &d, 1);
}

int WriteInt16(int fp, int16_t d)
{
	byte a[2];

	put_16(a, d);

	return write(fp, a, 2) / 2;
}

int WriteInt32(int fp, int32_t d)
{
	byte a[4];

	put_32(a, d);
	
	return write(fp, a, 4) / 4;
}

int WriteBytes(int fp, byte *d, int len)
{
	return write(fp, d, len);
}

int OpenRead(char *fn)
{
	return open(fn, O_RDONLY | O_BINARY);
}

void CloseRead(int fp)
{
	close(fp);
}

int ReadSeek(int fp, int offset, int whence)
{
	return lseek(fp, offset, whence);
}

int ReadLength(int fp)
{
	return filelength(fp);
}

int8_t ReadInt8(int fp)
{
	byte d;
	
	read(fp, &d, 1);
	
	return d;
}

int16_t ReadInt16(int fp)
{
	byte d[2];
	
	read(fp, d, 2);
	
	return get_16(d);
}

int32_t ReadInt32(int fp)
{
	byte d[4];
	
	read(fp, d, 4);
	
	return get_32(d);
}

int ReadBytes(int fp, byte *d, int len)
{
	return read(fp, d, len);
}
