
/*
 * +==========================================================
 * |	'sd_sdl.c':
 * |	Sound output engine: SDL implementation
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"
#include "SDL.h"

#include "fmopl.h"

// General audio frequency
#define	AUDIO_FREQ	22050

// Max # of audio samples in buffer
#define	AUDIO_SAMPLES	(1 << 12)

// (digisounds sampling frequency)
#define	digi_freq	(7 << 10)

//
// (digisounds debugging)
//

#ifdef	SG_CHANNEL

//
// Digital sounds playback
//
static byte *digi_ptr = NULL;		// (ptr to sampled digital data)
static unsigned digi_page = 0;		// (# of current digipage)
static unsigned digi_cnt = 0;		// (# of samples remaining in page)
static unsigned digi_total = 0;		// (# of total samples remaining to play)

// Reset digital sounds playback
static void digi_reset () {
digi_ptr = NULL;
digi_page = digi_cnt = digi_total = 0;
}	// digi_reset

// Stop digisound playback
static void digi_stop () {
SDL_LockAudio ();

digi_reset();

SDL_UnlockAudio ();
}	// digi_stop

// Play digital sound
static void digi_play (unsigned page, unsigned length) {
SDL_LockAudio ();

digi_page = page;
digi_total = length;
digi_cnt = 0;

SDL_UnlockAudio ();
}	// digi_play

// Feed digital data ('len' samples to 'stream' buffer)
// Return # of samples remaining
static unsigned digi_feed (Uint8 *stream, int len) {
while (len) {
unsigned count;

if (! digi_cnt) {
	if (! digi_total)
		{ digi_reset(); return len; }

	digi_ptr = PM_GetPage (digi_page ++);

	if (digi_ptr == 0)		// (nonexistent page!)
		{ digi_reset(); return len; }

	digi_cnt = digi_total < PMPageSize ? digi_total : PMPageSize;
	digi_total -= digi_cnt;
	}

count = len < digi_cnt ? len : digi_cnt;

memcpy (stream, digi_ptr, count);
digi_ptr += count;
digi_cnt -= count;
stream += count;
len -= count;
}	// (len)

return 0;
}	// digi_feed

#else					// SG_CHANNEL

// Total # of mixing channels
enum { N_CHAN = 8 };

// Digital playback channel
typedef struct {
	byte *digi_ptr;				// (ptr to sampled digital data)
	unsigned digi_page;		// (# of current digipage)
	unsigned digi_cnt;		// (# of samples remaining in page)
	unsigned digi_total;	// (# of total samples remaining to play)

	int id;								// Object id (if -1, none)
	fixed loc_x, loc_y;		// Sound location

	// Channel volumes: 0(max) .. 8(min)
	Uint8 LVol, RVol;
	} digi_chan;

// Array of channels
static digi_chan digi_array[N_CHAN];

void SetVolume (digi_chan *chan);

// Active channels table & count
// Channels 0..>acch_cnt are being used;
//	acch_cnt..>N_CHAN are vacant.
static unsigned acch_tab[N_CHAN];
static unsigned acch_cnt;

/* Define this to enable channel debugging */
#undef	DIGI_DEBUG

#ifdef	DIGI_DEBUG

void DBG_AddCh(unsigned ch) {
Diag(0, "DD: +%u", ch);
}	// DBG_AddCh

void DBG_DelCh(unsigned ch) {
Diag(0, "DD: -%u", ch);
}	// DBG_DelCh

void DBG_LstCh() {
unsigned i;
char outbuf[80];

sprintf(outbuf, "DD: ={");
for (i = 0; i != acch_cnt; ++ i)
	sprintf(outbuf+strlen(outbuf), "%u%s", acch_tab[i], i != acch_cnt-1 ? "," : "");
sprintf(outbuf+strlen(outbuf), "}");

Diag(0, outbuf);
}	// DBG_LstCh

#else

#define	DBG_AddCh(ch)

#define	DBG_DelCh(ch)

#define	DBG_LstCh()

#endif

// Reset digital sounds playback (channel 'no')
static void digi_reset_ch (unsigned no) {
  digi_chan *chan = digi_array+no;

	chan->digi_ptr = NULL;
	chan->digi_page = chan->digi_cnt = chan->digi_total = 0;

	// (mark this channel as vacant, if needed)
  unsigned i;
  for (i = 0; i != acch_cnt; ++ i)
		if (acch_tab[i] == no) {			// (found)
			if (-- acch_cnt != i) {			// (swapping indexes)
				acch_tab[i] = acch_tab[acch_cnt];		// (last used...)
				acch_tab[acch_cnt] = no;						// (...to become first free)
				}

			DBG_DelCh(no);
			break;
			}

}	// digi_reset_ch

// Reset digital sounds playback
static void digi_reset () {
unsigned i;

// TODO: if some channels were active before?

acch_cnt = 0;				// (no channels active)

for (i = 0; i != N_CHAN; ++ i) {
	digi_reset_ch (i);
	acch_tab[i] = i;		// (initially: # of channel)
	}
}	// digi_reset

// Stop digisound playback
static void digi_stop () {
SDL_LockAudio ();

digi_reset();

SDL_UnlockAudio ();
}	// digi_stop

// Play digital sound
static void digi_play (unsigned page, unsigned length,
		int id, fixed loc_x, fixed loc_y) {
SDL_LockAudio ();

// (get first free channel)
if (acch_cnt != N_CHAN) {
	unsigned ch = acch_tab[acch_cnt ++];		// (allocate)
	digi_chan *chan = digi_array + ch;

	if (chan->digi_total == 0) {
		chan->digi_page = page;
		chan->digi_total = length;
		chan->digi_cnt = 0;

		chan->id = id;
		chan->loc_x = loc_x, chan->loc_y = loc_y;
		SetVolume (chan);
		}

  DBG_AddCh (ch);
	}

SDL_UnlockAudio ();
}	// digi_play

// Feed digital data ('len' samples to 'stream' buffer)
// from channel 'no'.
// Return # of samples remaining
static unsigned digi_feed_ch (unsigned no, Uint8 *stream, int len) {
digi_chan *chan = digi_array+no;

while (len) {
unsigned count;

if (! chan->digi_cnt) {
	if (! chan->digi_total)
		{ digi_reset_ch(no); return len; }

	chan->digi_ptr = PM_GetPage (chan->digi_page ++);

	if (chan->digi_ptr == 0)		// (nonexistent page!)
		{ digi_reset_ch(no); return len; }

	chan->digi_cnt = chan->digi_total < PMPageSize ? chan->digi_total : PMPageSize;
	chan->digi_total -= chan->digi_cnt;
	}

count = len < chan->digi_cnt ? len : chan->digi_cnt;

memcpy (stream, chan->digi_ptr, count);
chan->digi_ptr += count;
chan->digi_cnt -= count;
stream += count;
len -= count;
}	// (len)

return 0;
}	// digi_feed_ch

#endif				// SG_CHANNEL

//
// Resamples audio data block
//
void ResampleAudioBlock (Uint8 *s_data, unsigned s_length,
	Uint8 *d_data, unsigned d_length) {
unsigned i;

if (d_length > s_length) {
    unsigned num = d_length * s_length;
    if (d_length & (d_length - 1)) {
    for (i = d_length; -- i; ) {
			num -= s_length;
			d_data[i] = s_data[num / d_length];
			}
    }
    else {		// (d_length is power of 2: optimize)
    for (i = d_length, d_length --; -- i; ) {
			num -= s_length;
			d_data[i] = s_data[num & d_length];
			}
    }
    }

else {	// (this can't happen...)
	}
}	// ResampleAudioBlock

//
//
// FM synth emulation
//
//

//
// (music/sfx structures)
//

#define PACKED __attribute__((packed))

typedef struct {
	byte reg;		// (OPL register)
	byte value;		// (OPL value)
	word delay;		// (delay)
} PACKED MusicRec;

typedef	struct {
	word length;		// (total length)
	MusicRec values[1];
} PACKED MusicGroup;

typedef	struct {
	Uint32 length;
	Uint16 priority;
} PACKED SoundCommon;

typedef	struct {
	byte	mChar,	cChar,
		mScale,	cScale,
		mAttack,cAttack,
		mSus,cSus,
		mWave,cWave,
		nConn,
		// These are only for Muse - these bytes are really unused
		voice,
		mode,
		unused[3];
} PACKED Instrument;

typedef	struct {
	SoundCommon	common;
	Instrument	inst;
	byte block, data[1];
} PACKED AdLibSound;

// Virtual OPL
FM_OPL *OPL;

// Music sampling countdown (-1 -> inactive)
int mus_remain = -1;
// Sound sampling countdown (-1 -> inactive)
int sfx_remain = -1;

static MusicGroup *mus_data = NULL;		// (music played)
static MusicRec *mus_Ptr = NULL;		// (ptr to music data)
static Uint16 mus_Len = 0;			// (length of music data)
static Uint32 mus_Time = 0;
static Uint32 mus_TimeCount = 0;

// Start music playback
static void mus_play(MusicGroup *music)
{
SDL_LockAudio();

mus_data = music;
mus_Ptr = music->values;
mus_Len = music->length;
mus_Time = 0;
mus_TimeCount = 0;

mus_remain = 0;

SDL_UnlockAudio();
}	// mus_play

// Stop music playback
static void mus_stop() {
SDL_LockAudio();

mus_data = NULL;
mus_remain = -1;

SDL_UnlockAudio();
}	// mus_stop

static byte *sfx_data = NULL;		// Sound data
static byte sfx_block = 0;		// Sound block
static unsigned sfx_len = 0;		// Sound length
static Uint16 sfx_prio = 0;

// Start SFX playback
static void sfx_play(AdLibSound *sound)
{
SDL_LockAudio();

if (sound->common.priority >= sfx_prio) {
	sfx_data=sound->data;
	sfx_block=((sound->block&7)<<2) | 0x20;
	sfx_len=sound->common.length;
	sfx_prio = sound->common.priority;

	Instrument *inst = &sound->inst;

	OPLWrite(OPL, alFreqL, 0);
	OPLWrite(OPL, alFreqH, 0);

	// Loading instrument data
	OPLWrite(OPL, 0+alChar, inst->mChar);
	OPLWrite(OPL, 0+alScale, inst->mScale);
	OPLWrite(OPL, 0+alAttack, inst->mAttack);
	OPLWrite(OPL, 0+alSus, inst->mSus);
	OPLWrite(OPL, 0+alWave, inst->mWave);
	OPLWrite(OPL, 3+alChar, inst->cChar);
	OPLWrite(OPL, 3+alScale, inst->cScale);
	OPLWrite(OPL, 3+alAttack, inst->cAttack);
	OPLWrite(OPL, 3+alSus, inst->cSus);
	OPLWrite(OPL, 3+alWave, inst->cWave);
	OPLWrite(OPL, alFeedCon, 0);

	sfx_remain = 0;
	}

SDL_UnlockAudio();
}	// sfx_play

// Stop SFX playback
static void sfx_stop() {
SDL_LockAudio();

sfx_remain = -1;

sfx_data = NULL;
sfx_block = 0;
sfx_len = 0;
sfx_prio = 0;

SDL_UnlockAudio();
}	// sfx_stop

// FX stream delay (22050/160 = 138 Hz)
#define	sfx_Interval	160

// Music stream delay (22050/32 = 689 Hz)
#define	mus_Interval	32

// Update OPL buffer
// (returns # of samples left)
unsigned UpdateOPL(Uint16 *buffer, int size) {
while (size) {
if (!mus_remain) {		// Music stream generate

	while(mus_Len && (mus_Time <= mus_TimeCount)) {
		OPLWrite (OPL, mus_Ptr->reg, mus_Ptr->value);
		mus_Time = mus_TimeCount + (mus_Ptr ++)->delay;
		mus_Len -= 4;
		}

	mus_TimeCount++;
	if(!mus_Len) {
		mus_Ptr = mus_data->values;
		mus_Len = mus_data->length;
		mus_TimeCount = mus_Time = 0;
		}

	mus_remain = mus_Interval;
	}	// mus_remain

if (!sfx_remain) {		// SFX stream generate
	byte value = *sfx_data++;
	if (value) {
		OPLWrite(OPL, alFreqL, value);
		OPLWrite(OPL, alFreqH, sfx_block);
		}
	else	OPLWrite(OPL, alFreqH, 0);

	if(--sfx_len)
		sfx_remain = sfx_Interval;
	else {
		OPLWrite(OPL, alFreqH, 0);
		sfx_prio = 0;
		sfx_remain = -1;
		}
	}	// sfx_remain

unsigned len;

if (mus_remain < 0)
	if (sfx_remain < 0)
		return size;
	else	len = sfx_remain;
else	if (sfx_remain < 0)
		len = mus_remain;
	else	len = (sfx_remain < mus_remain)? sfx_remain : mus_remain;

if (len > size) len = size;

if (len) {
	YM3812UpdateOne(OPL, buffer, len);
	buffer += len;
	size -= len;
	if (mus_remain >= 0) mus_remain -= len;
	if (sfx_remain >= 0) sfx_remain -= len;
	}	// (len)

}	// while (size)

return 0;
}	// UpdateOPL

//
//
//

enum	{ MIN_S16 = -(1 << 15), MAX_S16 = 1 << 15 };

#ifdef SG_CHANNEL

/* Mix digital (8 bit unsigned mono) data to target buffer */
static void DigiMixer (Uint8 *src, Uint8 *dest, unsigned count) {
Sint16 *dest16 = (Sint16 *) dest;
while (count --) {
  Sint32 value = ((*src ++ - 128) << 8) + *dest16;

  if (value < MIN_S16)			value = MIN_S16;
  else if (value > MAX_S16)	value = MAX_S16;

	*dest16 ++ = value;
	*dest16 ++ = value;
	}
}	// DigiMixer

#else

/* Mix digital (8 bit unsigned mono) data to target buffer
   (with channel volumes separate) */
static void DigiMixerEx (Uint8 *src, Uint8 *dest, unsigned count, int LVol, int RVol) {
Sint16 *dest16 = (Sint16 *) dest;
LVol = 16 - LVol; RVol = 16 - RVol;
while (count --) {
  Sint32 value = ((*src ++ - 128) << 8);

  Sint32 lvalue = *dest16 + (value * LVol >> 5);
	*dest16 ++ = lvalue < MIN_S16 ? MIN_S16 :
							 lvalue > MAX_S16 ? MAX_S16 : lvalue;

  Sint32 rvalue = *dest16 + (value * RVol >> 5);
	*dest16 ++ = rvalue < MIN_S16 ? MIN_S16 :
							 rvalue > MAX_S16 ? MAX_S16 : rvalue;
	}
}	// DigiMixerEx

#endif

/* Convert FM (16 bit signed mono) data to target buffer */
static void ConvertFM (Uint16 *src, Uint8 *dest, unsigned count) {
Sint16 *dest16 = (Sint16 *) dest;
while (count --) {
	Sint16 value = *src++;
	*dest16 ++ = value;
	*dest16 ++ = value;
	}
}	// ConvertFM

/*
 *
 * Audio callback function
 *
 */
static void SD_Callback(void *userdata, Uint8 *stream, int len) {
Uint8 digi_data [AUDIO_SAMPLES];
Uint16 fm_data [AUDIO_SAMPLES];

unsigned count = len/4, samples;

memset (stream, 0, len);

samples = count - UpdateOPL (fm_data, count);
if (samples) ConvertFM (fm_data, stream, samples);

// (Sample count)
unsigned d_len = count * digi_freq / AUDIO_FREQ;

#ifdef SG_CHANNEL

samples = d_len - digi_feed (digi_data, d_len);

if (samples) {
unsigned r_len = samples * AUDIO_FREQ / digi_freq;
ResampleAudioBlock (digi_data, d_len, digi_data, r_len);
DigiMixer (digi_data, stream, r_len);
}

#else

{ unsigned i;

DBG_LstCh();

// (iterate from end to start)
for (i = acch_cnt; i;) {
	unsigned chan = acch_tab[-- i];
	samples = d_len - digi_feed_ch (chan, digi_data, d_len);

	if (samples) {
	unsigned r_len = samples * AUDIO_FREQ / digi_freq;
	ResampleAudioBlock (digi_data, d_len, digi_data, r_len);
	DigiMixerEx (digi_data, stream, r_len, digi_array[chan].LVol, digi_array[chan].RVol);
	}

	}		// for(i)

}

#endif
}	// SD_Callback

static boolean SD_Started;	// (is sound started?)

SDMode SoundMode;		// (FM sounds mode)
SDSMode DigiMode;		// (digitized sounds mode)
SMMode MusicMode;		// (music mode)

///////////////////////////////////////////////////////////////////////////
//
//	SD_SetDigiDevice() - Sets which sound hardware to use for digital sound
//
///////////////////////////////////////////////////////////////////////////
boolean SD_SetDigiDevice(SDSMode mode)
{
    if (DigiMode != mode) {
	digi_stop();
	DigiMode = mode;
	}

    return true;
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_SetSoundMode() - Sets which sound hardware to use for sound effects
//
///////////////////////////////////////////////////////////////////////////
boolean SD_SetSoundMode(SDMode mode)
{
    if (SoundMode != mode) {
	sfx_stop();
	SoundMode = mode;
	}

    return true;
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_SetMusicMode() - sets the device to use for background music
//
///////////////////////////////////////////////////////////////////////////
boolean SD_SetMusicMode(SMMode mode)
{
    if (MusicMode != mode) {
	mus_stop();
	MusicMode = mode;
	}

    return true;
}

// Sound init report
void SD_InitMessage(SDL_AudioSpec *format) {
char drvrname [80+1];
SDL_AudioDriverName (drvrname, 80);

printf ("SDL Audio init OK (driver \"%s\", %u-bit, %s, %u Hz, %u spl/buf)\n",
	drvrname,
	format->format == AUDIO_U8 || format->format == AUDIO_S8 ? 8 : 16,
	format->channels > 1 ? "stereo" : "mono",
	format->freq, format->samples);
}	// SD_InitMessage

///////////////////////////////////////////////////////////////////////////
//
//	SD_Startup() - starts up the Sound Mgr
//
///////////////////////////////////////////////////////////////////////////
void SD_Startup()
{
	if (SD_Started)
		return;

/* Open the audio device */
SDL_AudioSpec desired, obtained;

desired.freq = AUDIO_FREQ;

/* 16 bit signed */
desired.format = AUDIO_S16MSB;

/* Stereo */
desired.channels = 2;

/* Buffer size */
desired.samples = AUDIO_SAMPLES;

/* Our callback function */
desired.callback = SD_Callback;

/* Don't seem to be needed */
desired.userdata=NULL;

/* Open the audio device */
if (SDL_OpenAudio(&desired, &obtained) < 0)
	Quit("Couldn't open audio: %s\n", SDL_GetError());

SD_InitMessage(&obtained);

/* Create OPL synth */
OPL=OPLCreate(OPL_TYPE_YM3812, 3579545, AUDIO_FREQ);

OPLWrite(OPL, 0x01, 0x20);	/* Set WSE=1 */
OPLWrite(OPL, 0x08, 0x00);	/* Set CSM=0 & SEL=0 */

/* Prepare callback for playing */
digi_reset ();

SDL_PauseAudio (false);
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_Shutdown() - shuts down the Sound Mgr
//
///////////////////////////////////////////////////////////////////////////
void SD_Shutdown()
{
	if (!SD_Started)
		return;

	SD_Started = false;

SDL_CloseAudio ();
OPLDestroy(OPL);
}

void SD_PlayDigi (unsigned page, unsigned length) {
digi_play (page, length, -1, -1, -1);			// (Note: play always)
}	// SD_PlayDigi

extern unsigned sound_map [];

///////////////////////////////////////////////////////////////////////////
//
//	SD_SoundPlaying() - returns the sound number that's playing, or 0 if
//		no sound is playing
//
///////////////////////////////////////////////////////////////////////////
word SD_SoundPlaying()
{
	// (Note: checks only FM sound)
	return sfx_remain >= 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_StopSound() - if a sound is playing, stops it
//
///////////////////////////////////////////////////////////////////////////
void SD_StopSound()
{
	if (DigiMode == sds_On) digi_reset();
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_WaitSoundDone() - waits until the current sound is done playing
//
///////////////////////////////////////////////////////////////////////////
void SD_WaitSoundDone()
{
while (SD_SoundPlaying()) wait_Ticks(1);
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_MusicOn() - turns on the sequencer
//
///////////////////////////////////////////////////////////////////////////
void SD_MusicOn()
{
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_MusicOff() - turns off the sequencer and any playing notes
//
///////////////////////////////////////////////////////////////////////////
void SD_MusicOff()
{
}

///////////////////////////////////////////////////////////////////////////
//
//	SD_StartMusic() - starts playing the music pointed to
//
///////////////////////////////////////////////////////////////////////////
void SD_StartMusic(int song)
{
if (MusicMode) {
	// TODO: uncache previous song?
	CA_CacheMusicChunk(song);
	MusicGroup *music = (MusicGroup *) CA_GetMusicChunk(song);
	mus_play(music);
	}
}

/*
 *		Positioned sounds
 */

#define ATABLEMAX 15
static Uint8 lefttable[ATABLEMAX][ATABLEMAX * 2] = {
{ 8, 8, 8, 8, 8, 8, 8, 8, 5, 3, 1, 0, 0, 0, 0, 0, 6, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 6, 4, 2, 0, 0, 0, 0, 0, 4, 6, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 6, 4, 2, 1, 0, 0, 0, 1, 4, 6, 6, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 7, 5, 3, 2, 1, 0, 1, 2, 4, 5, 6, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 6, 5, 3, 3, 2, 2, 3, 4, 5, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 6, 5, 4, 4, 4, 4, 5, 6, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 6, 6, 5, 5, 5, 6, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 6, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8}
};

static Uint8 righttable[ATABLEMAX][ATABLEMAX * 2] = {
{ 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7, 6, 0, 0, 0, 0, 0, 1, 3, 5, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 6, 4, 0, 0, 0, 0, 0, 2, 4, 6, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 6, 6, 4, 1, 0, 0, 0, 1, 2, 4, 6, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 6, 5, 4, 2, 1, 0, 1, 2, 3, 5, 7, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 6, 5, 4, 3, 2, 2, 3, 3, 5, 6, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 6, 6, 5, 4, 4, 4, 4, 5, 6, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 6, 6, 5, 5, 5, 6, 6, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 7, 7, 7, 6, 6, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
{ 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8}
};

void SetVolume (digi_chan *chan) {
if (chan) {
if (chan->id >= 0) {

//
// translate point to view centered coordinates
//
	int gx = chan->loc_x - viewx;
	int gy = chan->loc_y - viewy;

	int x = (FixedByFrac(gx,viewcos) - FixedByFrac(gy,viewsin)) >> TILESHIFT;
	int y = (FixedByFrac(gy,viewcos) + FixedByFrac(gx,viewsin)) >> TILESHIFT;

	if (y >= ATABLEMAX)
		y = ATABLEMAX - 1;
	else if (y <= -ATABLEMAX)
		y = -ATABLEMAX;
	if (x < 0)
		x = -x;
	if (x >= ATABLEMAX)
		x = ATABLEMAX - 1;

	chan->LVol	=	lefttable[x][y + ATABLEMAX];
	chan->RVol	=	righttable[x][y + ATABLEMAX];
}		// (chan->id >= 0)

else
	chan->LVol = chan->RVol = 0;
}		// (chan)
}		// SetVolume

void PlaySoundLocGlobal(enum SND_nums chunk, int id, fixed gx, fixed gy)
{
if (DigiMode) {
	if (SND_BEG <= chunk && chunk <= SND_END) {
	int index = sound_map [chunk-SND_BEG];
	if (index != -1) {
		unsigned page, length;
		GetAudioInfo (index, &page, &length);
		digi_play (page, length, id, gx, gy);
		return;
		}
	// (otherwise try FM sound)
	}
	}

if (SoundMode) {
	// TODO: uncache previous sfx?
	AdLibSound *sound;

	CA_CacheAudioChunk(chunk);
	sound = (AdLibSound *)CA_GetAudioChunk (chunk);

	sfx_play(sound);
	}
}

void UpdateSoundLoc(fixed x, fixed y, int angle) {
unsigned i;
for (i = acch_cnt; i;) {
	unsigned chan = acch_tab[-- i];
	SetVolume (&digi_array[chan]);
	}
}	// UpdateSoundLoc

// Play sound on tile (tx, ty)
void PlaySoundLocTile(enum SND_nums s, fixed tx, fixed ty) {
PlaySoundLocGlobal(s, (int)((tx<<6)|(ty)),
	(tx << TILESHIFT) + (1 << (TILESHIFT - 1)),
	(ty << TILESHIFT) + (1 << (TILESHIFT - 1)));
}		// PlaySoundLocTile

// Play sound for actor 'ob'
void PlaySoundLocActor(enum SND_nums s, objtype *ob) {
PlaySoundLocGlobal(s, ob->id, ob->x, ob->y);
}		// PlaySoundLocActor

// Play sound (unpositioned)
boolean SD_PlaySound(enum SND_nums s) {
PlaySoundLocGlobal (s, -1, -1, -1);
return true;
}		// SD_PlaySound

