
/*
 * +==========================================================
 * |	'vi_sdl.c':
 * |	Video output/user input engine: SDL implementation
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"
#include "SDL.h"

static SDL_Surface *surface;

extern void keyboard_handler(int code, int press);
extern boolean InternalKeyboard[NumCodes];

// Start screen drawing
byte *VL_BeginDraw() {
if (SDL_MUSTLOCK(surface)) {
	SDL_LockSurface(surface);
	}

return surface->pixels;
}		// VL_BeginDraw

// End screen drawing
void VL_EndDraw() {
if (SDL_MUSTLOCK(surface)) {
	SDL_UnlockSurface(surface);
	}
}		// VL_EndDraw

// (Update entire screen)
void VL_UpdateScreen()
{
	SDL_Flip(surface);
}

// (Update screen area)
void VL_UpdateScreenArea (unsigned t, unsigned h, unsigned l, unsigned w) {
SDL_UpdateRect (surface, l, t, w, h);
}	// VL_UpdateScreenArea

//
// SDL flag names / masks
//

static unsigned VL_flagmasks[] = {
	SDL_FULLSCREEN,
	SDL_DOUBLEBUF,
	SDL_HWSURFACE,
	SDL_HWPALETTE,
	SDL_HWACCEL,
	0
	};

static char *VL_flagnames[] = {
	"Full screen",
	"Double buffer",
	"HW surface",
	"HW palette",
	"HW accel",
	0
	};

void ShowFlags(unsigned flags, unsigned flagmasks[], char *flagnames[]) {
unsigned mask;
while ((mask = *flagmasks++)) {
	if (flags & mask)
		printf ("%s%s", *flagnames++, (flags &= ~mask) ? ", " : "");
	else flagnames++;
	}
}	// ShowFlags

// Video init report
void VL_InitMessage(SDL_Surface *surface) {
char drvrname [80+1];
SDL_VideoDriverName (drvrname, 80);

printf ("SDL Video init OK (driver \"%s\", %u*%u, %u bpp, ",
	drvrname,
	surface->w, surface->h, surface->format->BitsPerPixel);
ShowFlags (surface->flags, VL_flagmasks, VL_flagnames);
printf (")\n");
}	// VL_InitMessage

/*
=======================
=
= VL_Startup
=
=======================
*/

// Set game caption
void VL_SetCaption (char *caption) {
SDL_WM_SetCaption (caption, caption);
}	// VL_SetCaption

void VL_Startup()
{
	vwidth = 320;
	vheight = 200;
	
	vwidth *= vscale;
	vheight *= vscale;

	/* TODO: renable parachute for release version? */		
	/* TODO: not always init audio? */
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE) < 0) {
		Quit("Couldn't init SDL");
	}

	{
	unsigned flags = SDL_SWSURFACE|SDL_HWPALETTE|SDL_DOUBLEBUF|SDL_HWACCEL;

	if (fullscreen) flags = SDL_FULLSCREEN;

	surface = SDL_SetVideoMode(vwidth, vheight, 8, flags);

	if (surface == NULL) {
		SDL_Quit();
		Quit("Couldn't set %ux%u mode", vwidth, vheight);
		}

	// Sound init report
	VL_InitMessage(surface);

	if (surface->flags & SDL_FULLSCREEN)
		SDL_ShowCursor(0);
	}
}

/*
=======================
=
= VL_Shutdown
=
=======================
*/

void VL_Shutdown()
{
	SDL_Quit();
}

/* ======================================================================== */

/*
=================
=
= VL_SetPalette
=
=================
*/

void VL_SetPalette(const byte *palette)
{
	SDL_Color colors[256];
	int i;

	for (i = 0; i != 256; i++)
	{	const byte *palentry = &palette[i*3];
		colors[i].r = palentry[0] << 2;
		colors[i].g = palentry[1] << 2;
		colors[i].b = palentry[2] << 2;
	}
	SDL_SetColors(surface, colors, 0, 256);
}

/*
=================
=
= VL_GetPalette
=
=================
*/

void VL_GetPalette(byte *palette)
{
	int i;
	const SDL_Color *colors = surface->format->palette->colors;
	for (i = 0; i != 256; i++)
	{	byte *palentry = &palette[i*3];
		palentry[0] = colors[i].r >> 2;
		palentry[1] = colors[i].g >> 2;
		palentry[2] = colors[i].b >> 2;
	}
}

static int XKeysymToScancode(unsigned int keysym)
{
	switch (keysym) {
		case SDLK_KP_ENTER:
		case SDLK_RETURN:
			return sc_Enter;
		case SDLK_ESCAPE:
			return sc_Escape;
		case SDLK_SPACE:
			return sc_Space;
		case SDLK_BACKSPACE:
			return sc_BackSpace;
		case SDLK_TAB:
			return sc_Tab;
		case SDLK_LALT:
			return sc_Alt;
		case SDLK_LCTRL:
			return sc_Control;
		case SDLK_CAPSLOCK:
			return sc_CapsLock;
		case SDLK_LSHIFT:
			return sc_LShift;
		case SDLK_RSHIFT:
			return sc_RShift;
		
		case SDLK_BACKQUOTE:
			return sc_Tilde;

		case SDLK_LEFTBRACKET:
			return sc_LtBr;
		case SDLK_RIGHTBRACKET:
			return sc_RtBr;
		case SDLK_LESS:
		case SDLK_COMMA:
			return sc_LT;
		case SDLK_GREATER:
		case SDLK_PERIOD:
			return sc_GT;
		case SDLK_QUESTION:
		case SDLK_SLASH:
			return sc_Ques;

		case SDLK_PLUS:
		case SDLK_KP_PLUS:
			return sc_Plus;

		case SDLK_MINUS:
		case SDLK_KP_MINUS:
			return sc_Minus;

		case SDLK_UP:
		case SDLK_KP8:
			return sc_UpArrow;
		case SDLK_DOWN:
		case SDLK_KP2:
			return sc_DownArrow;
		case SDLK_LEFT:
		case SDLK_KP4:
			return sc_LeftArrow;
		case SDLK_RIGHT:
		case SDLK_KP6:
			return sc_RightArrow;
		case SDLK_HOME:
		case SDLK_KP7:
			return sc_Home;
		case SDLK_END:
		case SDLK_KP1:
			return sc_End;
		case SDLK_PAGEUP:
		case SDLK_KP9:
			return sc_PgUp;
		case SDLK_PAGEDOWN:
		case SDLK_KP3:
			return sc_PgDn;
			
		case SDLK_INSERT:
		case SDLK_KP0:
			return sc_Insert;
		case SDLK_DELETE:
		case SDLK_KP_PERIOD:
			return sc_Delete;

		case SDLK_F1:
			return sc_F1;
		case SDLK_F2:
			return sc_F2;
		case SDLK_F3:
			return sc_F3;
		case SDLK_F4:
			return sc_F4;
		case SDLK_F5:
			return sc_F5;
		case SDLK_F6:
			return sc_F6;
		case SDLK_F7:
			return sc_F7;
		case SDLK_F8:
			return sc_F8;
		case SDLK_F9:
			return sc_F9;
		case SDLK_F10:
			return sc_F10;
		case SDLK_F11:
			return sc_F11;
		case SDLK_F12:
			return sc_F12;	
		
		case SDLK_1:
			return sc_1;
		case SDLK_2:
			return sc_2;
		case SDLK_3:
			return sc_3;
		case SDLK_4:
			return sc_4;
		case SDLK_5:
			return sc_5;
		case SDLK_6:
			return sc_6;
		case SDLK_7:
			return sc_7;
		case SDLK_8:
			return sc_8;
		case SDLK_9:
			return sc_9;
		case SDLK_0:
			return sc_0;
		
		case SDLK_a:
			return sc_A;
		case SDLK_b:
			return sc_B;
		case SDLK_c:
			return sc_C;
		case SDLK_d:
			return sc_D;
		case SDLK_e:
			return sc_E;
		case SDLK_f:
			return sc_F;
		case SDLK_g:
			return sc_G;
		case SDLK_h:
			return sc_H;
		case SDLK_i:
			return sc_I;
		case SDLK_j:
			return sc_J;
		case SDLK_k:
			return sc_K;
		case SDLK_l:
			return sc_L;
		case SDLK_m:
			return sc_M;
		case SDLK_n:
			return sc_N;
		case SDLK_o:
			return sc_O;
		case SDLK_p:
			return sc_P;
		case SDLK_q:
			return sc_Q;
		case SDLK_r:
			return sc_R;
		case SDLK_s:
			return sc_S;
		case SDLK_t:
			return sc_T;
		case SDLK_u:
			return sc_U;
		case SDLK_v:
			return sc_V;
		case SDLK_w:
			return sc_W;
		case SDLK_x:
			return sc_X;
		case SDLK_y:
			return sc_Y;
		case SDLK_z:
			return sc_Z;
		
		case SDLK_PAUSE:
			return 0xE1;

		default:
			return sc_None;
	}
}

static void VL_SyncCursor (SDL_GrabMode mode) {
	SDL_ShowCursor (mode == SDL_GRAB_OFF &&
		!(surface->flags & SDL_FULLSCREEN) ? SDL_ENABLE : SDL_DISABLE);
}		// VL_SyncCursor

// Fullscreen mode toggle
// (returns set fullscreen state)
boolean VL_ToggleFullScreen() {
	SDL_WM_ToggleFullScreen(surface);
	VL_SyncCursor(SDL_WM_GrabInput(SDL_GRAB_QUERY));
	return surface->flags & SDL_FULLSCREEN;
}		// VL_ToggleFullScreen

// Mouse grab toggle
// (returns set mouse grab state)
boolean VL_ToggleMouseGrab() {
	boolean state = (SDL_WM_GrabInput(SDL_GRAB_QUERY) == SDL_GRAB_ON) ?
			SDL_GRAB_OFF : SDL_GRAB_ON;
	state = SDL_WM_GrabInput(state);
	VL_SyncCursor(state);
	return state;
}		// VL_ToggleMouseGrab

void INL_Update()
{
	SDL_Event event;
	boolean DebouncedKeyboard[NumCodes];
	
	memcpy(DebouncedKeyboard, InternalKeyboard, sizeof(DebouncedKeyboard));
		
	if (SDL_PollEvent(&event)) {
		do {
			switch(event.type) {
				case SDL_KEYDOWN:
					keyboard_handler(XKeysymToScancode(event.key.keysym.sym), 1);
					break;
				case SDL_KEYUP:
					keyboard_handler(XKeysymToScancode(event.key.keysym.sym), 0);
					break;
				default:
					break;
			}
		} while (SDL_PollEvent(&event));
	}

	/* ctrl-z for iconify window? */
}

void IN_GetMouseDelta(int *dx, int *dy)
{
	int x, y;
	
	SDL_GetRelativeMouseState(&x, &y);
	
	if (dx)
		*dx = x;
	if (dy)
		*dy = y;
}

byte IN_MouseButtons()
{
	return SDL_GetMouseState(NULL, NULL);
}

//
// TODO:
// implement joystick interface...
//

///////////////////////////////////////////////////////////////////////////
//
//	IN_SetupJoy() - Sets up thresholding values and calls INL_SetJoyScale()
//		to set up scaling values
//
///////////////////////////////////////////////////////////////////////////
void IN_SetupJoy(word joy,word minx,word maxx,word miny,word maxy)
{
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_StartJoy() - Detects & auto-configures the specified joystick
//					The auto-config assumes the joystick is centered
//
///////////////////////////////////////////////////////////////////////////
boolean INL_StartJoy(word joy)
{
	return false;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_ShutJoy() - Cleans up the joystick stuff
//
///////////////////////////////////////////////////////////////////////////
void INL_ShutJoy(word joy)
{
	JoysPresent[joy] = false;
}


/*
===================
=
= IN_JoyButtons
=
===================
*/

byte IN_JoyButtons()
{
	return 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	IN_GetJoyAbs() - Reads the absolute position of the specified joystick
//
///////////////////////////////////////////////////////////////////////////
void IN_GetJoyAbs(word joy,word *xp,word *yp)
{
	*xp = 0;
	*yp = 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_GetJoyDelta() - Returns the relative movement of the specified
//		joystick (from +/-127)
//
///////////////////////////////////////////////////////////////////////////
void INL_GetJoyDelta(word joy,int *dx,int *dy)
{
	*dx = 0;
	*dy = 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	INL_GetJoyButtons() - Returns the button status of the specified
//		joystick
//
///////////////////////////////////////////////////////////////////////////
word INL_GetJoyButtons(word joy)
{
	return 0;
}

//
// -
// -- Game timing --
// -
//

// Time base as set by 'set_TimeCount'
static unsigned long timebase;
static clock_t clockbase;

// Set timecount to 'time'
void set_TimeCount(unsigned long time) {
clockbase = SDL_GetTicks();
timebase = time;
}	// set_TimeCount

// Get timecount
unsigned long get_TimeCount() {
return ((unsigned long) (SDL_GetTicks() - clockbase) * TickBase / 1000) + timebase;
}	// get_TimeCount

// Wait for 'ticks'
void wait_Ticks(int ticks) {
SDL_Delay((ticks * 1000) / TickBase);
}	// wait_Ticks


