
/*
 * +==========================================================
 * |	'wl_act1.c':
 * |	Game engine: statics, doors and pushwalls.
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"

/*
=============================================================================

							STATICS

=============================================================================
*/

statobj_t statobjlist[MAXSTATS], *laststatobj;

struct
{
	int	picnum;
	stat_t	type;
} static statinfo[] =
{
{SPR_STAT_0},				// puddle          spr1v
{SPR_STAT_1,block},			// Green Barrel    "
{SPR_STAT_2,block},			// Table/chairs    "
{SPR_STAT_3,block},			// Floor lamp      "
{SPR_STAT_4},				// Chandelier      "
{SPR_STAT_5,block},			// Hanged man      "
{SPR_STAT_6,bo_alpo},			// Bad food        "
{SPR_STAT_7,block},			// Red pillar      "
{SPR_STAT_8,block},			// Tree            spr2v
{SPR_STAT_9},				// Skeleton flat   "
{SPR_STAT_10,block},			// WOLF: Sink      "
					// SPEAR: gibs
{SPR_STAT_11,block},			// Potted plant    "
{SPR_STAT_12,block},			// Urn             "
{SPR_STAT_13,block},			// Bare table      "
{SPR_STAT_14},				// Ceiling light   "
{SPR_STAT_15},				// WOLF: Kitchen stuff   "
					// SPEAR: Gibs
{SPR_STAT_16,block},			// suit of armor   spr3v
{SPR_STAT_17,block},			// Hanging cage    "
{SPR_STAT_18,block},			// SkeletoninCage  "
{SPR_STAT_19},				// Skeleton relax  "
{SPR_STAT_20,bo_key1},			// Key 1           "
{SPR_STAT_21,bo_key2},			// Key 2           "
{SPR_STAT_22,block},			// stuff				(SOD:gibs)
{SPR_STAT_23},				// stuff
{SPR_STAT_24,bo_food}, 			// Good food       spr4v
{SPR_STAT_25,bo_firstaid},		// First aid       "
{SPR_STAT_26,bo_clip},			// Clip            "
{SPR_STAT_27,bo_machinegun},		// Machine gun     "
{SPR_STAT_28,bo_chaingun},		// Gatling gun     "
{SPR_STAT_29,bo_cross},			// Cross           "
{SPR_STAT_30,bo_chalice},		// Chalice         "
{SPR_STAT_31,bo_bible},			// Bible           "
{SPR_STAT_32,bo_crown},			// crown           spr5v
{SPR_STAT_33,bo_fullheal},		// one up          "
{SPR_STAT_34,bo_gibs},			// gibs            "
{SPR_STAT_35,block},			// barrel          "
{SPR_STAT_36,block},			// well            "
{SPR_STAT_37,block},			// Empty well      "
{SPR_STAT_38,bo_gibs},			// Gibs 2          "
{SPR_STAT_39,block},			// flag		   "
{SPR_STAT_40,block},			// WOLF: Call Apogee	   spr7v 
					// SPEAR: Red light
{SPR_STAT_41},				// junk            "
{SPR_STAT_42},				// junk 	   "
{SPR_STAT_43},				// junk            "
{SPR_STAT_44},				// WOLF: pots      "
					// SPEAR: Gibs!
{SPR_STAT_45,block},			// stove           " (SOD:gibs)
{SPR_STAT_46,block},			// spears          " (SOD:gibs)
{SPR_STAT_47},				// vines	   "
{SPR_STAT_48,block},			// marble pillar
{SPR_STAT_49,bo_25clip},		// bonus 25 clip
{SPR_STAT_50,block},			// truck
{SPR_STAT_51,bo_spear},			// SPEAR OF DESTINY!

{SPR_STAT_26,bo_clip2},			// Clip            "

{-1}					// terminator
};

// Fix statics according to game type
void SetupStatics() {
int i;

for (i = 0; ; ++ i)
    switch (statinfo[i].picnum) {
	case -1:	return;

	case SPR_STAT_15:
	    statinfo[i].type = GAME_WOLF ? 0 : block;
	    break;

	case SPR_STAT_40:
	    statinfo[i].type = GAME_WOLF ? block : 0;
	    break;

	case SPR_STAT_44:
	    statinfo[i].type = GAME_WOLF ? 0 : block;
	    break;
	}
}	// SetupStatics

/*
===============
=
= InitStaticList
=
===============
*/

void InitStaticList()
{
	laststatobj = &statobjlist[0];
}

/*
===============
=
= SpawnStatic
=
===============
*/

void SpawnStatic(int tilex, int tiley, int type)
{
	laststatobj->shapenum = statinfo[type].picnum;
	laststatobj->visspot = &spotvis
		[laststatobj->tilex = tilex][laststatobj->tiley = tiley];

	switch (statinfo[type].type)
	{
	case block:
		actorat[tilex][tiley] = 1;	// consider it a blocking tile
	case dressing:
		laststatobj->flags = 0;
		break;

	case	bo_cross:
	case	bo_chalice:
	case	bo_bible:
	case	bo_crown:
	case	bo_fullheal:
		if (!loadedgame)
			gamestate.treasuretotal++;

	case	bo_firstaid:
	case	bo_key1:
	case	bo_key2:
	case	bo_key3:
	case	bo_key4:
	case	bo_clip:
	case	bo_25clip:
	case	bo_machinegun:
	case	bo_chaingun:
	case	bo_food:
	case	bo_alpo:
	case	bo_gibs:
	case	bo_spear:
		laststatobj->flags = FL_BONUS;
		laststatobj->itemnumber = statinfo[type].type;
		break;
	default: 
		break;
	}

	laststatobj++;

	if (laststatobj == &statobjlist[MAXSTATS])
		Quit ("Too many static objects!\n");
}

/*
===============
=
= PlaceItemType
=
= Called during game play to drop actors' items.  It finds the proper
= item number based on the item type (bo_???).  If there are no free item
= spots, nothing is done.
=
===============
*/

void PlaceItemType(int itemtype, int tilex, int tiley)
{
	int type;
	statobj_t *spot;

//
// find the item number
//
	for (type = 0; ; type++)
	{
		if (statinfo[type].picnum == -1) /* end of list */
			Quit("PlaceItemType: couldn't find type!");
		if (statinfo[type].type == itemtype)
			break;
	}

//
// find a spot in statobjlist to put it in
//
	for (spot=&statobjlist[0] ; ; spot++)
	{
		if (spot==laststatobj)
		{
			if (spot == &statobjlist[MAXSTATS])
				return;	/* no free spots */
			laststatobj++;	/* space at end */
			break;
		}

		if (spot->shapenum == -1) /* -1 is a free spot */
			break;
	}
//
// place it
//
	spot->shapenum = statinfo[type].picnum;
	spot->tilex = tilex;
	spot->tiley = tiley;
	spot->visspot = &spotvis[tilex][tiley];
	spot->flags = FL_BONUS;
	spot->itemnumber = statinfo[type].type;

	map_gfx [tilex][tiley] = -spot->shapenum - 1;
}

/*
=============================================================================

							DOORS

doorobjlist[] holds most of the information for the doors

doorposition[] holds the amount the door is open, ranging from 0 to 0xffff
	this is directly accessed by AsmRefresh during rendering

The number of doors is limited to 64 because a spot in tilemap holds the
	door number in the low 6 bits, with the high bit meaning a door center
	and bit 6 meaning a door side tile

Open doors conect two areas, so sounds will travel between them and sight
	will be checked when the player is in a connected area.

Areaconnect is incremented/decremented by each door. If >0 they connect

Every time a door opens or closes the areabyplayer matrix gets recalculated.
	An area is true if it connects with the player's current spot.

=============================================================================
*/

#define OPENTICS	300

doorobj_t	doorobjlist[MAXDOORS],*lastdoorobj;
int			doornum;

uint16_t	doorposition[MAXDOORS];	// leading edge of door 0=closed
					// 0xffff = fully open
byte		areaconnect[NUMAREAS][NUMAREAS];

boolean		areabyplayer[NUMAREAS];


/*
==============
=
= ConnectAreas
=
= Scans outward from playerarea, marking all connected areas
=
==============
*/

void RecursiveConnect(int areanumber)
{
	int i;

	for (i = 0; i < NUMAREAS; i++)
	{
		if (areaconnect[areanumber][i] && !areabyplayer[i])
		{
			areabyplayer[i] = true;
			RecursiveConnect(i);
		}
	}
}

void ConnectAreas()
{
	memset(areabyplayer, 0, sizeof(areabyplayer));
	areabyplayer[player->areanumber] = true;
	RecursiveConnect(player->areanumber);
}

void InitAreas()
{
	memset(areabyplayer, 0, sizeof(areabyplayer));
	areabyplayer[player->areanumber] = true;
}

/*
===============
=
= InitDoorList
=
===============
*/

void InitDoorList()
{
	memset(areabyplayer, 0, sizeof(areabyplayer));
	memset(areaconnect, 0, sizeof(areaconnect));

	lastdoorobj = &doorobjlist[0];
	doornum = 0;
}

/*
===============
=
= SpawnDoor
=
===============
*/

void SpawnDoor(int tilex, int tiley, boolean vertical, int lock)
{
	word *map;

	if (doornum == MAXDOORS)
		Quit ("%u+ doors on level!", MAXDOORS);

	doorposition[doornum] = 0;		// doors start out fully closed
	lastdoorobj->tilex = tilex;
	lastdoorobj->tiley = tiley;
	lastdoorobj->vertical = vertical;
	lastdoorobj->lock = lock;
	lastdoorobj->action = dr_closed;

	actorat[tilex][tiley] = doornum | AM_DOOR;	// consider it a solid wall

//
// make the door tile a special tile, and mark the adjacent tiles
// for door sides
//
	tilemap[tilex][tiley] = doornum | AM_DOOR;
	map = (word *)(mapsegs[0] + farmapylookup[tiley]+tilex);
	if (vertical)
	{
		*map = map[-1];                 // set area number
		tilemap[tilex][tiley-1] |= AM_DSID;
		tilemap[tilex][tiley+1] |= AM_DSID;
	}
	else
	{
		*map = map[-MAPWDT];		// set area number
		tilemap[tilex-1][tiley] |= AM_DSID;
		tilemap[tilex+1][tiley] |= AM_DSID;
	}

	doornum++;
	lastdoorobj++;
}

/*
=====================
=
= OpenDoor
=
=====================
*/

void OpenDoor(int door)
{
	if (doorobjlist[door].action == dr_open)
		doorobjlist[door].ticcount = 0;		// reset open time
	else
		doorobjlist[door].action = dr_opening;	// start it opening
}

/*
=====================
=
= CloseDoor
=
=====================
*/

void CloseDoor(int door)
{
	int tilex, tiley, area;
	int actor;
	objtype *check;

//
// don't close on anything solid
//
	tilex = doorobjlist[door].tilex;
	tiley = doorobjlist[door].tiley;

	if (actorat[tilex][tiley])
		return;

	if (player->tilex == tilex && player->tiley == tiley)
		return;

	if (doorobjlist[door].vertical)
	{
		if (player->tiley == tiley)
		{
			if (((player->x+MINDIST) >>TILESHIFT) == tilex)
				return;
			if (((player->x-MINDIST) >>TILESHIFT) == tilex)
				return;
		}

		if ((actor = actorat[tilex-1][tiley]) & AM_OBJID)
			check = &objlist[actor & ~AM_OBJID];
		else
			check = NULL;

		if (check && ((check->x+MINDIST) >> TILESHIFT) == tilex)
			return;
		
		if ((actor = actorat[tilex+1][tiley]) & AM_OBJID)
			check = &objlist[actor & ~AM_OBJID];
		else
			check = NULL;

		if (check && ((check->x-MINDIST) >> TILESHIFT) == tilex)
			return;
	}
	else if (!doorobjlist[door].vertical)
	{
		if (player->tilex == tilex)
		{
			if ( ((player->y+MINDIST) >>TILESHIFT) == tiley )
				return;
			if ( ((player->y-MINDIST) >>TILESHIFT) == tiley )
				return;
		}
		
		if ((actor = actorat[tilex][tiley-1]) & AM_OBJID)
			check = &objlist[actor & ~AM_OBJID];
		else
			check = NULL;

		if (check && ((check->y+MINDIST) >> TILESHIFT) == tiley )
			return;
		
		if ((actor = actorat[tilex][tiley+1]) & AM_OBJID)
			check = &objlist[actor & ~AM_OBJID];
		else
			check = NULL;

		if (check && ((check->y-MINDIST) >> TILESHIFT) == tiley )
			return;
	}

//
// play door sound if in a connected area
//
	area = *(mapsegs[0] + farmapylookup[doorobjlist[door].tiley]
			+doorobjlist[door].tilex)-AREATILE;
	if (areabyplayer[area])
	{
		PlaySoundLocTile(CLOSEDOORSND,doorobjlist[door].tilex,doorobjlist[door].tiley);	// JAB
	}

	doorobjlist[door].action = dr_closing;
//
// make the door space solid
//
	actorat[tilex][tiley] = door | AM_DOOR;
}

/*
=====================
=
= OperateDoor
=
= The player wants to change the door's direction
=
=====================
*/

void OperateDoor(int door)
{
	int	lock;

	lock = doorobjlist[door].lock;
	if (lock >= dr_lock1 && lock <= dr_lock4)
	{	unsigned key = 1 << (lock-dr_lock1);
		if (! (gamestate.keys & key))
		{
			FlashKey (key);
			SD_PlaySound (NOWAYSND);	// locked
			return;
		}
	}

	switch (doorobjlist[door].action)
	{
	case dr_closed:
	case dr_closing:
		OpenDoor (door);
		break;
	case dr_open:
	case dr_opening:
		CloseDoor (door);
		break;
	}
}

/*
===============
=
= DoorOpen
=
= Close the door after three seconds
=
===============
*/

void DoorOpen (int door)
{
	if ( (doorobjlist[door].ticcount += tics) >= OPENTICS)
		CloseDoor (door);
}



/*
===============
=
= DoorOpening
=
===============
*/

void DoorOpening(int door)
{
	int		area1,area2;
	word *map;
	long	position;
	int tilex, tiley;

	tilex = doorobjlist[door].tilex;
	tiley = doorobjlist[door].tiley;

	position = doorposition[door];
	if (!position)
	{
	//
	// door is just starting to open, so connect the areas
	//
		map = (word *)(mapsegs[0] +
			farmapylookup[doorobjlist[door].tiley] +
			doorobjlist[door].tilex);

		if (doorobjlist[door].vertical)
		{
			area1 =	map[ 1];
			area2 =	map[-1];
		}
		else
		{
			area1 =	map[-MAPWDT];
			area2 =	map[ MAPWDT];
		}
		area1 -= AREATILE;
		area2 -= AREATILE;
		areaconnect[area1][area2]++;
		areaconnect[area2][area1]++;
		ConnectAreas ();
		if (areabyplayer[area1])
		{
			PlaySoundLocTile(OPENDOORSND,
			doorobjlist[door].tilex,doorobjlist[door].tiley);	// JAB
		}
	}

//
// slide the door by an adaptive amount
//
	position += tics<<10;
	if (position >= 0xffff)
	{	//
		// door is all the way open
		//
		position = 0xffff;
		doorobjlist[door].ticcount = 0;
		doorobjlist[door].action = dr_open;
		actorat[tilex][tiley] = 0;

	}

	map_off[tilex][tiley] = position >> (TILESHIFT-6);
	doorposition[door] = position;
}


/*
===============
=
= DoorClosing
=
===============
*/

void DoorClosing(int door)
{
	int		area1,area2;
	word *map;
	long	position;
	int		tilex,tiley;

	tilex = doorobjlist[door].tilex;
	tiley = doorobjlist[door].tiley;

	if ((actorat[tilex][tiley] != (door | AM_DOOR))
	|| (player->tilex == tilex && player->tiley == tiley) )
	{			// something got inside the door
		OpenDoor(door);
		return;
	};

	position = doorposition[door];

//
// slide the door by an adaptive amount
//
	position -= tics<<10;
	if (position <= 0)
	{
	//
	// door is closed all the way, so disconnect the areas
	//
		position = 0;

		doorobjlist[door].action = dr_closed;

		map = (word *)(mapsegs[0] + farmapylookup[doorobjlist[door].tiley]
			+doorobjlist[door].tilex);

		if (doorobjlist[door].vertical)
		{
			area1 =	map[1];
			area2 =	map[-1];
		}
		else
		{
			area1 =	map[-MAPWDT];
			area2 =	map[MAPWDT];
		}
		area1 -= AREATILE;
		area2 -= AREATILE;
		areaconnect[area1][area2]--;
		areaconnect[area2][area1]--;

		ConnectAreas ();
	}

	map_off[tilex][tiley] = position >> (TILESHIFT-6);
	doorposition[door] = position;
}

/*
=====================
=
= MoveDoors
=
= Called from PlayLoop
=
=====================
*/

void MoveDoors()
{
	int door;

	if (gamestate.victoryflag)	// don't move door during victory sequence
		return;

	for (door = 0; door != doornum; door++)
		switch(doorobjlist[door].action)
		{
		case dr_open:
			DoorOpen (door);
			break;

		case dr_opening:
			DoorOpening(door);
			break;

		case dr_closing:
			DoorClosing(door);
			break;

		default:
			break;
		}
}


/*
=============================================================================

						PUSHABLE WALLS

=============================================================================
*/

unsigned pwallstate;
unsigned pwallpos;	// amount a pushable wall has been moved (0-63)
unsigned pwallx, pwally;
int pwalldir;

/*
===============
=
= PushWall
=
===============
*/

void PushWall(int checkx, int checky, int dir)
{
	int oldtile;

	if (pwallstate)
		return;

	pwallx = checkx;
	pwally = checky;

	if (! (oldtile = tilemap[checkx][checky]))
		return;

	switch (dir)
	{
	case di_north:
		checky --;
		break;

	case di_east:
		checkx ++;
		break;

	case di_south:
		checky ++;
		break;

	case di_west:
		checkx --;
		break;
	}

	if (actorat[checkx][checky])
		{
		SD_PlaySound(NOWAYSND);
		return;
		}

	actorat[checkx][checky] = tilemap[checkx][checky] = oldtile;
	gamestate.secretcount++;
	pwalldir = dir;
	pwallstate = 1;
	pwallpos = 0;
	tilemap[pwallx][pwally] |= AM_PUSH;
	*(mapsegs[1]+farmapylookup[pwally]+pwallx) = 0;	// remove P tile info

	map_gfx[checkx][checky] = map_gfx[pwallx][pwally];
	map_dir[checkx][checky] = MP_INV | (map_dir[pwallx][pwally] = ROT_CW(pwalldir));
	map_off[checkx][checky] = map_off[pwallx][pwally] = 0;

	SD_PlaySound(PUSHWALLSND);
}

/*
=================
=
= MovePWalls
=
=================
*/

void MovePWalls()
{
	int oldblock, oldtile;
	int checkx = pwallx, checky = pwally;

	if (!pwallstate)
		return;

	oldblock = pwallstate/128;

	pwallstate += tics;

	pwallpos = (pwallstate/2)&63;

	if (pwallstate/128 != oldblock)
	{
		// block crossed into a new block
		oldtile = tilemap[pwallx][pwally] & 63;

		//
		// the old tile can now be walked into
		//
		tilemap[pwallx][pwally] = actorat[pwallx][pwally] = 0;
		*(mapsegs[0]+farmapylookup[pwally]+pwallx) =
			player->areanumber+AREATILE;

		map_gfx[pwallx][pwally] = 0;

		//
		// check new tile
		//
		switch (pwalldir)
			    {
			    case di_north:
				checky = --pwally - 1;
				break;

			    case di_east:
				checkx = ++pwallx + 1;
				break;

			    case di_south:
				checky = ++pwally + 1;
				break;

			    case di_west:
				checkx = --pwallx - 1;
				break;
			    }

		//
		// see if it should be pushed farther
		// (not farther than 2 tiles)
		//
		if (pwallstate>=256 || (actorat[checkx][checky]))
		{
			pwallstate = pwallpos = 0;
			map_off[pwallx][pwally] = 0;
			map_dir[pwallx][pwally] &= ~MP_INV;
		}
		else
		{
			actorat[checkx][checky] =
			tilemap[checkx][checky] = oldtile;
			tilemap[pwallx][pwally] = oldtile | AM_PUSH;

			map_gfx[checkx][checky] = map_gfx[pwallx][pwally];
			map_dir[checkx][checky] = map_dir[pwallx][pwally];
			map_dir[pwallx][pwally] &= ~MP_INV;
			map_off[checkx][checky] = map_off[pwallx][pwally] = 0;
		}
	}

	else {
	    switch (pwalldir)
		{
		case di_north:
			checky = pwally - 1;
			break;

		case di_east:
			checkx = pwallx + 1;
			break;

		case di_south:
			checky = pwally + 1;
			break;

		case di_west:
			checkx = pwallx - 1;
			break;
		}

	map_off[pwallx][pwally] = map_off[checkx][checky] = pwallpos;
	}

}

