
/*
 * +==========================================================
 * |	'wl_agent.c':
 * |	Game engine: player control & actions.
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"

/*
=============================================================================

						 LOCAL CONSTANTS

=============================================================================
*/

#define MOVESCALE		150
#define BACKMOVESCALE		100
#define ANGLESCALE		20

/*
=============================================================================

						 GLOBAL VARIABLES

=============================================================================
*/

//
// player state info
//
long		thrustspeed;

unsigned	plux, pluy;	// player coordinates scaled to unsigned

int		anglefrac;

objtype		*LastAttacker;

/*
=============================================================================

						 LOCAL VARIABLES

=============================================================================
*/

struct atkinf
{
	char tics, attack, frame; 	// attack is 1 for gun, 2 for knife
} static attackinfo[4][14] =
{
{ {6,0,1},{6,2,2},{6,0,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,0,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,3,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,4,3},{6,-1,4} },
};

/*
=============================================================================

						CONTROL STUFF

=============================================================================
*/

/*
======================
=
= CheckWeaponChange
=
= Keys 1-4 change weapons
=
======================
*/

void CheckWeaponChange()
{
	int i;

	if (!gamestate.ammo)		// must use knife with no ammo
		return;

	for (i = wp_knife; i <= wp_chaingun; i++)
		if ((gamestate.weaponflag & (1 << i)) &&
		     buttonstate[bt_readyknife+i-wp_knife])
		{
			gamestate.weapon = gamestate.chosenweapon = i;
			DrawWeapon();
			return;
		}

}

void CycleWeapon(boolean forward) {
int i = gamestate.weapon;

for (;;) {
    if (forward) {
	if (i++ == wp_chaingun) i = wp_knife;
	}
    else {
	if (i-- == wp_knife) i = wp_chaingun;
	}

    if (gamestate.weaponflag & (1 << i))
	break;
    }	// (for)

gamestate.weapon = gamestate.chosenweapon = i;
DrawWeapon();
}	// CycleWeapon

/*
=======================
=
= ControlMovement
=
= Takes controlx,controly, and buttonstate[bt_strafe]
=
= Changes the player's angle and position
=
= There is an angle hack because when going 70 fps, the roundoff becomes
= significant
=
=======================
*/

void ControlMovement(objtype *ob)
{
	int		angle;
	int		angleunits;

	thrustspeed = 0;

//
// first, check for map scroll mode
//
	if (map_mode && map_scroll)
	{
		MapViewScroll(true, controlx, controly);
		return;
	}

//
// side to side move
//
	if (buttonstate[bt_strafe] || strafeoff)
	{
	//
	// strafing
	//
		controlx += strafeoff;

		if (controlx > 0)
		{
			angle = ob->angle - ANGLES/4;
			if (angle < 0)
				angle += ANGLES;
			Thrust (angle, controlx*MOVESCALE);	// move to left
		}
		else if (controlx < 0)
		{
			angle = ob->angle + ANGLES/4;
			if (angle >= ANGLES)
				angle -= ANGLES;
			Thrust (angle,-controlx*MOVESCALE);	// move to right
		}
	} else {
	//
	// not strafing
	//
		anglefrac += controlx;
		angleunits = anglefrac/ANGLESCALE;
		anglefrac -= angleunits*ANGLESCALE;
		ob->angle -= angleunits;

		if (ob->angle >= ANGLES)
			ob->angle -= ANGLES;
		if (ob->angle < 0)
			ob->angle += ANGLES;

	}

//
// forward/backwards move
//
	if (controly < 0)
	{
		Thrust (ob->angle,-controly*MOVESCALE);		// move forwards
	}
	else if (controly > 0)
	{
		angle = ob->angle + ANGLES/2;
		if (angle >= ANGLES)
			angle -= ANGLES;
		Thrust (angle,controly*BACKMOVESCALE);		// move backwards
	}

	if (gamestate.victoryflag)		// watching the BJ actor
		return;
}

/*
===============
=
= UpdateFace
=
= Calls draw face if time to change
=
===============
*/

int facecount;

void UpdateFace()
{
	if (SD_SoundPlaying() == GETGATLINGSND)
		return;

	facecount += tics;
	if (facecount > US_RndT())
	{
		gamestate.faceframe = (US_RndT()>>6);
		if (gamestate.faceframe==3)
			gamestate.faceframe = 1;

		facecount = 0;
		DrawFace();
	}
}

/*
===============
=
= TakeDamage
=
===============
*/

void TakeDamage(int points, objtype *attacker)
{
	LastAttacker = attacker;

	if (gamestate.victoryflag)
		return;

	if (gamestate.difficulty==gd_baby)
	    points>>=2;

	if (!godmode)
		gamestate.health -= points;

	if (gamestate.health<=0)
	{
		gamestate.health = 0;
		playstate = ex_died;
		killerobj = attacker;
	}

	StartDamageFlash(points);

	//
	// MAKE BJ'S EYES BUG IF MAJOR DAMAGE!
	//
	if (GAME_SPEAR && points > 30 && gamestate.health!=0 && !godmode)
	{
		facecount = 0;
		face_ouch = true;
	}

	DrawHealth();
	DrawFace();
}

/*
===============
=
= HealSelf
=
===============
*/

boolean HealSelf(int points)	// (returns true, if actually healed)
{
	if (gamestate.health == MAX_HEALTH) return false;

	if ((gamestate.health += points) > MAX_HEALTH)
		gamestate.health = MAX_HEALTH;

	DrawHealth();
	DrawFace();
	return true;
}

/*
===============
=
= GiveExtraMan
=
===============
*/

boolean GiveExtraMan()		// (returns true, if life obtained)
{
	if (gamestate.lives < MAX_LIVES) {
		gamestate.lives++;

		DrawLives();
		SD_PlaySound(BONUS1UPSND);
		return true;
		}

return false;
}

//===========================================================================

/*
===============
=
= GivePoints
=
===============
*/

void GivePoints(long points)
{
	gamestate.score += points;
	while (gamestate.score >= gamestate.nextextra)
	{
		gamestate.nextextra += EXTRAPOINTS;
		GiveExtraMan();
	}

	DrawScore();
}

//===========================================================================

/*
===============
=
= GiveAmmo
=
===============
*/

boolean GiveAmmo(int ammo)	// (returns true, if ammo was needed)
{
	if (!gamestate.ammo)	// knife was out
	{
		if (!gamestate.attackframe)
		{
			gamestate.weapon = gamestate.chosenweapon;
			DrawWeapon();
		}
	}

	if (gamestate.ammo == MAX_AMMO) return false;

	if ((gamestate.ammo += ammo) > MAX_AMMO)
		gamestate.ammo = MAX_AMMO;

	DrawAmmo();
	return true;
}

//===========================================================================

/*
==================
=
= GiveWeapon
=
==================
*/

boolean GiveWeapon(int weapon)		// (returns true, if weapon needed)
{
	boolean ammoflag = GiveAmmo(6);

	if (gamestate.weaponflag & (1 << weapon)) return ammoflag;
	gamestate.weaponflag |= (1 << weapon);

	if (gamestate.chosenweapon < weapon) {
	    gamestate.weapon = gamestate.chosenweapon = weapon;

	    DrawWeapon();
	    }

	return ammoflag;
}

//===========================================================================

/*
==================
=
= GiveKey
=
==================
*/

void GiveKey(int key)
{
	gamestate.keys |= (1<<key);
	DrawKeys();
}

/*
 * Flasing keys
 */

#define	KEYFLASHTIMEOUT	180
#define	KEYFLASHDELAY	30

int timeout;
int flashkey;

void FlashKey (int key) {
if (timeout) DrawKeys ();

timeout = KEYFLASHTIMEOUT+1;
DrawKey ((flashkey = key == 1), true);
}	// FlashKey

void KeysFlashing () {
int oldstate = timeout / KEYFLASHDELAY, newstate;

if ((timeout -= tics) < 0) timeout = 0;

newstate = timeout / KEYFLASHDELAY;

if ((oldstate ^ newstate) & 1)
	DrawKey (flashkey, newstate & 1);
}	// KeysFlashing

/*
==================
=
= GiveTreasure
=
==================
*/

void GiveTreasure (int points)
{
	GivePoints (points);
	gamestate.treasurecount++;
}

/*
=============================================================================

							MOVEMENT

=============================================================================
*/

/*
===================
=
= GetBonus
=
===================
*/
void GetBonus (statobj_t *check)
{
	switch (check->itemnumber)
	{
	case	bo_firstaid:
		if (! HealSelf(25)) return;

		SD_PlaySound (HEALTH2SND);
		break;

	case	bo_food:
		if (! HealSelf(10)) return;

		SD_PlaySound (HEALTH1SND);
		break;

	case	bo_alpo:
		if (! HealSelf(4)) return;

		SD_PlaySound (HEALTH1SND);
		break;

	case	bo_gibs:
		if (gamestate.health >= 10) return;
		HealSelf(1);

		SD_PlaySound (SLURPIESND);
		break;

	case	bo_key1:
	case	bo_key2:
	case	bo_key3:
	case	bo_key4:
		GiveKey (check->itemnumber - bo_key1);
		SD_PlaySound (GETKEYSND);
		break;

	case	bo_cross:
		SD_PlaySound (BONUS1SND);
		GiveTreasure(100);
		break;

	case	bo_chalice:
		SD_PlaySound (BONUS2SND);
		GiveTreasure(500);
		break;

	case	bo_bible:
		SD_PlaySound (BONUS3SND);
		GiveTreasure(1000);
		break;

	case	bo_crown:
		SD_PlaySound (BONUS4SND);
		GiveTreasure(5000);
		break;

	case	bo_clip:
		if (! GiveAmmo(8)) return;

		SD_PlaySound (GETAMMOSND);
		break;

	case	bo_clip2:
		if (! GiveAmmo(4)) return;

		SD_PlaySound (GETAMMOSND);
		break;

	case	bo_25clip:
		if (! GiveAmmo(25)) return;

		SD_PlaySound (GETAMMOBOXSND);
		break;

	case	bo_machinegun:
		if (! GiveWeapon(wp_machinegun)) return;

		SD_PlaySound (GETMACHINESND);
		break;

	case	bo_chaingun:
		if (! GiveWeapon(wp_chaingun)) return;

		SD_PlaySound (GETGATLINGSND);
		facecount = 0;
		face_gotgat = true;
		DrawFace ();
		break;

	case	bo_fullheal:
		SD_PlaySound (BONUS1UPSND);
		HealSelf(MAX_HEALTH);
		GiveAmmo(25);
		GiveExtraMan ();
		gamestate.treasurecount++;
		break;

	case	bo_spear:
		spearflag = true;
		spearx = player->x;
		speary = player->y;
		spearangle = player->angle;
		playstate = ex_completed;
	}

	StartBonusFlash ();
	check->shapenum = -1;			// remove from list

	map_gfx[check->tilex][check->tiley] = 0;
}

/*
===================
=
= TryMove
=
= returns true if move ok
=
===================
*/

boolean TryMove(objtype *ob)
{
	int		xl,yl,xh,yh,x,y;
	objtype		*check;
	long		deltax,deltay;

	xl = (ob->x-PLAYERSIZE) >> TILESHIFT;
	yl = (ob->y-PLAYERSIZE) >> TILESHIFT;

	xh = (ob->x+PLAYERSIZE) >> TILESHIFT;
	yh = (ob->y+PLAYERSIZE) >> TILESHIFT;

//
// check for solid walls
//
	for (y=yl;y<=yh;y++)
		for (x=xl;x<=xh;x++)
		{
			if (actorat[x][y] && !(actorat[x][y] & AM_OBJID))
				return false;
		}

//
// check for actors
//
	if (yl>0)
		yl--;
	if (yh<MAPHGT-1)
		yh++;
	if (xl>0)
		xl--;
	if (xh<MAPWDT-1)
		xh++;

	for (y=yl;y<=yh;y++)
		for (x=xl;x<=xh;x++)
		{	int actor;
			if ((actor = actorat[x][y]) & AM_OBJID) {
				check = &objlist[actor & ~AM_OBJID]; 

				if (check->flags & FL_SHOOTABLE)
				{
					deltax = ob->x - check->x;
					if (deltax < -MINACTORDIST || deltax > MINACTORDIST)
						continue;
					deltay = ob->y - check->y;
					if (deltay < -MINACTORDIST || deltay > MINACTORDIST)
						continue;

					return false;
				}
			}
		}

	return true;
}

/*
===================
=
= ClipMove
=
===================
*/

void ClipMove(objtype *ob, long xmove, long ymove)
{
	long	basex,basey;

	basex = ob->x;
	basey = ob->y;

	ob->x = basex+xmove;
	ob->y = basey+ymove;
	if (TryMove(ob))
		return;

	if (noclip && ob->x > 2*TILEGLOBAL && ob->y > 2*TILEGLOBAL &&
	   ob->x < (((long)(MAPWDT-1))<<TILESHIFT) &&
	   ob->y < (((long)(MAPHGT-1))<<TILESHIFT))
		return;		// walk through walls

	if (!SD_SoundPlaying())
		SD_PlaySound (HITWALLSND);

	ob->x = basex+xmove;
	ob->y = basey;
	if (TryMove(ob))
		return;

	ob->x = basex;
	ob->y = basey+ymove;
	if (TryMove(ob))
		return;

	ob->x = basex;
	ob->y = basey;
}

//==========================================================================

/*
===================
=
= CheckTaking
=
===================
*/

void CheckTaking (int tilex, int tiley) {
// TODO: this may be optimized;
// problem with not taking several objects?
statobj_t	*statptr;

for (statptr = statobjlist; statptr != laststatobj; statptr ++)
    if (statptr->tilex == tilex && statptr->tiley == tiley &&
    	statptr->shapenum > 0 && (statptr->flags & FL_BONUS))
	{ GetBonus (statptr); return; }
}	// CheckTaking

/*
===================
=
= VictoryTile
=
===================
*/

void VictoryTile()
{
	if (GAME_WOLF) SpawnBJVictory();
	gamestate.victoryflag = true;
}

/*
===================
=
= Thrust
=
===================
*/

void Thrust(int angle, long speed)
{
	long xmove,ymove;
	unsigned offset;

	//
	// ZERO FUNNY COUNTER IF MOVED!
	//
	if (speed)
		funnyticount = 0;

	thrustspeed += speed;

//
// moving bounds speed
//
	if (speed >= MINDIST*2)
		speed = MINDIST*2-1;

	xmove = FixedByFrac(speed,costable[angle]);
	ymove = -FixedByFrac(speed,sintable[angle]);

	ClipMove(player,xmove,ymove);

	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;

	offset = farmapylookup[player->tiley]+player->tilex;
	player->areanumber = *(mapsegs[0] + offset) - AREATILE;

//
// check for object taken
//
	CheckTaking (player->tilex, player->tiley);

	if (*(mapsegs[1] + offset) == EXITTILE)
		VictoryTile();
}

/*
=============================================================================

								ACTIONS

=============================================================================
*/

/*
===============
=
= Cmd_Fire
=
===============
*/

void Cmd_Fire()
{
	buttonheld[bt_attack] = true;

	gamestate.weaponframe = 0;

	player->state = s_attack;

	gamestate.attackframe = 0;
	gamestate.attackcount =
		attackinfo[gamestate.weapon][gamestate.attackframe].tics;
	gamestate.weaponframe =
		attackinfo[gamestate.weapon][gamestate.attackframe].frame;
}

//===========================================================================

/*
===============
=
= Cmd_Use
=
===============
*/

void Cmd_Use()
{
	int checkx, checky, doornum, dir;
	boolean elevatorok;

//
// find which cardinal direction the player is facing
//
	if (player->angle < ANGLES/8 || player->angle > 7*ANGLES/8)
	{
		checkx = player->tilex+1;
		checky = player->tiley;
		dir = di_east;
		elevatorok = true;
	}
	else if (player->angle < 3*ANGLES/8)
	{
		checkx = player->tilex;
		checky = player->tiley-1;
		dir = di_north;
		elevatorok = false;
	}
	else if (player->angle < 5*ANGLES/8)
	{
		checkx = player->tilex-1;
		checky = player->tiley;
		dir = di_west;
		elevatorok = true;
	}
	else
	{
		checkx = player->tilex;
		checky = player->tiley+1;
		dir = di_south;
		elevatorok = false;
	}

	doornum = tilemap[checkx][checky];
	if (*(mapsegs[1]+farmapylookup[checky]+checkx) == PUSHABLETILE)
	{
	//
	// pushable wall
	//

		PushWall (checkx,checky,dir);
		return;
	}
	if (!buttonheld[bt_use] && doornum == ELEVATORTILE && elevatorok)
	{
	//
	// use elevator
	//
		buttonheld[bt_use] = true;

		tilemap[checkx][checky]++;		// flip switch
		if (*(mapsegs[0]+farmapylookup[player->tiley]+player->tilex) == ALTELEVATORTILE)
			playstate = ex_secretlevel;
		else
			playstate = ex_completed;
		SD_PlaySound(LEVELDONESND);
		SD_WaitSoundDone();
	}
	else if (!buttonheld[bt_use] && doornum & AM_DOOR)
	{
		buttonheld[bt_use] = true;
		OperateDoor(doornum & ~AM_DOOR);
	}
	else
		SD_PlaySound(DONOTHINGSND);

}

/*
=============================================================================

						   PLAYER CONTROL

=============================================================================
*/

/*
===============
=
= SpawnPlayer
=
===============
*/

void SpawnPlayer (int tilex, int tiley, int dir)
{
	player->obclass = playerobj;
	player->active = true;
	player->tilex = tilex;
	player->tiley = tiley;
	player->areanumber =
		*(mapsegs[0] + farmapylookup[player->tiley]+player->tilex);
	player->x = ((long)tilex<<TILESHIFT)+TILEGLOBAL/2;
	player->y = ((long)tiley<<TILESHIFT)+TILEGLOBAL/2;
	player->state = s_player;
	player->angle = (1-dir)*90;
	if (player->angle<0)
		player->angle += ANGLES;
	player->flags = FL_NEVERMARK;
	Thrust (0,0);				// set some variables

	InitAreas();
}

//===========================================================================

/*
===============
=
= T_KnifeAttack
=
= Update player hands, and try to do damage when the proper frame is reached
=
===============
*/

void KnifeAttack (objtype *ob)
{
	objtype *check,*closest;
	long	dist;

	SD_PlaySound (ATKKNIFESND);
	// actually fire
	dist = 0x7fffffff;
	closest = NULL;
	for (check=ob->next; check; check=check->next)
		if ((check->flags & FL_SHOOTABLE)
		    && (check->flags & FL_VISABLE)
		    && abs (check->viewx-centerx) < shootdelta)
		{
			if (check->transx < dist)
			{
				dist = check->transx;
				closest = check;
			}
		}

	if (!closest || dist > 0x18000l)
	{
	// missed
		return;
	}

	// hit something
	DamageActor(closest, US_RndT() >> 4);
}

void GunAttack(objtype *ob)
{
	objtype *check, *closest, *oldclosest;
	int	damage;
	int	dx,dy,dist;
	long	viewdist;

	switch (gamestate.weapon)
	{
	case wp_pistol:
		SD_PlaySound (ATKPISTOLSND);
		break;
	case wp_machinegun:
		SD_PlaySound (ATKMACHINEGUNSND);
		break;
	case wp_chaingun:
		SD_PlaySound (ATKGATLINGSND);
		break;
	default:
		break;
	}

	madenoise = true;

//
// find potential targets
//
	viewdist = 0x7fffffffl;
	closest = NULL;

	while (1)
	{
		oldclosest = closest;

		for (check=ob->next; check; check=check->next)
			if ( (check->flags & FL_SHOOTABLE)
			  && (check->flags & FL_VISABLE)
			  && abs (check->viewx-centerx) < shootdelta)
			{
				if (check->transx < viewdist)
				{
					viewdist = check->transx;
					closest = check;
				}
			}

		if (closest == oldclosest)
			return;		// no more targets, all missed

	//
	// trace a line from player to enemey
	//
		if (CheckLine(closest))
			break;

	}

//
// hit something
//
	dx = abs(closest->tilex - player->tilex);
	dy = abs(closest->tiley - player->tiley);
	dist = dx>dy ? dx:dy;

	if (dist<2)
		damage = US_RndT() / 4;
	else if (dist<4)
		damage = US_RndT() / 6;
	else
	{
		if ((US_RndT() / 12) < dist)		// missed
			return;
		damage = US_RndT() / 6;
	}

	DamageActor (closest,damage);
}

//===========================================================================

/*
===============
=
= VictorySpin
=
===============
*/

void VictorySpin()
{
	long desty;

	if (player->angle > 270)
	{
		player->angle -= tics * 3;
		if (player->angle < 270)
			player->angle = 270;
	}
	else if (player->angle < 270)
	{
		player->angle += tics * 3;
		if (player->angle > 270)
			player->angle = 270;
	}

	desty = (((long)player->tiley-5)<<TILESHIFT)-0x3000;

	if (player->y > desty)
	{
		player->y -= tics*4096;
		if (player->y < desty)
			player->y = desty;
	}
}

//
//===========================================================================
//
// GetWeaponSprite
//
//===========================================================================
//

int GetWeaponSprite () {

static int weaponscale[NUMWEAPONS] = {
	SPR_KNIFEREADY,
	SPR_PISTOLREADY,
	SPR_MACHINEGUNREADY,
	SPR_CHAINREADY
	};

if (GAME_WOLF && gamestate.victoryflag)
    return ((player->state == s_deathcam) && (get_TimeCount() & 32)) ?
	SPR_DEATHCAM : -1;

if (gamestate.weapon != -1)
	return weaponscale[gamestate.weapon]+gamestate.weaponframe;

return -1;
}	// GetWeaponSprite

//===========================================================================

/*
===============
=
= T_Attack
=
===============
*/

void T_Attack(objtype *ob)
{
	struct atkinf *cur;

	UpdateFace();

	if (gamestate.victoryflag)		// watching the BJ actor
	{
		VictorySpin();
		return;
	}

	if (buttonstate[bt_use] && !buttonheld[bt_use])
		buttonstate[bt_use] = false;

	if (buttonstate[bt_attack] && !buttonheld[bt_attack])
		buttonstate[bt_attack] = false;

	ControlMovement(ob);
	if (gamestate.victoryflag)		// watching the BJ actor
		return;

	plux = player->x >> UNSIGNEDSHIFT;		// scale to fit in unsigned
	pluy = player->y >> UNSIGNEDSHIFT;
	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;

//
// change frame and fire
//
	gamestate.attackcount -= tics;
	while (gamestate.attackcount <= 0)
	{
		cur = &attackinfo[gamestate.weapon][gamestate.attackframe];
		switch (cur->attack)
		{
		case -1:
			ob->state = s_player;
			if (!gamestate.ammo)
			{
				gamestate.weapon = wp_knife;
				DrawWeapon ();
			}
			else
			{
				if (gamestate.weapon != gamestate.chosenweapon)
				{
					gamestate.weapon = gamestate.chosenweapon;
					DrawWeapon ();
				}
			};
			gamestate.attackframe = gamestate.weaponframe = 0;
			return;

		case 4:
			if (!gamestate.ammo)
				break;
			if (buttonstate[bt_attack])
				gamestate.attackframe -= 2;
		case 1:
			if (!gamestate.ammo)
			{	// can only happen with chain gun
				gamestate.attackframe++;
				break;
			}
			GunAttack (ob);
			gamestate.ammo--;
			DrawAmmo ();
			break;

		case 2:
			KnifeAttack (ob);
			break;

		case 3:
			if (gamestate.ammo && buttonstate[bt_attack])
				gamestate.attackframe -= 2;
			break;
		}

		gamestate.attackcount += cur->tics;
		gamestate.attackframe++;
		gamestate.weaponframe =
			attackinfo[gamestate.weapon][gamestate.attackframe].frame;
	}

}

/*
===============
=
= T_Player
=
===============
*/

void T_Player(objtype *ob)
{
	if (gamestate.victoryflag)		// watching the BJ actor
	{
		VictorySpin();
		return;
	}

	UpdateFace();
	CheckWeaponChange();
	KeysFlashing();

	if (buttonstate[bt_use])
		Cmd_Use();

	if (buttonstate[bt_attack] && !buttonheld[bt_attack])
		Cmd_Fire();

	ControlMovement(ob);
	if (gamestate.victoryflag)			// watching the BJ actor
		return;

	plux = player->x >> UNSIGNEDSHIFT;		// scale to fit in unsigned
	pluy = player->y >> UNSIGNEDSHIFT;
	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;
}

