
/*
 * +==========================================================
 * |	'wl_draw.c':
 * |	Framebuffer pixel-level drawing
 * |	(3D view rendering + 2D vismap rendering)
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h" 

/* C WallRefresh() and related code
   originally from David Haslam -- dch@sirius.demon.co.uk */

/* the door is the last picture before the sprites */
#define DOORWALL	(PMSpriteStart-8)

#define ACTORSIZE	0x4000

#define	OBJSIZE		0x2000

static unsigned wallheight[MAXVIEWWIDTH];

static fixed focallength;
static fixed scale;
static long heightnumerator;

#define NOASM

#ifndef NOASM

#define FixedByFrac(x, y) \
__extension__  \
({ unsigned long z; \
 asm("imull %2; shrdl $16, %%edx, %%eax" : "=a" (z) : "a" (x), "q" (y) : "%edx"); \
 z; \
})

#endif

// TODO: Add local cacheing of sprites/tiles...

#define	TEXTMASK 0xfc0

/* ======================================================================== */

// TODO: move GetSprite here from "id_ca"?

static void WallRefresh();
static void MapRefresh ();

//
// (for debug draw of gfx && sprites)
//

void DrawGraphic (byte *src, boolean sprite,
	int scrx, int scalew, int scry, int scaleh,
	byte bgshift, byte bgcol0, byte bgcol1) {
byte *gfxp = VL_BeginDraw() + (scry * vwidth) + scrx;
int offset = vwidth - GFXWDT*scalew;
int x, y;

if (! src) return;

for (y = 0; y != GFXHGT; ++ y) {
    int chgt = scaleh;

    while (chgt --) {

    for (x = 0; x != GFXWDT; ++ x) {
	byte val = src [x*GFXHGT + y];
	if (sprite && val == 0xff)
	    val = (((x ^ y) >> bgshift) & 1) ? bgcol0 : bgcol1;

	{ int cwdt = scalew;

	while (cwdt --) *gfxp ++ = val;
	}
	}

    gfxp += offset;
    }

    }

VL_EndDraw();
}	// DrawGraphic

// Draw histogram, using bgcol0/bgcol1
// (assumes array of 'data' contains 'width' entries with 'height' range)
void DrawHisto (unsigned *data,
	int scrx, int width, int scry, int height, byte bgcol0, byte bgcol1) {
byte *gfxp = VL_BeginDraw() + (scry * vwidth) + scrx;
int y, x, offset = vwidth - width;

for (y = 0; y != height; ++ y) {
    for (x = 0; x != width; ++ x)
	*gfxp ++ = y < data[x] ? bgcol0 : bgcol1;

    gfxp += offset;
    }

VL_EndDraw();
}	// DrawHisto

void ShowFrameRate (unsigned frame_rate) {
// TODO: frame rate display
}	// ShowFrameRate

/*
====================
=
= CalcProjection
=
====================
*/

static const double radtoint = (double)FINEANGLES/2.0/PI;

void CalcProjection(long focal)
{
	int     i;
	long    intang;
	double angle, tang, facedist;
	int     halfview;

	focallength = focal;
	facedist = focal+MINDIST;
	halfview = viewwidth/2;               /* half view in pixels */

/*
 calculate scale value for vertical height calculations
 and sprite x calculations
*/
	scale = halfview*facedist/(VIEWGLOBAL/2);

/*
 divide heightnumerator by a posts distance to get the posts height for
 the heightbuffer.  The pixel height is height>>2
*/
	heightnumerator = (TILEGLOBAL*scale)>>6;

/* calculate the angle offset from view angle of each pixel's ray */
	for (i = 0; i < halfview; i++) {
		tang = ((double)i)*VIEWGLOBAL/viewwidth/facedist;
		angle = atan(tang);
		intang = angle*radtoint;
		pixelangle[halfview-1-i] = intang;
		pixelangle[halfview+i] = -intang;
	}
}

/*
========================
=
= TransformActor
=
= Takes paramaters:
=   gx,gy		: globalx/globaly of point
=
= globals:
=   viewx,viewy		: point of view
=   viewcos,viewsin	: sin/cos of viewangle
=   scale		: conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
========================
*/

static void TransformActor(objtype *ob)
{
	fixed gx,gy,nx,ny;

//
// translate point to view centered coordinates
//
	gx = ob->x - viewx;
	gy = ob->y - viewy;

//
// calculate newx
//
	nx = FixedByFrac(gx,viewcos) - FixedByFrac(gy,viewsin) - ACTORSIZE;
			// fudge the shape forward a bit, because
			// the midpoint could put parts of the shape
			// into an adjacent wall

//
// calculate newy
//
	ny = FixedByFrac(gy,viewcos) + FixedByFrac(gx,viewsin);

//
// calculate perspective ratio
//
	ob->transx = nx;
	ob->transy = ny;

	if (nx < MINDIST)	/* too close, don't overflow the divide */
		{
		ob->viewheight = 0;
		}
	else {
		ob->viewx = centerx + ny*scale/nx;	
		ob->viewheight = heightnumerator/(nx>>8);
		}
}

/*
========================
=
= TransformStatic
=
= Takes paramaters:
=   tx,ty		: tile the object is centered in
=
= globals:
=   viewx,viewy		: point of view
=   viewcos,viewsin	: sin/cos of viewangle
=   scale		: conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
=
========================
*/

static void TransformStatic(int tx, int ty, int *dispx, int *dispheight)
{
	fixed gx,gy,nx,ny;

//
// translate point to view centered coordinates
//

#define	OFFSET	0x8000

	gx = ((long)tx<<TILESHIFT)+OFFSET-viewx;
	gy = ((long)ty<<TILESHIFT)+OFFSET-viewy;

//
// calculate newx & newy
//
	nx = FixedByFrac(gx,viewcos) - FixedByFrac(gy,viewsin) - OBJSIZE;
	ny = FixedByFrac(gy,viewcos) + FixedByFrac(gx,viewsin);

//
// calculate perspective ratio
//
	if (nx<MINDIST)			/* too close, don't overflow the divide */
	{
		*dispheight = 0;
		return;
	}

	*dispx = centerx + ny*scale/nx;	

	*dispheight = heightnumerator/(nx>>8);

//
// see if it should be grabbed
//

#if 0
	if ((nx<TILEGLOBAL) && (ny>-TILEGLOBAL/2) && (ny<TILEGLOBAL/2))
		return true;
	else
		return false;
#endif
}

/* ======================================================================== */

/*
=====================
=
= CalcRotate
=
=====================
*/

static int CalcRotate(objtype *ob)
{
	int	angle,viewangle;

	/* this isn't exactly correct, as it should vary by a trig value, */
	/* but it is close enough with only eight rotations */

	viewangle = player->angle + (centerx - ob->viewx)/8;

	if (ob->obclass == rocketobj || ob->obclass == hrocketobj)
		angle =  (viewangle-180)- ob->angle;
	else
		angle =  (viewangle-180) - (ob->dir)*ANGLES/8;

	angle+=ANGLES/16;
	while (angle>=ANGLES)
		angle-=ANGLES;
	while (angle<0)
		angle+=ANGLES;

	if (gamestates[ob->state].rotate == 2)  // 2 rotation pain frame
		return 4*(angle/(ANGLES/2));    // seperated by 3

	return angle/(ANGLES/8);
}

/*
========================
=
= ScaleLine
=
========================
*/

static byte *gfxbuf = 0;				// (controlled by VL_BeginDraw/VL_EndDraw)

static void ScaleLine (unsigned height, byte *source, int x, boolean trans)
{
	unsigned y, frac, delta;

	if (height) {
		frac = (64 << 16) / height;
		delta = (64 << 16) - frac*height;
		
		if (height < viewheight)
			y = yoffset + (viewheight - height) / 2;
		else {
			y = ((height - viewheight) / 2) * frac;
			source += y >> 24;
			height = viewheight;
			delta += y;
			y = yoffset;
			}

	if (trans) {
	    byte *vid = gfxbuf + (y * vwidth) + x + xoffset, db;

	    while (height --) {
		if ((db = source[delta >> 16]) != 255) *vid = db;
		vid += vwidth;
		delta += frac;
		}

	    }

	else {
	    byte *vid = gfxbuf + (yoffset * vwidth) + x + xoffset;
	    unsigned h_ceil = y - yoffset;
	    unsigned h_floor = viewheight - h_ceil - height;

	    while (h_ceil --) {
		*vid = px_ceiling;
		vid += vwidth;
		}

	    while (height --) {
		*vid = source[delta >> 16];
		vid += vwidth;
		delta += frac;
		}

	    while (h_floor --) {
		*vid = px_floor;
		vid += vwidth;
		}
	    }

	}
}	// ScaleLine

/*
=====================
=
= ScaleShape
=
=====================
*/

// Scale shape (with improved clipping)
void ScaleShape (int xcenter, int shapenum, unsigned height, boolean no_vclip) {
BBox *box;
byte *sprite = GetSprite(shapenum, &box);

unsigned scaler = (64 << (16 + 2)) / height;
unsigned tx;

int vx = xcenter - (height >> 3);
int ex = vx + (box->r << 16) / scaler;			// (sprite right limit)
vx += (tx = box->l << 16) / scaler;					// (sprite left limit)

// (visibility clipping)

if (ex <= 0 || vx >= viewwidth)
	return;			// (completely out of H-range)

// ==> (vx < viewwidth && ex > 0)
if (vx < 0) {			// (clip left)
	tx += scaler * (- vx);
	vx = 0;
	}

// ==> (vx >= 0)
if (ex > viewwidth)		// (clip right)
	ex = viewwidth;

unsigned count = ex - vx;
while (count --) {
    if (no_vclip || wallheight[vx] < height)
	ScaleLine (height >> 2, sprite + ((tx >> 16) << 6), vx, true);

	tx += scaler;
	vx ++;
	}	// while
}	// ScaleShape

/*
=====================
=
= DrawScaleds
=
= Draws all objects that are visible
=
=====================
*/

#define MAXVISABLE      64

typedef struct {
	int viewx;
	int viewheight;
	int shapenum;
} visobj_t;

static int compare_viewheight (const void *vis1, const void *vis2) {
return ((visobj_t *) vis1)->viewheight - ((visobj_t *) vis2)->viewheight;
}	// compare_viewheight

static void DrawScaleds()
{
	byte		*tilespot,*visspot;
	unsigned	spotloc;

	statobj_t	*statptr;
	objtype		*obj;

	visobj_t vislist[MAXVISABLE], *visptr, *visstep;

	visptr = vislist;

//
// place static objects
//
	for (statptr = statobjlist; statptr != laststatobj; statptr++)
	{
		if ((visptr->shapenum = statptr->shapenum) == -1)
			continue;	/* object has been deleted */

		if (!*statptr->visspot)
			continue;			/* not visable */

		TransformStatic(statptr->tilex, statptr->tiley,
		    &visptr->viewx, &visptr->viewheight);

		if (!visptr->viewheight)
			continue;			/* too close to the object */

		if (visptr < &vislist[MAXVISABLE-1])	/* don't let it overflow */
			visptr++;
	}

//
// place active objects
//
	for (obj = player->next; obj; obj = obj->next)
	{
		if (!(visptr->shapenum = gamestates[obj->state].shapenum))
			continue;  // no shape

		spotloc = (obj->tilex << 6) + obj->tiley;
		visspot = &spotvis[0][0] + spotloc;
		tilespot = &tilemap[0][0] + spotloc;

		//
		// could be in any of the nine surrounding tiles
		//
		if (*visspot
		|| (visspot[-1] && !tilespot[-1])
		|| (visspot[ 1] && !tilespot[ 1])
		|| (visspot[-MAPWDT-1] && !tilespot[-MAPWDT-1])
		|| (visspot[-MAPWDT  ] && !tilespot[-MAPWDT  ])
		|| (visspot[-MAPWDT+1] && !tilespot[-MAPWDT+1])
		|| (visspot[ MAPWDT+1] && !tilespot[ MAPWDT+1])
		|| (visspot[ MAPWDT  ] && !tilespot[ MAPWDT  ])
		|| (visspot[ MAPWDT-1] && !tilespot[ MAPWDT-1]))
		{
			obj->active = true;
			TransformActor(obj);
			if (!obj->viewheight)
				continue;		// too close or far away

			visptr->viewx = obj->viewx;
			visptr->viewheight = obj->viewheight;
			if (visptr->shapenum == -1)
				visptr->shapenum = obj->temp1;	// special shape

			if (gamestates[obj->state].rotate)
				visptr->shapenum += CalcRotate(obj);

			if (visptr < &vislist[MAXVISABLE-1])	/* don't let it overflow */
				visptr++;
			obj->flags |= FL_VISABLE;
		} else
			obj->flags &= ~FL_VISABLE;
	}

//
// draw from back to front
//

	if (visptr != vislist) {
	qsort (vislist, visptr-vislist, sizeof (visobj_t), compare_viewheight);

	for (visstep = vislist; visstep != visptr; visstep++)
	    ScaleShape (visstep->viewx, visstep->shapenum, visstep->viewheight, false);
	}
}

/* ======================================================================== */

/*
========================
=
= ThreeDRefresh
=
========================
*/

#define	px_void	0

unsigned int px_ceiling, px_floor;

// (in "wl_play.c")
extern unsigned frame_rate;

boolean map_mode = false;

boolean map_scroll = false;

boolean show_rate = false;

void ThreeDRefresh()
{

gfxbuf = VL_BeginDraw();

// (Refresh some globals)
viewangle = player->angle;
viewsin = sintable[viewangle];
viewcos = costable[viewangle];
viewx = player->x - FixedByFrac(focallength, viewcos);
viewy = player->y + FixedByFrac(focallength, viewsin);

if (map_mode) {
	MapRefresh();
	}

else {	int weapon;

	/* clear out the traced array */
	memset(spotvis, 0, sizeof(spotvis));

	/* draw floor, ceiling & walls */
	WallRefresh();

	/* draw all the scaled images */
	DrawScaleds();

	/* draw player's weapon */
	weapon = GetWeaponSprite ();

	if (weapon != -1)
	    ScaleShape(viewwidth/2, weapon, viewheight << 2, true);

	/* if demo playback/record */
	if (demorecord || demoplayback)
	    ScaleShape(viewwidth/2, SPR_DEMO, viewheight << 2, true);

	// Refresh map visibility data
	{ unsigned x, y;

	for (y = 0; y != MAPHGT; ++ y)
	    for (x = 0; x != MAPWDT; ++ x)
		if (spotvis[x][y]) map_dir[x][y] &= ~MP_HID;
	}

	}	// ! map_mode

if (show_rate) ShowFrameRate (frame_rate);

/* show screen and time last cycle */	

	VL_UpdateScreenArea (yoffset, viewheight, xoffset, viewwidth);
  VL_EndDraw ();
	frameon++;
}

/* ======================================================================== */

#if 0

// (These aren't used anymore and preserved for reference only)

void ScaleShapeOld(int xcenter, int shapenum, unsigned height, boolean no_vclip)
{
	byte *sprite = GetSprite (shapenum);
	unsigned int scaler = (64 << 16) / (height >> 2);
	int x, p;

	for (p = xcenter - (height >> 3), x = 0; x < (64 << 16); x += scaler, p++) {
	    if (p >= 0 && p < viewwidth &&
	        (no_vclip || wallheight[p] < height))
		ScaleLine (height >> 2, sprite + ((x >> 16) << 6), p, true);
	}
}

void SimpleScaleShapeOld(int xcenter, int shapenum, unsigned height)
{
ScaleShapeOld(xcenter, shapenum, height << 2, true);
}

#endif

/* ======================================================================== */

// Get page for door 'doornum'
static int GetDoorPg (int doornum) {
    switch(doorobjlist[doornum].lock) {
	case dr_normal:
		return DOORWALL;

	case dr_lock1:
	case dr_lock2:
	case dr_lock3:
	case dr_lock4:
		return DOORWALL+6;

	case dr_elevator:
		return DOORWALL+4;
	}	// switch

return 0;
}	// GetDoorPg

#define DEG90	900
#define DEG180	1800
#define DEG270	2700
#define DEG360	3600

// TODO: map bounds clipping in raycasting

/*
====================
=
= WallRefresh
=
====================
*/

#define TILE(n) ((n)>>TILESHIFT)

void ScaleWallPost(int xp, int yp,
	unsigned postx, unsigned wallpic, unsigned textoff) {
	int height;

  // calculates the height of xp,yp from viewx,viewy

	fixed nx = FixedByFrac((xp - viewx), viewcos) -
		   FixedByFrac((yp - viewy), viewsin);

  //
  // calculate perspective ratio (heightnumerator/(nx>>8))
  //
	if (nx < MINDIST)
		nx = MINDIST;		/* don't let divide overflow */

	wallheight[postx] = height = heightnumerator/(nx>>8);

	ScaleLine((height & 0xFFF8) >> 2,
		PM_GetPage(wallpic) + textoff, postx, false);
}	// ScaleWallPost

// (These are common for all rays)
static unsigned xpartialup, xpartialdown, ypartialup, ypartialdown;
static unsigned focaltx, focalty;
static int midangle;

// (where ray ended)
int xintercept, yintercept;

// (texture offset)
unsigned short textoff;
unsigned short wallpic;

static void TraceRay(unsigned postx) {
	int angle;		/* ray angle through `postx` */
	unsigned xpartial, ypartial;
	int doorhit;
	int xstep, ystep;
	int xtile, ytile;
	int xtilestep, ytilestep;
	int xcheck, ycheck;
	unsigned offset;
	unsigned doornum;

	unsigned tilehit;

	angle = midangle + pixelangle[postx];

	if (angle < 0) {
		/* -90 - -1 degree arc */
		angle += FINEANGLES;
	} else if (angle >= DEG360)
		angle -= FINEANGLES;

	// angle is 0..360

	if (angle < DEG90) {			/* 0-89 degree arc */
		xtilestep = 1;
		ytilestep = -1;
		xstep = finetangent[DEG90-1-angle];
		ystep = -finetangent[angle];
		xpartial = xpartialup;
		ypartial = ypartialdown;
	} else if (angle < DEG180) {		/* 90-179 degree arc */
		xtilestep = -1;
		ytilestep = -1;
		xstep = -finetangent[angle-DEG90];
		ystep = -finetangent[DEG180-1-angle];
		xpartial = xpartialdown;
		ypartial = ypartialdown;
	} else if (angle < DEG270) {		/* 180-269 degree arc */
		xtilestep = -1;
		ytilestep = 1;
		xstep = -finetangent[DEG270-1-angle];
		ystep = finetangent[angle-DEG180];
		xpartial = xpartialdown;
		ypartial = ypartialup;
	} else {				/* 270-359 degree arc */
		xtilestep = 1;
		ytilestep = 1;
		xstep = finetangent[angle-DEG270];
		ystep = finetangent[DEG360-1-angle];
		xpartial = xpartialup;
		ypartial = ypartialup;
	}

	yintercept = viewy + FixedByFrac(xpartial, ystep); // + xtilestep;
	xtile = focaltx + xtilestep;

	xintercept = viewx + FixedByFrac(ypartial, xstep); // + ytilestep;
	ytile = focalty + ytilestep;

/* CORE LOOP */

VertCheck:
	/* check intersections with vertical walls */
	ycheck = TILE (yintercept);
	if (ytilestep >= 0 ? (ycheck >= ytile) : (ycheck <= ytile))
		{ xcheck = TILE (xintercept); goto HorizEntry; }

VertEntry:
	tilehit = tilemap[xtile][ycheck];

	if (tilehit) {
	if (tilehit & AM_DOOR) {
		if ((tilehit & AM_PUSH) == AM_PUSH) {
			/* vertical pushwall */
			doorhit = yintercept + (signed)((signed)pwallpos * ystep) / 64;

			if (TILE(doorhit) != ycheck) 
				goto VertPass;

			yintercept = doorhit;
			xintercept = xtile << TILESHIFT;

			// Hit vertical pushwall!
			textoff = (yintercept >> 4) & TEXTMASK;
			offset = pwallpos << 10;

			if (xtilestep == -1) {
				xintercept += TILEGLOBAL-offset;
				textoff = TEXTMASK-textoff;
			} else
				xintercept += offset;

			wallpic = vertwall[tilehit&~AM_PUSH];
			return;

		} else {
			/* vertical door */
			doorhit = yintercept + ystep / 2;

			if (TILE(doorhit) != ycheck)
				goto VertPass;

			/* check door position */
			doornum = tilehit&~AM_DOOR;
			if ((uint16_t) doorhit < doorposition[doornum])
				goto VertPass;

			yintercept = doorhit;
			xintercept = (xtile << TILESHIFT) + TILEGLOBAL/2;

			/* Hit vertical door! */
			wallpic = GetDoorPg(doornum)+1;
			textoff = ((yintercept-doorposition[doornum]) >> 4) & TEXTMASK;
			return;
		}
	} else {
		xintercept = xtile << TILESHIFT;

		// Hit vertical wall
		spotvis[xtile][ycheck] = 1;
		textoff = (yintercept >> 4) & TEXTMASK;

		if (xtilestep == -1) {
			textoff = TEXTMASK-textoff;
			xintercept += TILEGLOBAL;
			}

		if (tilehit & AM_DSID) {	// check for adjacent doors
			ytile = yintercept>>TILESHIFT;
			if (tilemap[xtile-xtilestep][ytile] & AM_DOOR)
				wallpic = DOORWALL+3;
			else
				wallpic = vertwall[tilehit & ~AM_DSID];
		} else		wallpic = vertwall[tilehit];

		return;
		}

	return;
	}	// (tilehit)

VertPass:
	spotvis[xtile][ycheck] = 1;
	xtile += xtilestep;
	yintercept += ystep;
	goto VertCheck;

HorizCheck:
	/* check intersections with horizontal walls */
	xcheck = TILE (xintercept);
	if (xtilestep >= 0 ? (xcheck >= xtile) : (xcheck <= xtile))
		{ ycheck = TILE (yintercept); goto VertEntry; }

HorizEntry:
	tilehit = tilemap[xcheck][ytile];

	if (tilehit) {
	if (tilehit & AM_DOOR) {
		if ((tilehit & AM_PUSH) == AM_PUSH) {
			/* horizontal pushwall */
			doorhit = xintercept + (signed)((signed)pwallpos * xstep) / 64;

			if (TILE(doorhit) != xcheck)
				goto HorizPass;

			xintercept = doorhit;
			yintercept = ytile << TILESHIFT;

			// Hit horizontal pushwall!
			textoff = (xintercept >> 4) & TEXTMASK;
			offset = pwallpos << 10;

			if (ytilestep == -1)
				yintercept += TILEGLOBAL-offset;
			else {
				textoff = TEXTMASK-textoff;
				yintercept += offset;
			}

			wallpic = horizwall[tilehit&~AM_PUSH];
			return;

		} else {
			/* horizontal door */
			doorhit = xintercept + xstep / 2;

			if (TILE(doorhit) != xcheck)
				goto HorizPass;

			/* check door position */
			doornum = tilehit&~AM_DOOR;
			if ((uint16_t) doorhit < doorposition[doornum])
				goto HorizPass;

			xintercept = doorhit;
			yintercept = (ytile << TILESHIFT) + TILEGLOBAL/2;

			/* Hit horizontal door! */
			wallpic = GetDoorPg(doornum);
			textoff = ((xintercept-doorposition[doornum]) >> 4) & TEXTMASK;
			return;
		}
	} else {
		yintercept = ytile << TILESHIFT;

		// Hit horizontal wall
		spotvis[xcheck][ytile] = 1;
		textoff = (xintercept >> 4) & TEXTMASK;

		if (ytilestep == -1)
			yintercept += TILEGLOBAL;
		else
			textoff = TEXTMASK - textoff;

		if (tilehit & AM_DSID) {	// check for adjacent doors
			xtile = xintercept>>TILESHIFT;
			if (tilemap[xtile][ytile-ytilestep] & AM_DOOR)
				wallpic = DOORWALL+2;
			else
				wallpic = horizwall[tilehit & ~AM_DSID];
		} else		wallpic = horizwall[tilehit];

		return;
		}

	return;
	}	// (tilehit)

HorizPass:
	spotvis[xcheck][ytile] = 1;
	ytile += ytilestep;
	xintercept += xstep;
	goto HorizCheck;

}	// TraceRay

boolean optimised_draw = false;

void RenderSpan (unsigned from, unsigned to) {
unsigned postx;
for (postx = from; postx != to; postx ++) {
	TraceRay (postx);
	ScaleWallPost(xintercept, yintercept, postx, wallpic, textoff);
	}	// for (postx)
}	// RenderSpan

static void WallRefresh()
{
	unsigned postx;
	midangle = viewangle*(FINEANGLES/ANGLES);

	xpartialup = TILEGLOBAL-(xpartialdown = (viewx&(TILEGLOBAL-1)));
	ypartialup = TILEGLOBAL-(ypartialdown = (viewy&(TILEGLOBAL-1)));

	focaltx = TILE (viewx);
	focalty = TILE (viewy);

	for (postx = 0; postx != viewwidth; postx ++) {
	TraceRay (postx);
	ScaleWallPost(xintercept, yintercept, postx, wallpic, textoff);
	}	// for (postx)

}	// WallRefresh

//
//
// LoneWolf AutoMap
//
//

//
// map_gfx array:
//	0: empty
//	>0: wall tile page #
//	<0: sprite #
//

int map_gfx [MAPWDT][MAPHGT];

//
// map_dir: map tile orientation (di_*), polarity and visibility
//

byte map_dir [MAPWDT][MAPHGT];

//
// map_off: map tiles offset (0..63)
//

byte map_off [MAPWDT][MAPHGT];

// Is map fullview mode active?
boolean map_all = false;

static byte dir_table [16] = {
	di_north,		// 0x0: ....
	di_west,		// 0x1: ...W
	di_east,		// 0x2: ..E.
	di_north,		// 0x3: ..EW
	di_north,		// 0x4: .N..
	di_north,		// 0x5: .N.W
	di_north,		// 0x6: .NE.
	di_north,		// 0x7: .NEW

	di_south,		// 0x8: S...
	di_south,		// 0x9: S..W
	di_south,		// 0xA: S.E.
	di_south,		// 0xB: S.EW
	di_east,		// 0xC: SN..
	di_west,		// 0xD: SN.W
	di_east,		// 0xE:	SNE.
	di_north		// 0xF: SNEW
	};

/* Initialise graphic maps */
void MapViewInit () {
int xtile, ytile;

for (xtile = 0; xtile != MAPWDT; ++ xtile)
    for (ytile = 0; ytile != MAPHGT; ++ ytile) {
	int gfx = 0;
	byte dir = 0;
	byte tile;

	if ((tile = tilemap[xtile][ytile])) {

	if (tile & AM_DOOR) {
	unsigned doornum = tile&~AM_DOOR;
	boolean vertical = doorobjlist[doornum].vertical;

	gfx = GetDoorPg (doornum) + (vertical? 1 : 0);
	dir = vertical ? di_east : di_north;
	}
	else {	dir = dir_table [
			(! xtile           || tilemap[xtile-1][ytile] ? 1 : 0) |
			(xtile+1 == MAPWDT || tilemap[xtile+1][ytile] ? 2 : 0) |
			(! ytile           || tilemap[xtile][ytile-1] ? 4 : 0) |
			(ytile+1 == MAPHGT || tilemap[xtile][ytile+1] ? 8 : 0) ];

		gfx = (dir == di_east || dir == di_west ? vertwall : horizwall)
			[tile & 63];
		}

	gfx ++;
	}	// (tile)

	map_gfx [xtile][ytile] = gfx;
	map_dir [xtile][ytile] = dir | MP_HID;
	map_off [xtile][ytile] = 0;
    }


// (place static onto map)
{
statobj_t *statptr;

for (statptr = &statobjlist[0]; statptr != laststatobj; statptr ++) {
	if (statptr->shapenum == -1)
		continue;		/* object has been deleted */

	map_gfx [statptr->tilex][statptr->tiley] =
		-statptr->shapenum - 1;
	}
}

}	// MapViewInit

// Map origin (rel. to view origin)
static int map_originx = 0;
static int map_originy = 0;

// Map offset (per view pixel)
static int map_deltax = 1 << 10;
static int map_deltay = 1 << 10;

#define	MIN_DELTA	1 << 5
#define	MAX_DELTA	1 << 13

boolean map_textured = true;

void Restrict (int *pv, int min, int max) {
if (pv) {
	if (*pv < min) *pv = min;
	else if (*pv > max) *pv = max;
	}
}	// Restrict

// (get map origin)
void MapGetOrigin (int *px, int *py) {
if (px) *px = map_originx + map_deltax * (viewwidth / 2);
if (py) *py = map_originy + map_deltay * (viewheight / 2);
}	// MapGetOrigin

// (set map origin)
void MapSetOrigin (int x, int y) {
map_originx = x - map_deltax * (viewwidth / 2);
map_originy = y - map_deltay * (viewheight / 2);
}	// MapSetOrigin

#define	TOTALWDT	(MAPWDT << TILESHIFT)
#define	TOTALHGT	(MAPHGT << TILESHIFT)

void MapViewScroll (boolean relative, int offset_x, int offset_y) {
if (map_scroll) {
int x, y;

MapGetOrigin (&x, &y);

if (relative) {
	x += offset_x*map_deltax;
	y += offset_y*map_deltay;

	Restrict (&x, 0, TOTALWDT);
	Restrict (&y, 0, TOTALHGT);
	}
else {	x = (offset_x < 0 ? 0 : offset_x > 0 ? TOTALWDT : TOTALWDT / 2);
	y = (offset_y < 0 ? 0 : offset_y > 0 ? TOTALHGT : TOTALHGT / 2);
	}

MapSetOrigin (x, y);
}	// (map_scroll)
}	// MapViewScroll

void MapViewZoom (boolean relative, int by) {
int x, y;

if (map_scroll) MapGetOrigin (&x, &y);

if (relative) {
	map_deltax += by << 4;
	map_deltay += by << 4;
	}
else {
	map_deltax = map_deltay = 1 << (by+4);
	}

Restrict (&map_deltax, MIN_DELTA, MAX_DELTA);
Restrict (&map_deltay, MIN_DELTA, MAX_DELTA);

if (map_scroll) MapSetOrigin (x, y);
}	// MapViewZoom

#define	PLAYER_COLOR	0x0F

/* Plot point at (x, y) with color
   (operation is view-relative && with clipping) */
void MapPlot (int y, int x, unsigned color) {
if (0 <= y && y < viewheight && 0 <= x && x < viewwidth)
	gfxbuf [(yoffset + y) * vwidth + (xoffset + x)] = color;
}	// MapPlot

/* Draw line from (x0, y0) to (x1, y1)
   (starting point included, ending point excluded) */
void MapLine (int y0, int x0, int y1, int x1, unsigned color) {
int sx, sy;
unsigned dx, dy, dev;

// (perform basic clipping)
if ((y0 < 0 && y1 < 0) ||
    (y0 >= viewheight && y1 >= viewheight) ||

    (x0 < 0 && x1 < 0) ||
    (x0 >= viewwidth && x1 >= viewwidth))

	return;

if (x1 >= x0)	dx = x1 - x0, sx = 1;
else		dx = x0 - x1, sx = -1;

if (y1 >= y0)	dy = y1 - y0, sy = 1;
else		dy = y0 - y1, sy = -1;

if (dx >= dy) {
  dev = sx > 0 ? (dx >> 1) : dx - 1 - (dx >> 1);

  while (x0 != x1) {
	MapPlot (y0, x0, color); x0 += sx;
	if ((dev += dy) >= dx) dev -= dx, y0 += sy;
	}	// while (x0 != x1)
  }	// dx >= dy

else {
  dev = sy > 0 ? (dy >> 1) : dy - 1 - (dy >> 1);

  while (y0 != y1) {
	MapPlot (y0, x0, color); y0 += sy;
	if ((dev += dx) >= dy) dev -= dy, x0 += sx;
	}	// while (y0 != y1)
  }	// dx < dy

}	/* MapLine */

// Draw arrow for player at (px, py)
void MapArrow (int px, int py, int scalex, int scaley,
	int angle, unsigned color) {
int x, y, x0, y0, x1, y1, x2, y2;

scalex /= map_deltax;
scaley /= map_deltay;

x = (px - map_originx) / map_deltax;
y = (py - map_originy) / map_deltay;

#define	MapScaleX	((scalex * (int)costable[angle]) >> TILESHIFT)
#define	MapScaleY	((scaley * (int)sintable[angle]) >> TILESHIFT)

x0 = x + MapScaleX;
y0 = y - MapScaleY;

if ((angle += 120) >= ANGLES) angle -= ANGLES;

x1 = x + MapScaleX;
y1 = y - MapScaleY;

if ((angle += 120) >= ANGLES) angle -= ANGLES;

x2 = x + MapScaleX;
y2 = y - MapScaleY;

// (perform basic clipping)
if ((y0 < 0 && y1 < 0 && y2 < 0) ||
    (y0 >= viewheight && y1 >= viewheight && y2 >= viewheight) ||

    (x0 < 0 && x1 < 0 && x2 < 0) ||
    (x0 >= viewwidth && x1 >= viewwidth && x2 >= viewwidth))

	return;

MapLine (y0, x0, y1, x1, color);
MapLine (y1, x1, y, x,   color);
MapLine (y, x, y2, x2,   color);
MapLine (y2, x2, y0, x0, color);
}	// MapArrow

void MapTiling () {
byte *gfxp = gfxbuf + (yoffset * vwidth) + xoffset;

unsigned vx, vy;
int xtile, ytile;
int px = map_originx, py = map_originy;

for (vy = 0; vy != viewheight; ++ vy) {
    ytile = TILE(py);

    if (0 <= ytile && ytile < MAPHGT) {

    for (vx = 0; vx != viewwidth; ++ vx) {
	xtile = TILE(px);

	if (0 <= xtile && xtile < MAPWDT) {
	int gfx = map_gfx[xtile][ytile];
	int dir = map_dir[xtile][ytile];
	int off = map_off[xtile][ytile] & 63;

	if (!map_all && (dir & MP_HID))
		{ *gfxp ++ = px_void; goto done; }

	int rx, ry;

#define	RTILE	(TILEGLOBAL-1)

	switch (dir & MP_DIR) {
		case di_north:	rx = px, ry = py;
				break;
		case di_west:	rx = RTILE-py, ry = px;
				break;
		case di_south:	rx = RTILE-px, ry = RTILE-py;
				break;
		case di_east:	rx = py, ry = RTILE-px;
				break;
		}	// switch

	rx >>= (TILESHIFT-6);
	ry >>= (TILESHIFT-6);
	rx &= GFXWDT-1;
	ry &= GFXHGT-1;

	if (dir & MP_INV) {
	if (rx < off) rx += (64-off);
	else { * gfxp++ = px_floor; goto done; }
	}
	else {
	if (rx >= off) rx -= off;
	else { * gfxp++ = px_floor; goto done; }
	}

	{
	unsigned offset = rx*GFXHGT + ry;

	if (gfx > 0) {
		byte *page = PM_GetPage (gfx - 1);
		*gfxp ++ = page [offset];
		}

	else if (gfx < 0) {
		byte *page = GetSprite (-gfx - 1, 0);
		if ((*gfxp ++ = page [offset]) == 0xFF)
			gfxp[-1] = px_floor;
		}

	else	*gfxp ++ = px_floor;
	}
	}

	else *gfxp ++ = px_void;

	done:
	px += map_deltax;
	}	// for (vx)

    px -= viewwidth * map_deltax;
    }

    else
    for (vx = 0; vx != viewwidth; ++ vx)
	*gfxp ++ = px_void;

    gfxp += vwidth - viewwidth;

    py += map_deltay;
    }		// for (vy)

}	// MapTiling

enum { XX_Side = 16 };

enum { XX_Color = 0x0F };

void MapRefresh () {
if (! map_scroll)
	MapSetOrigin (player->x, player->y);

MapTiling ();

if (map_scroll)	{	// (crosshair)
	int cy = viewheight / 2, cx = viewwidth / 2;
	MapLine (cy, cx-XX_Side, cy, cx+XX_Side, XX_Color);
	MapLine (cy-XX_Side, cx, cy+XX_Side, cx, XX_Color);
	}

// Draw player
MapArrow (player->x, player->y, PLAYERSIZE, PLAYERSIZE,
	player->angle, PLAYER_COLOR);
}	// MapRefresh

