
/*
 * +==========================================================
 * |	'wl_main.c':
 * |	Game main file.
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"
#include "inttypes.h"

/*
=============================================================================

						   WOLFENSTEIN 3-D

					  An Id Software production

						   by John Carmack

=============================================================================
*/

#define FOCALLENGTH     0x5800		/* in global coordinates */

int diag_level = 0;

char str[80], str2[20];

int viewwidth, viewheight;
int viewwidthwin, viewheightwin;	/* for borders */
int xoffset, yoffset;
int vwidth, vheight;			/* size of screen */
int vscale = 1;				/* screen scale factor */
boolean fullscreen;			/* fullscreen mode */
int viewsize;

int centerx;
int shootdelta;			/* pixels away from centerx a target can be */

boolean startgame,loadedgame;
int mouseadjustment;

long frameon;
long lasttimecount;
fixed viewsin, viewcos;
fixed viewx, viewy;		/* the focal point */
int viewangle;

int pixelangle[MAXVIEWWIDTH];
long finetangent[FINEANGLES/4];
int horizwall[MAXWALLTILES], vertwall[MAXWALLTILES];

fixed sintable[ANGLES+ANGLES/4+1], *costable = sintable+(ANGLES/4);

static boolean showversion;

// Engine version
#define	ENGINE_VER	0x01020304

char *GAMENAME, *GAMELBL, *GAMEEXT;
#define	LBLLEN	4

// Read-write resources:
char configname[] = "config";

static char GAMEHDR[] =	"LONEWOLF";
#define	GAMEHDRLEN	8

static char SAVTYPE[] = "SAV\0";
#define SAVTYPELEN	4

static char CFGTYPE[] = "CFG\0";
#define	CFGTYPELEN	4

// Tags
static char T_SAV_GSTATE[]	= "{GSTATE}";
static char T_SAV_RATIOS[]	= "{RATIOS}";
static char T_SAV_TILEMAP[]	= "{TILEMAP}";
static char T_SAV_ACTORMAP[]	= "{ACTORMAP}";
static char T_SAV_AREAS[]	= "{AREAS}";
static char T_SAV_OBJECTS[]	= "{OBJECTS}";
static char T_SAV_STATICS[]	= "{STATICS}";
static char T_SAV_DOORS[]	= "{DOORS}";
static char T_SAV_PWALL[]	= "{PWALL}";
static char T_SAV_VISMAP[]	= "{VISMAP}";

static char T_CFG_SCORES[]	= "{SCORES}";
static char T_CFG_VIEWSIZE[]	= "{VIEWSIZE}";

static char T_DONE[]		= "{EOD}";

enum { T_LEN_LIMIT = 16 };

#define	ReadUInt8(fd)	((uint8_t) ReadInt8(fd))
#define	ReadUInt16(fd)	((uint16_t) ReadInt16(fd))

/*
 * Default scores
 */

HighScore	Scores[MaxScores] = {
	{ "id software-'92",	10000,	1 },
	{ "Adrian Carmack",		10000,	1 },
	{ "John Carmack",			10000,	1 },
	{ "Kevin Cloud",			10000,	1 },
	{ "Tom Hall",					10000,	1 },
	{ "John Romero",			10000,	1 },
	{ "Jay Wilbur",				10000,	1 },
	};

/*
========================
=
= FixedByFrac (FixedMul)
=
= multiply two 16/16 bit, 2's complement fixed point numbers
=
========================
*/

fixed FixedByFrac(fixed a, fixed b)
{
	return (fixed) (((int64_t) a * (int64_t) b) >> TILESHIFT);
}

/*
=====================
=
= CalcTics
=
=====================
*/

void CalcTics()
{
	int newtime;
	int ticcount;
	
	if (demoplayback || demorecord)
		ticcount = DEMOTICS - 1; /* [70/4] 17.5 Hz */
	else
		ticcount = 0 + 1; /* 35 Hz */

	wait_Ticks(ticcount);
	newtime = get_TimeCount();
	tics = newtime - lasttimecount;
	lasttimecount = newtime;
	
	if (demoplayback || demorecord)
		tics = DEMOTICS;
	else if (tics > MAXTICS)
		tics = MAXTICS;
}

/* ======================================================================== */

int CheckBytes (int fd, byte *buf, byte *check, unsigned len) {
ReadBytes(fd, buf, len);
return strncmp(buf, check, len);
}	// CheckBytes

static void WriteTag (int fd, char *tag) {
WriteBytes (fd, tag, strlen(tag));
}	// WriteTag

static void VerifyTag(int fd, char *tag) {
byte check [T_LEN_LIMIT];
unsigned len = strlen(tag);

if (len >= T_LEN_LIMIT) return;

ReadBytes (fd, check, len);
if (strncmp (tag, check, len))
	Quit ("Tag check %s failed!", tag);
}	// VerifyTag

static void DiskFlopAnim(int x, int y)
{
	static char which = 0;
	
	if (!x && !y)
		return;

	VWB_DrawPicSync(x, y, C_DISKLOADING1PIC+which);

	which ^= 1;
}

static int32_t CalcFileChecksum(int fd, int len)
{
	int32_t cs;
	int i;
	int8_t c1, c2;
	
	c1 = ReadInt8(fd);
	cs = 0;
	for (i = 0; i < len - 1; i++) {
		c2 = ReadInt8(fd);
		cs += c1 ^ c2;
		c1 = c2;
	}
	return cs;
}

int WriteConfig()
{
	int i;
	int fd;
	int32_t cs;
	char *fname = resource_name(configname, false);
	
	fd = OpenWrite(fname);

	if (fd != -1) {
		WriteBytes(fd, GAMEHDR, GAMEHDRLEN);
		WriteBytes(fd, CFGTYPE, CFGTYPELEN);
		WriteInt32(fd, ENGINE_VER);
		WriteBytes(fd, GAMELBL, LBLLEN);
		WriteInt32(fd, time(NULL));		/* Time */
		WriteInt32(fd, 0x00000000);		/* Padding */
		WriteInt32(fd, 0x00000000);		/* Checksum (placeholder) */

		WriteTag (fd, T_CFG_SCORES);
		for (i = 0; i != MaxScores; i++) {
			WriteBytes(fd, (byte *)Scores[i].name, MaxHighName+1);
			WriteInt32(fd, Scores[i].score);
			WriteInt32(fd, Scores[i].completed);
			WriteInt32(fd, Scores[i].episode);
		}

		WriteTag (fd, T_CFG_VIEWSIZE);
		WriteInt32(fd, viewsize);

		WriteTag (fd, T_DONE);
		CloseWrite(fd);

		fd = OpenRead(fname);
		ReadSeek(fd, 32, SEEK_SET);
		cs = CalcFileChecksum(fd, ReadLength(fd) - 32);
		CloseRead(fd);

		fd = OpenWriteAppend(fname);
		WriteSeek(fd, 28, SEEK_SET);
		WriteInt32(fd, cs);
		
		CloseWrite(fd);
	}
	
	return 0;
}

static void SetDefaults()
{
	viewsize = 15;
}

int ReadConfig()
{
	int fd, configokay;
	byte buf[8];
	int32_t v, v1;
	int i;
	
	configokay = 0;
	
	fd = OpenRead(resource_name(configname, false));

	if (fd != -1) {
		if (CheckBytes(fd, buf, GAMEHDR, GAMEHDRLEN))
			goto configend;

		if (CheckBytes(fd, buf, CFGTYPE, CFGTYPELEN))
			goto configend;

		v = ReadInt32(fd);
		if (v != ENGINE_VER)
			goto configend;

		if (CheckBytes(fd, buf, GAMELBL, LBLLEN))
			goto configend;

		ReadInt32(fd);	/* skip over time */
		ReadInt32(fd);	/* skip over padding */

		v = ReadInt32(fd);	/* get checksum */
		v1 = CalcFileChecksum(fd, ReadLength(fd) - 32);
		if (v != v1)
			goto configend;

		ReadSeek(fd, 32, SEEK_SET);

		VerifyTag (fd, T_CFG_SCORES);
		for (i = 0; i != MaxScores; i ++) {
			ReadBytes(fd, (byte *)Scores[i].name, MaxHighName+1);
			Scores[i].score = ReadInt32(fd);
			Scores[i].completed = ReadInt32(fd);
			Scores[i].episode = ReadInt32(fd);
		}
		
		VerifyTag (fd, T_CFG_VIEWSIZE);
		viewsize = ReadInt32(fd);

		VerifyTag (fd, T_DONE);

		if (is_game == GAME_WL1) {
		MainMenu[readthis].active = 1;
		MainItems.curpos = 0;
		}

		configokay = 1;
	} 
	
configend:	
	
	if (fd != -1)
		CloseRead(fd);
	
	if (!configokay) {
		printf("Config: Setting defaults..\n");
		SetDefaults();
	}
	
	mouseenabled = false;

	joystickenabled = false;
	joypadenabled = false;
	joystickport = 0;

	mouseadjustment = 5;

	SD_SetMusicMode(smm_On);
	SD_SetSoundMode(sdm_On);
	SD_SetDigiDevice(sds_On);
	
	return 0;
}

// End of actor list
#define	EOL		0xFFFF

int SaveTheGame(char *fn, char *tag, int dx, int dy)
{
	objtype *ob;
	int fd, i, x, y;
	int32_t cs;
	char *fname = resource_name (fn, false);

	unsigned n_statics;

	fd = OpenWrite(fname);

	if (fd != -1) {
		WriteBytes(fd, GAMEHDR, GAMEHDRLEN);
		WriteBytes(fd, SAVTYPE, SAVTYPELEN);
		WriteInt32(fd, ENGINE_VER);
		WriteBytes(fd, GAMELBL, LBLLEN);

		WriteInt32(fd, time(NULL));
		WriteInt32(fd, 0x00000000);

		WriteInt32(fd, 0x00000000);	/* write checksum (placeholder) */

		WriteBytes(fd, tag, 32);	/* write savegame name */

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_GSTATE);
		WriteInt8(fd, gamestate.difficulty);
		WriteInt8(fd, gamestate.mapon);
		WriteInt32(fd, gamestate.oldscore);
		WriteInt32(fd, gamestate.score);
		WriteInt32(fd, gamestate.nextextra);
		WriteInt8(fd, gamestate.lives);
		WriteInt8(fd, gamestate.health);
		WriteInt8(fd, gamestate.ammo);
		WriteInt8(fd, gamestate.keys);
		WriteInt8(fd, gamestate.weaponflag);
		WriteInt8(fd, gamestate.weapon);
		WriteInt8(fd, gamestate.chosenweapon);
		WriteInt16(fd, gamestate.faceframe);
		WriteInt16(fd, gamestate.attackframe);
		WriteInt16(fd, gamestate.attackcount);
		WriteInt16(fd, gamestate.weaponframe);
		WriteInt8(fd, gamestate.episode);
		WriteInt16(fd, gamestate.secretcount);
		WriteInt16(fd, gamestate.treasurecount);
		WriteInt16(fd, gamestate.killcount);
		WriteInt16(fd, gamestate.secrettotal);
		WriteInt16(fd, gamestate.treasuretotal);
		WriteInt16(fd, gamestate.killtotal);
		WriteInt32(fd, gamestate.TimeCount);
		WriteInt32(fd, gamestate.killx);
		WriteInt32(fd, gamestate.killy);
		WriteInt8(fd, gamestate.victoryflag);

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_RATIOS);
		for (i = 0; i != game_levels; i++) {
			WriteInt16(fd, LevelRatios[i].kill);
			WriteInt16(fd, LevelRatios[i].secret);
			WriteInt16(fd, LevelRatios[i].treasure);
			WriteInt32(fd, LevelRatios[i].time);
		}

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_TILEMAP);
		WriteBytes(fd, (byte *) tilemap, MAPWDT*MAPHGT);

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_ACTORMAP);
		for (x = 0; x != MAPWDT; x++)
			for (y = 0; y != MAPHGT; y++)
				WriteInt16(fd, actorat[x][y]);

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_AREAS);
		WriteBytes(fd, (byte *)areaconnect, NUMAREAS * NUMAREAS);

		DiskFlopAnim(dx, dy);

		for (i = 0; i != NUMAREAS; i++)
			WriteInt8(fd, areabyplayer[i]);

		WriteTag (fd, T_SAV_OBJECTS);
		for (ob = player; ob; ob = ob->next) {
			DiskFlopAnim(dx, dy);

			WriteInt16(fd, ob->id);
			WriteInt8(fd, ob->active);
			WriteInt32(fd, ob->ticcount);
			WriteInt8(fd, ob->obclass);
			WriteInt16(fd, ob->state);
			WriteInt8(fd,  ob->flags);
			WriteInt32(fd, ob->distance);
			WriteInt8(fd, ob->dir);
			WriteInt32(fd, ob->x);
			WriteInt32(fd, ob->y);
			WriteInt8(fd, ob->tilex);
			WriteInt8(fd, ob->tiley);
			WriteInt8(fd,  ob->areanumber);
			WriteInt32(fd, ob->viewx);
			WriteInt32(fd, ob->viewheight);
			WriteInt32(fd, ob->transx);
			WriteInt32(fd, ob->transy);
			WriteInt16(fd, ob->angle);
			WriteInt16(fd, ob->hitpoints);
			WriteInt16(fd, ob->speed);
			WriteInt32(fd, ob->temp1);
			WriteInt32(fd, ob->temp2);
			WriteInt32(fd, ob->temp3);
		}

		WriteInt16(fd, EOL);		/* end of actor list */

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_STATICS);
		n_statics = laststatobj - statobjlist;
		WriteInt16(fd, n_statics);	/* ptr offset */

		for (i = 0; i != n_statics; i ++) {
			WriteInt8(fd,  statobjlist[i].tilex);
			WriteInt8(fd,  statobjlist[i].tiley);
			WriteInt32(fd, statobjlist[i].shapenum);
			WriteInt8(fd,  statobjlist[i].flags);
			WriteInt8(fd,  statobjlist[i].itemnumber);
		}

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_DOORS);
		WriteInt8(fd, doornum);
		for (i = 0; i != doornum; i++) {
			WriteInt16(fd, doorposition[i]);
			WriteInt8(fd,  doorobjlist[i].tilex);
			WriteInt8(fd,  doorobjlist[i].tiley);
			WriteInt8(fd,  doorobjlist[i].vertical);
			WriteInt8(fd,  doorobjlist[i].lock);
			WriteInt8(fd,  doorobjlist[i].action);
			WriteInt32(fd, doorobjlist[i].ticcount);
		}

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_PWALL);
		WriteInt32(fd, pwallstate);
		WriteInt8(fd, pwallx);
		WriteInt8(fd, pwally);
		WriteInt8(fd, pwalldir);
		WriteInt8(fd, pwallpos);

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_SAV_VISMAP);

		for (y = 0; y != MAPHGT; y++)
		    for (x = 0; x < MAPWDT; x += 8) {
			byte vdata = 0;
			for (i = 0; i != 8; ++ i)
			    if (! (map_dir[x+i][y] & MP_HID)) vdata |= 1 << i;
			WriteInt8(fd, vdata);
			}

		DiskFlopAnim(dx, dy);

		WriteTag (fd, T_DONE);

		CloseWrite(fd);

		fd = OpenRead(fname);
		ReadSeek(fd, 64, SEEK_SET);
		cs = CalcFileChecksum(fd, ReadLength(fd) - 64);
		CloseRead(fd);
		
		fd = OpenWriteAppend(fname);
		WriteSeek(fd, 28, SEEK_SET);
		WriteInt32(fd, cs);

		CloseWrite(fd);
	} else {
		Message(STR_NOSPACE1 "\n" STR_NOSPACE2);

		IN_ClearKeysDown();
		IN_Ack();

		return -1;
	}
	
	return 0;
}

int ReadSaveTag(char *fn, char *tag)
{
	byte buf[8];
	int fd;
	int32_t v;

	fd = OpenRead(resource_name (fn, false));
	if (fd == -1)
		goto rstfail;

	if (CheckBytes(fd, buf, GAMEHDR, GAMEHDRLEN))
		goto rstfail;

	if (CheckBytes(fd, buf, SAVTYPE, SAVTYPELEN))
		goto rstfail;

	v = ReadInt32(fd);
	if (v != ENGINE_VER)
		goto rstfail;

	if (CheckBytes(fd, buf, GAMELBL, LBLLEN))
		goto rstfail;

	ReadInt32(fd);
	ReadInt32(fd);

	v = ReadInt32(fd); /* get checksum */

	ReadSeek(fd, 64, SEEK_SET);
	if (v != CalcFileChecksum(fd, ReadLength(fd) - 64))
		goto rstfail;

	ReadSeek(fd, 32, SEEK_SET);
	ReadBytes(fd, (byte *)tag, 32);

	CloseRead(fd);

	return 0;

rstfail:
	if (fd != -1)
		CloseRead(fd);

	return -1;
}

int LoadTheGame(char *fn, int dx, int dy)
{
	byte buf[8];
	int fd, i, x, y, id;
	int32_t version, cs;
	unsigned n_statics;

	fd = OpenRead(resource_name (fn, false));

	if (fd == -1)
		goto loadfail;

	if (CheckBytes(fd, buf, GAMEHDR, GAMEHDRLEN))
		goto loadfail;

	if (CheckBytes(fd, buf, SAVTYPE, SAVTYPELEN))
		goto loadfail;

	version = ReadInt32(fd);
	if (version != ENGINE_VER)
		goto loadfail;

	if (CheckBytes(fd, buf, GAMELBL, LBLLEN))
		goto loadfail;

	ReadInt32(fd);		// time
	ReadInt32(fd);		// zero record (?)

	cs = ReadInt32(fd);	/* get checksum */

	ReadSeek(fd, 64, SEEK_SET);
	if (cs != CalcFileChecksum(fd, ReadLength(fd) - 64))
		goto loadfail;

	ReadSeek(fd, 64, SEEK_SET);

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_GSTATE);
	gamestate.difficulty	= ReadUInt8(fd);
	gamestate.mapon		= ReadUInt8(fd);
	gamestate.oldscore	= ReadInt32(fd);
	gamestate.score		= ReadInt32(fd);
	gamestate.nextextra	= ReadInt32(fd);
	gamestate.lives		= ReadUInt8(fd);
	gamestate.health	= ReadUInt8(fd);
	gamestate.ammo		= ReadUInt8(fd);
	gamestate.keys		= ReadUInt8(fd);
	gamestate.weaponflag	= ReadUInt8(fd);
	gamestate.weapon	= ReadUInt8(fd);
	gamestate.chosenweapon	= ReadUInt8(fd);
	gamestate.faceframe	= ReadUInt16(fd);
	gamestate.attackframe	= ReadUInt16(fd);
	gamestate.attackcount	= ReadUInt16(fd);
	gamestate.weaponframe	= ReadUInt16(fd);
	gamestate.episode	= ReadUInt8(fd);
	gamestate.secretcount	= ReadUInt16(fd);
	gamestate.treasurecount	= ReadUInt16(fd);
	gamestate.killcount	= ReadUInt16(fd);
	gamestate.secrettotal	= ReadUInt16(fd);
	gamestate.treasuretotal = ReadUInt16(fd);
	gamestate.killtotal	= ReadUInt16(fd);
	gamestate.TimeCount	= ReadInt32(fd);
	gamestate.killx		= ReadInt32(fd);
	gamestate.killy		= ReadInt32(fd);
	gamestate.victoryflag	= ReadUInt8(fd);

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_RATIOS);
	for (i = 0; i != game_levels; i++) {
		LevelRatios[i].kill	= ReadUInt16(fd);
		LevelRatios[i].secret	= ReadUInt16(fd);
		LevelRatios[i].treasure	= ReadUInt16(fd);
		LevelRatios[i].time	= ReadInt32(fd);
	}

	DiskFlopAnim(dx, dy);

	SetupGameLevel();

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_TILEMAP);
	ReadBytes(fd, (byte *) tilemap, MAPWDT*MAPHGT);

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_ACTORMAP);
	for (x = 0; x != MAPWDT; x++)
		for (y = 0; y != MAPHGT; y++)
			actorat[x][y] = ReadUInt16(fd);

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_AREAS);
	ReadBytes(fd, (byte *)areaconnect, NUMAREAS * NUMAREAS);
	
	DiskFlopAnim(dx, dy);
	
	for (i = 0; i != NUMAREAS; i ++)
		areabyplayer[i] = ReadUInt8(fd);

	DiskFlopAnim(dx, dy);

	InitActorList();

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_OBJECTS);

	/* player ptr already set up */
	id			= ReadUInt16(fd); /* get id */
	player->active		= ReadUInt8(fd);
	player->ticcount	= ReadInt32(fd);
	player->obclass		= ReadUInt8(fd);
	player->state		= ReadUInt16(fd);
	player->flags		= ReadUInt8(fd);
	player->distance	= ReadInt32(fd);
	player->dir		= ReadUInt8(fd);
	player->x		= ReadInt32(fd);
	player->y		= ReadInt32(fd);
	player->tilex		= ReadUInt8(fd);
	player->tiley		= ReadUInt8(fd);
	player->areanumber	= ReadUInt8(fd);
	player->viewx		= ReadInt32(fd);
	player->viewheight	= ReadInt32(fd);
	player->transx		= ReadInt32(fd);
	player->transy		= ReadInt32(fd);
	player->angle		= ReadUInt16(fd);
	player->hitpoints	= ReadUInt16(fd);
	player->speed		= ReadUInt16(fd);
	player->temp1		= ReadInt32(fd);
	player->temp2		= ReadInt32(fd);
	player->temp3		= ReadInt32(fd);

	/* update the id */
	for (x = 0; x != MAPWDT; x++)
		for (y = 0; y != MAPHGT; y++)
			if (actorat[x][y] == (id | AM_OBJID))
				actorat[x][y] = player->id | AM_OBJID;

	while (1) {
		DiskFlopAnim(dx, dy);

		id			= ReadUInt16(fd);

		if (id == EOL)	 /* end of actor list */
			break;

		GetNewActor();
		
		new->active		= ReadUInt8(fd);
		new->ticcount		= ReadInt32(fd);
		new->obclass		= ReadUInt8(fd);
		new->state		= ReadUInt16(fd);
		new->flags		= ReadUInt8(fd);
		new->distance		= ReadInt32(fd);
		new->dir		= ReadUInt8(fd);
		new->x			= ReadInt32(fd);
		new->y			= ReadInt32(fd);
		new->tilex		= ReadUInt8(fd);
		new->tiley		= ReadUInt8(fd);
		new->areanumber		= ReadUInt8(fd);
		new->viewx		= ReadInt32(fd);
		new->viewheight		= ReadInt32(fd);
		new->transx		= ReadInt32(fd);
		new->transy		= ReadInt32(fd);
		new->angle		= ReadUInt16(fd);
		new->hitpoints		= ReadUInt16(fd);
		new->speed		= ReadUInt16(fd);
		new->temp1		= ReadInt32(fd);
		new->temp2		= ReadInt32(fd);
		new->temp3		= ReadInt32(fd);

		for (x = 0; x != MAPWDT; x ++)
			for (y = 0; y != MAPHGT; y ++)
				if (actorat[x][y] == (id | AM_OBJID))
					actorat[x][y] = new->id | AM_OBJID;
	}

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_STATICS);
	n_statics = ReadUInt16(fd);
	laststatobj = statobjlist + n_statics;

	for (i = 0; i != n_statics; i ++) {
		int tilex = statobjlist[i].tilex = ReadUInt8(fd);
		int tiley = statobjlist[i].tiley = ReadUInt8(fd);
		statobjlist[i].shapenum		= ReadInt32(fd);
		statobjlist[i].flags		= ReadUInt8(fd);
		statobjlist[i].itemnumber	= ReadUInt8(fd);
		statobjlist[i].visspot 		= &spotvis[tilex][tiley];
		}

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_SAV_DOORS);

	// TODO: open doors must be placed on vismap...
	doornum = ReadUInt8(fd);
	for (i = 0; i != doornum; i++) {
		doorposition[i] 	= ReadUInt16(fd);
		doorobjlist[i].tilex	= ReadUInt8(fd);
		doorobjlist[i].tiley	= ReadUInt8(fd);
		doorobjlist[i].vertical = ReadUInt8(fd);
		doorobjlist[i].lock	= ReadUInt8(fd);
		doorobjlist[i].action	= ReadUInt8(fd);
		doorobjlist[i].ticcount	= ReadInt32(fd);
	}

	DiskFlopAnim(dx, dy);

	// TODO: pushwall must be placed on vismap...
	VerifyTag (fd, T_SAV_PWALL);
	pwallstate 	= ReadInt32(fd);
	pwallx		= ReadUInt8(fd);
	pwally		= ReadUInt8(fd);
	pwalldir	= ReadUInt8(fd);
	pwallpos	= ReadUInt8(fd);

	DiskFlopAnim(dx, dy);

	MapViewInit ();

	VerifyTag (fd, T_SAV_VISMAP);

	for (y = 0; y != MAPHGT; y++)
	    for (x = 0; x < MAPWDT; x += 8) {
		byte vdata = ReadInt8(fd);
		for (i = 0; i != 8; ++ i)
		    if (vdata & (1<<i)) map_dir[x+i][y] &= ~MP_HID;
		}

	DiskFlopAnim(dx, dy);

	VerifyTag (fd, T_DONE);
	CloseRead(fd);

	return 0;

loadfail:
	if (fd != -1)
		CloseRead(fd);
		
	Message(STR_SAVECHT1"\n"
		STR_SAVECHT2"\n"
		STR_SAVECHT3"\n"
		STR_SAVECHT4);
			
	IN_ClearKeysDown();
	IN_Ack();
	
	NewGame(1, 0);
	return -1;
}

/* ======================================================================== */

/*
==================
=
= BuildTables
=
==================
*/

static const double radtoint = (double)FINEANGLES/2.0/PI;

void BuildTables()
{
	int i;
	double tang, angle, anglestep;
	fixed value;

/* calculate fine tangents */

	finetangent[0] = 0;
	for (i = 1; i < FINEANGLES/8; i++) {
		tang = tan((double)i/radtoint);
		finetangent[i] = tang*TILEGLOBAL;
		finetangent[FINEANGLES/4-1-i] = TILEGLOBAL/tang;
	}
	
	/* fight off asymptotic behaviour at 90 degrees */
	finetangent[FINEANGLES/4-1] = finetangent[FINEANGLES/4-2]+1;
	
//
// costable overlays sintable with a quarter phase shift
// ANGLES is assumed to be divisable by four
//

	angle = 0.0;
	anglestep = PI/2.0/ANGLEQUAD;
	for (i = 0; i <= ANGLEQUAD; i++) {
		value = GLOBAL1*sin(angle);
		
		sintable[i] = 
		sintable[i+ANGLES] =  
		sintable[ANGLES/2-i] = value;
		
		sintable[ANGLES-i] =
		sintable[ANGLES/2+i] = -value;
		
		angle += anglestep;
	}
}

/*
===================
=
= SetupWalls
=
= Map tile values to scaled pics
=
===================
*/

void SetupWalls()
{
	int i;

	for (i=1;i<MAXWALLTILES;i++)
	{
		horizwall[i]=(i-1)*2;
		vertwall[i]=(i-1)*2+1;
	}
}

void ClearView () {
VL_Bar(xoffset, yoffset, viewwidth, viewheight, 0);
}	// ClearView

void ShowViewSize(int width)
{
	int oldwidth,oldheight;

	oldwidth = viewwidthwin;
	oldheight = viewheightwin;

	viewwidthwin = width*16;
	viewheightwin = width*16*HEIGHTRATIO;
	ClearView();
	DrawPlayBorder();

	viewheightwin = oldheight;
	viewwidthwin = oldwidth;
}

void NewViewSize(int width)
{
	if (width > MAXVIEWSIZE)
		width = MAXVIEWSIZE;
	if (width < MINVIEWSIZE)
		width = MINVIEWSIZE;

	width *= vwidth / ScrWdt;

	if ((width*16) > vwidth)
		width = vwidth / 16;

	if ((width*16*HEIGHTRATIO) > (vheight - 40*vheight/ScrHgt))
		width = (vheight - 40*vheight/ScrHgt)/8;

	viewwidthwin = width*16*ScrWdt/vwidth;
	viewheightwin = width*16*HEIGHTRATIO*ScrWdt/vwidth;
	viewsize = width*ScrWdt/vwidth;

	viewwidth = width*16;
	viewheight = width*16*HEIGHTRATIO;

	centerx = viewwidth/2-1;
	shootdelta = viewwidth/10;

	yoffset = (vheight-STATUSLINES*vheight/ScrHgt-viewheight)/2;
	xoffset = (vwidth-viewwidth)/2;

//
// calculate trace angles and projection constants
//
	CalcProjection(FOCALLENGTH);

}

//===========================================================================

/* ======================================================================== */

/*
==========================
=
= ShutdownId
=
= Shuts down all ID_?? managers
=
==========================
*/

void ShutdownId()
{
	US_Shutdown();
	SD_Shutdown();
	IN_Shutdown();
	VW_Shutdown();
	CA_Shutdown();
	PM_Shutdown();
	MM_Shutdown();
}

/*
=====================
=
= NewGame
=
= Set up new game to start from the beginning
=
=====================
*/

void NewGame(int difficulty, int episode)
{
	memset(&gamestate, 0, sizeof(gamestate));
	
	gamestate.difficulty = difficulty;
	gamestate.weapon = gamestate.chosenweapon = wp_pistol;
	gamestate.weaponflag = (1 << wp_knife) | (1 << wp_pistol);
	gamestate.health = MAX_HEALTH;
	gamestate.ammo = STARTAMMO;
	gamestate.lives = 3;
	gamestate.nextextra = EXTRAPOINTS;
	gamestate.episode = episode;
	gamestate.mapon = 0;

	startgame = true;
}

char *GameCaption (boolean flag);

/*
==========================
=
= InitGame
=
= Load a few things right away
=
==========================
*/

void InitGame()
{
	int i;

	MM_Startup(); 
	PM_Startup();
	CA_Startup();
	VW_Startup();
	IN_Startup();
	SD_Startup();
	US_Startup();
	
//
// build some tables
//

	for (i = 0; i < MAPHGT; i++)
	{
		farmapylookup[i] = i*64;
	}

	ReadConfig();

/* set game caption */
VL_SetCaption(GameCaption(false));

/* load in and lock down some basic chunks */

	CA_CacheGrChunk(STARTFONT);
	CA_CacheGrChunk(TILE8);

	CacheGrLump (LATCHPICS_LUMP_START, LATCHPICS_LUMP_END);

	BuildTables();
	SetupWalls();

	NewViewSize(viewsize);

//
// initialize variables
//
	InitRedShifts();

	IN_CheckAck();
}

int playlevel = -1;

/*
=====================
=
= DemoLoop
=
=====================
*/

boolean demotest;			// (test demo playback)
char *demofile;				// (file to play demo from)
int no_demos;				// (# of demos)

void DemoLoop()
{
	static int LastDemo;
	
	int i;

//
// check: play selected level
//
	if (playlevel >= 0) {

		NoWait = true;
		NewGame(1,0);

		gamestate.difficulty = 2;

		SetMapIndex (playlevel);

		GameLoop();

		Quit ("Test play complete");
		}

//
// main game cycle
//

	LastDemo = 0;
	
	StartCPMusic(INTROSONG);

	if (!NoWait)
		PG13();

	if (demofile) {
	IN_ClearKeysDown();
	if (PlayDemoFromFile(demofile)) {
		IN_UserInput(3 * 70);
		VW_FadeOut();
		}
	}


	if (demotest)
	    for (i = 0; i < no_demos; i++)
		PlayDemo(i);

	while (1)
	{
		while (!NoWait)
		{
//
// title page
//
		if (GAME_SPEAR) {
			CA_CacheGrChunk (TITLEPALETTE);

			CA_CacheGrChunk (TITLE1PIC);
			VWB_DrawPic (0,0,TITLE1PIC);
			CA_UnCacheGrChunk (TITLE1PIC);

			CA_CacheGrChunk (TITLE2PIC);
			VWB_DrawPic (0,80,TITLE2PIC);
			CA_UnCacheGrChunk(TITLE2PIC);
			VW_UpdateScreen();
			VL_FadeIn(0,255,CA_GetGrChunk(TITLEPALETTE),30);

			CA_UnCacheGrChunk (TITLEPALETTE);
		} else {
			VL_CacheScreen(TITLEPIC);
			VW_UpdateScreen ();
			VW_FadeIn();
		}

			if (IN_UserInput(TickBase*15))
				break;
			VW_FadeOut();
//
// credits page
//
			VL_CacheScreen(CREDITSPIC);
			VW_UpdateScreen();
			VW_FadeIn ();
			if (IN_UserInput(TickBase*10))
				break;
			VW_FadeOut ();
//
// high scores
//
			DrawHighScores();
			VW_UpdateScreen();
			VW_FadeIn();

			if (IN_UserInput(TickBase*10))
				break;
//
// demo
//
			PlayDemo(LastDemo++%no_demos);

			if (playstate == ex_abort)
				break;
			StartCPMusic(INTROSONG);
		}

		VW_FadeOut();

		// TODO -- special option for demo recording...
		if (IN_KeyDown(sc_Tab) && demorecord)
			RecordDemo();
		else
			US_ControlPanel(0);

		if (startgame || loadedgame)
		{
			GameLoop();
			VW_FadeOut();
			StartCPMusic(INTROSONG);
		}
	}
}

/*
 *	Game specific values
 */

enum { NO_ENTRY = 0xffff };

//
// Data LUMPs
//

// (Wolfenstein)

word README_LUMP_START, README_LUMP_END;
word CONTROLS_LUMP_START, CONTROLS_LUMP_END;

// (Spear Of Destiny)

word BACKDROP_LUMP_START, BACKDROP_LUMP_END;
word CONTROL_LUMP_START, CONTROL_LUMP_END;
word OPTIONS_LUMP_START, OPTIONS_LUMP_END;
word SOUND_LUMP_START, SOUND_LUMP_END;
word NEWGAME_LUMP_START, NEWGAME_LUMP_END;
word LOADSAVE_LUMP_START, LOADSAVE_LUMP_END;
word HIGHSCORES_LUMP_START, HIGHSCORES_LUMP_END;
word ENDGAME_LUMP_START, ENDGAME_LUMP_END;
word LEVELEND_LUMP_START, LEVELEND_LUMP_END;
word TITLESCREEN_LUMP_START, TITLESCREEN_LUMP_END;
word EASTEREGG_LUMP_START, EASTEREGG_LUMP_END;
word COPYPROT_LUMP_START, COPYPROT_LUMP_END;
word LATCHPICS_LUMP_START, LATCHPICS_LUMP_END;

// Audio offsets

word STARTADLIBSOUNDS;
word STARTMUSIC;

word NUMSOUNDS;
word NUMSNDCHUNKS;

//
// Amount of each data item
//
word NUMGRCHUNKS, NUMPICS, NUMEXTERNS;

//
// File offsets for data items
//

word STARTEXTERNS;

//
// Index map arrays
//

word GFX_RSM [GFX_END - GFX_BEG];
word SND_RSM [SND_END - SND_BEG];
word MUS_RSM [MUS_END - MUS_BEG];

//
//	Initialise appropriate graphics
//

void init_GFX (GFX_Init *Init_GFX) {
if (Init_GFX) {
unsigned i;

// (setup some globals)

NUMGRCHUNKS	= Init_GFX->N_GRCHUNKS;
NUMPICS		= Init_GFX->N_PICS;
NUMEXTERNS	= Init_GFX->N_EXTERNS;

// (setup graphics map)

for (i = 0; i != GFX_END-GFX_BEG; ++ i)
	GFX_RSM[i] = NO_ENTRY;

uint16_t *Order_GFX = Init_GFX->Order_GFX;
unsigned Order_GFX_len = Init_GFX->Order_GFX_len;

for (i = 0; i != Order_GFX_len; ++ i) {
    unsigned index = Order_GFX[i];
    if (GFX_BEG <= index && index < GFX_END)
	GFX_RSM [index-GFX_BEG] = i;
    }

// (setup lump values)

AssignWord *Values_GFX = Init_GFX->Values_GFX;
unsigned Values_GFX_len = Init_GFX->Values_GFX_len;

for (; Values_GFX_len --; Values_GFX ++)
	* Values_GFX->dest = Values_GFX->value;

}	// (Init_GFX)
}	// init_GFX

//
//	Initialise appropriate sounds & music
//

void init_AUD (AUD_Init *Init_AUD) {
if (Init_AUD) {
unsigned i;

// (setup some globals)

NUMSOUNDS = STARTADLIBSOUNDS = Init_AUD->N_SOUNDS;
NUMSNDCHUNKS = Init_AUD->N_SNDCHUNKS;
STARTMUSIC = Init_AUD->STARTMUSIC;

// (setup sounds map)

for (i = 0; i != SND_END-SND_BEG; ++ i)
	SND_RSM[i] = NO_ENTRY;

uint16_t *Order_SND = Init_AUD->Order_SND;
unsigned Order_SND_len = Init_AUD->Order_SND_len;

for (i = 0; i != Order_SND_len; ++ i) {
    unsigned index = Order_SND[i];
    if (SND_BEG <= index && index < SND_END)
	SND_RSM [index-SND_BEG] = i;
    }

// (setup music map)

for (i = 0; i != MUS_END-MUS_BEG; ++ i)
	MUS_RSM[i] = NO_ENTRY;

uint16_t *Order_MUS = Init_AUD->Order_MUS;
unsigned Order_MUS_len = Init_AUD->Order_MUS_len;

for (i = 0; i != Order_MUS_len; ++ i) {
    unsigned index = Order_MUS[i];
    if (MUS_BEG <= index && index < MUS_END)
	MUS_RSM [index-MUS_BEG] = i;
    }

}	// (Init_AUD)
}	// init_AUD

void MapSpritePages ();
void MapSoundPages ();

/* ======================================================================== */


// Type of game being played
enum GAME_TYPE is_game;

int game_levels;

unsigned char *gamepal;

struct GAME_CONFIG {
	enum GAME_TYPE type;
	char *tag;
	char *name;
	char *ext;
	char *label;
	} config_data[] = {

	{	GAME_WL1, "WL1",
		"Wolfenstein 3D Shareware",
		"wl1", "WL1\0"
	},

	{	GAME_WL3, "WL3",
		"Wolfenstein 3D (3 levels)",
		"wl3", "WL3\0"
	},

	{	GAME_WL6, "WL6",
		"Wolfenstein 3D",
		"wl6", "WL6\0"
	},

	{	GAME_SDM, "SDM",
		"Spear of Destiny Demo",
		"sdm", "SDM\0"
	},

	{	GAME_SOD, "SOD",
		"Spear of Destiny",
		"sod", "SOD\0"
	},

	{	GAME_SD2, "SD2",
		"SOD Mission 2: Return to Danger",
		"sd2", "SD2\0"
	},

	{	GAME_SD3, "SD3",
		"SOD Mission 3: Ultimate Challenge",
		"sd3", "SD3\0"
	}

	};

#define	TOTAL_GAMES	(sizeof(config_data) / sizeof(struct GAME_CONFIG))

static char *game_name = 0;		// (symbolic game name)

boolean dump_music = false;

void ListGames () {
int i;

printf ("The following game types are supported:\n");

for (i = 0; i != TOTAL_GAMES; ++ i) {
    printf ("[%s]: %s\n",
	config_data[i].tag, config_data[i].name);
    }

}	// ListGames

// Path to resources
char *path_resource = 0;

// Path to user data
char *path_user = 0;

//
// Game options
//

boolean alt_gfx = false;		// (alternate graphic file, WL6)

struct GAME_OPT {
	char *optname;
	enum {	OPT_BOOL,
		OPT_NUM,
		OPT_STR
		} opttype;
	void *optptr;
	} options [] = {

	{ "gametype",	OPT_STR,	&game_name	},
	{ "diaglevel",	OPT_NUM,	&diag_level	},

	{ "altgfx",	OPT_BOOL,	&alt_gfx	},

	{ "pathres",	OPT_STR,	&path_resource	},
	{ "pathloc",	OPT_STR,	&path_user	},

	{ "nowait",	OPT_BOOL,	&NoWait		},
	{ "nojoy",	OPT_BOOL,	&checkjoys	},
	{ "nomouse",	OPT_BOOL,	&MousePresent	},

	{ "nomusic",	OPT_BOOL,	&MusicMode	},
	{ "nosound",	OPT_BOOL,	&SoundMode	},
	{ "nosfx",	OPT_BOOL,	&DigiMode	},

	{ "vscale",	OPT_NUM,	&vscale		},
	{ "fullscreen",	OPT_BOOL,	&fullscreen	},

	{ "playlevel",	OPT_NUM,	&playlevel	},

	{ "playdemo",	OPT_STR,	&demofile	},

	{ "demotest",	OPT_BOOL,	&demotest	},

	{ "demorecord",	OPT_BOOL,	&demorecord	},

	{ "dump_music",	OPT_BOOL,	&dump_music	},

	{ "version",	OPT_BOOL,	&showversion	},

	{ 0 }
	};


void ParseOptions (int ac, char **av) {
char *p;
while ((p = *av ++)) {
	struct GAME_OPT *optrec;

	if (*p != '-') continue;
	else ++p;

	for (optrec = options; optrec->optname; ++ optrec)
	    if (! strcmp (p, optrec->optname)) {
		void *optptr = optrec->optptr;

		switch (optrec->opttype) {
		    case OPT_BOOL:
			*(boolean *)optptr = ! *(boolean *)optptr;
			break;

		    case OPT_NUM:
			if (*av) *(int *)optptr = atoi(*av++);
			break;

		    case OPT_STR:
			if (*av) *(char **)optptr = *av++;
			break;
		    }

		break;
	    }
	}
}	// ParseOptions

void ShowOptions () {
struct GAME_OPT *optrec;

for (optrec = options; optrec->optname; ++ optrec) {
	void *optptr;
	printf ("%s=", optrec->optname);

	optptr = optrec->optptr;

	switch (optrec->opttype) {
	    case OPT_BOOL:
		printf ("%s", *(boolean *)optptr ? "true":"false");
		break;

	    case OPT_NUM:
		printf ("%i", *(int *)optptr);
		break;

	    case OPT_STR:
		printf ("%s", *(char **)optptr);
		break;
	    }

	putchar ('\n');
	}	// for
}	// ShowOptions

/*
==========================
=
= WolfMain
=
==========================
*/

void SetupMenu();		// (in "wl_menu.c")

static GFX_Init *get_GFX() {
    switch (is_game) {
	case GAME_SOD:
	case GAME_SD2:
	case GAME_SD3:	return &Init_GFX_SOD;

	case GAME_SDM:	return &Init_GFX_SDM;

	case GAME_WL3:
	case GAME_WL6:	return alt_gfx ? &Init_GFX_WL6A : &Init_GFX_WL6;

	case GAME_WL1:	return &Init_GFX_WL1;
	}

return 0;
}

static AUD_Init *get_AUD() {
return (GAME_SPEAR)? &Init_AUD_SOD : &Init_AUD_WL;
}

#define	VERSION "0.86"

#define PACKED __attribute__((packed))

typedef struct {
	byte reg;		// (OPL register)
	byte value;		// (OPL value)
	word delay;		// (delay)
} PACKED MusicRec;

typedef	struct {
	word length;		// (total length)
	MusicRec values[1];
} PACKED MusicGroup;

void DumpMusicData() {

char namebuf[80];

AUD_Init *aud_record = get_AUD();

uint16_t *aud_list = aud_record->Order_MUS;
unsigned aud_count = aud_record->Order_MUS_len, no;

printf ("Dumping music files...\n");

for (no = 0; no != aud_count; ++ no) {
	unsigned song = aud_list[no];
	CA_CacheMusicChunk(song);
	MusicGroup *music = (MusicGroup *) CA_GetMusicChunk(song);

	sprintf(namebuf, "%sMus%3u_%u", game_name, no, song);

	int fd = OpenWrite(namebuf);

	WriteBytes(fd, (byte *)music->values, music->length);
	
	CloseWrite(fd);

	CA_UnCacheMusicChunk(song);
	}
}	// DumpMusicData

int WolfMain(int argc, char *argv[])
{
	int i;

	printf ("LoneWolf engine: version %s\n(build %s %s)\n", VERSION, __DATE__, __TIME__);

	ParseOptions (argc, argv);
  if (showversion) {
  	ShowOptions();
		Quit(NULL);
		}

	if (game_name == NULL)
		Quit ("Game type must be specified: use 'gametype' option.");

	for (i = 0; i != TOTAL_GAMES; ++ i) {
	if (! strcmp (game_name, config_data[i].tag)) {
	is_game = config_data[i].type;
	GAMENAME = config_data[i].name;
	GAMEEXT = config_data[i].ext;
	GAMELBL = config_data[i].label;
	break;
	}
	}

	if (i == TOTAL_GAMES) {
	ListGames ();
	Quit ("Unknown game type [%s]!", game_name);
	}

	game_levels = GAME_SPEAR ? 20 : 8;

	no_demos = is_game == GAME_SDM ? 1 : 4;

	gamepal = GAME_SPEAR ? Palette_SPEAR : Palette_WOLF;

	strcpy(extension, GAMEEXT);

	printf("Game selected: [%s] (%s)\n", game_name, GAMENAME);

	init_GFX (get_GFX ());
	init_AUD (get_AUD ());

	MapSpritePages ();
	MapSoundPages ();

	SetupStatics();

	SetupMenu();

	InitGame();

	if (dump_music) DumpMusicData ();

	DemoLoop();

	Quit("Demo loop exited???");

	return 0;
}	/* WolfMain */

#include <stdarg.h>

/*
==========================
=
= Quit
=
==========================
*/

void Quit(char *error, ...)
{
	if (!error || !*error)
		WriteConfig();

	ShutdownId();

	if (error && *error) {
		va_list ap;

		fprintf(stderr, "Quit: ");

		va_start (ap, error);
		vfprintf(stderr, error, ap);
		va_end (ap);

		fputc ('\n', stderr);
		exit(EXIT_FAILURE);
 	}
	exit(EXIT_SUCCESS);
}	/* Quit */

/*
==========================
=
= Diag
=
==========================
*/

void Diag(unsigned level, char *msg, ...)
{
if (level <= diag_level) {
FILE *output = level ? stderr : stdout;

va_list ap;

if (level) fprintf(output, "<Diag %u>: ", level);

va_start (ap, msg);
vfprintf (output, msg, ap);
va_end (ap);

fputc ('\n', output);
}
}	/* Diag */

/*
==========================
=
= SDL_main
=
==========================
*/

int SDL_main (int argc, char **argv) {
return WolfMain (argc, argv);
}		// SDL_main

