
/*
 * +==========================================================
 * |	'wl_play.c':
 * |	Playloop & misc player control.
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"

/*
=============================================================================

						 GLOBAL VARIABLES

=============================================================================
*/

boolean		madenoise;		// true when shooting or screaming

exit_t		playstate;

int		DebugOk;

objtype 	objlist[MAXACTORS],*new,*obj,*player,*lastobj,
			*objfreelist,*killerobj;

unsigned	farmapylookup[MAPHGT];

boolean		singlestep,godmode,noclip;

byte		tilemap[MAPWDT][MAPHGT];	// wall values only
byte		spotvis[MAPWDT][MAPHGT];
uint16_t	actorat[MAPWDT][MAPHGT];

int tics;

//
// control info
//
boolean		mouseenabled,joystickenabled,joypadenabled;
int		joystickport;
int		dirscan[4] = {sc_UpArrow,sc_RightArrow,sc_DownArrow,sc_LeftArrow};
int		buttonscan[NUMBUTTONS] =
		{sc_Control,sc_Alt,sc_RShift,sc_Space,sc_1,sc_2,sc_3,sc_4};
int		buttonmouse[4]={bt_attack,bt_strafe,bt_use,bt_nobutton};
int		buttonjoy[4]={bt_attack,bt_strafe,bt_use,bt_run};

boolean		buttonheld[NUMBUTTONS];

boolean		demorecord,demoplayback;
byte		*demoptr, *lastdemoptr;
byte		*demobuffer;

//
// curent user input
//
int		controlx,controly;	/* range from -100 to 100 per tic */
int		strafeoff;
boolean		buttonstate[NUMBUTTONS];

// Get absolute map # for current level
unsigned GetMapIndex() {
return gamestate.mapon + (GAME_WOLF ? 10*gamestate.episode : 0);
}	// GetMapIndex

// Set current level by absolute map #
void SetMapIndex(unsigned level) {
if (GAME_WOLF) {
	gamestate.episode = level/10;
	gamestate.mapon = level%10;
} else {
	gamestate.episode = 0;
	gamestate.mapon = level;
	}
}	// SetMapIndex

static void RemoveObj(objtype *gone);

/*
=============================================================================

						 LOCAL VARIABLES

=============================================================================
*/

/* LIST OF SONGS FOR EACH VERSION */

static const int songs_WOLF[] = {

/* Episode One */
 GETTHEM_MUS,
 SEARCHN_MUS,
 POW_MUS,
 SUSPENSE_MUS,
 GETTHEM_MUS,
 SEARCHN_MUS,
 POW_MUS,
 SUSPENSE_MUS,
 WARMARCH_MUS,	/* Boss level */
 CORNER_MUS,	/* Secret level */

/* Episode Two */
 NAZI_OMI_MUS,
 PREGNANT_MUS,
 GOINGAFT_MUS,
 HEADACHE_MUS,
 NAZI_OMI_MUS,
 PREGNANT_MUS,
 HEADACHE_MUS,
 GOINGAFT_MUS,
 WARMARCH_MUS,	/* Boss level */
 DUNGEON_MUS,	/* Secret level */

/*  Episode Three */
 INTROCW3_MUS,
 NAZI_RAP_MUS,
 TWELFTH_MUS,
 ZEROHOUR_MUS,
 INTROCW3_MUS,
 NAZI_RAP_MUS,
 TWELFTH_MUS,
 ZEROHOUR_MUS,
 ULTIMATE_MUS,	/* Boss level */
 PACMAN_MUS,	/* Secret level */

/* Episode Four */
 GETTHEM_MUS,
 SEARCHN_MUS,
 POW_MUS,
 SUSPENSE_MUS,
 GETTHEM_MUS,
 SEARCHN_MUS,
 POW_MUS,
 SUSPENSE_MUS,
 WARMARCH_MUS,	/* Boss level */
 CORNER_MUS,	/* Secret level */

/* Episode Five */
 NAZI_OMI_MUS,
 PREGNANT_MUS,
 GOINGAFT_MUS,
 HEADACHE_MUS,
 NAZI_OMI_MUS,
 PREGNANT_MUS,
 HEADACHE_MUS,
 GOINGAFT_MUS,
 WARMARCH_MUS,	/* Boss level */
 DUNGEON_MUS,	/* Secret level */

/* Episode Six */
 INTROCW3_MUS,
 NAZI_RAP_MUS,
 TWELFTH_MUS,
 ZEROHOUR_MUS,
 INTROCW3_MUS,
 NAZI_RAP_MUS,
 TWELFTH_MUS,
 ZEROHOUR_MUS,
 ULTIMATE_MUS,	/* Boss level */
 FUNKYOU_MUS	/* Secret level */

};

//////////////////////////////////////////////////////////////
//
// SPEAR OF DESTINY TRACKS
//
//////////////////////////////////////////////////////////////

static const int songs_SPEAR[] = {

 XTIPTOE_MUS,
 XFUNKIE_MUS,
 XDEATH_MUS,
 XGETYOU_MUS,
 ULTIMATE_MUS,	/* Trans Grosse BOSS */

 DUNGEON_MUS,
 GOINGAFT_MUS,
 POW_MUS,
 TWELFTH_MUS,
 ULTIMATE_MUS,	/* Barnacle Wilhelm BOSS */

 NAZI_OMI_MUS,
 GETTHEM_MUS,
 SUSPENSE_MUS,
 SEARCHN_MUS,
 ZEROHOUR_MUS,
 ULTIMATE_MUS,	/* Super Mutant BOSS */

 XPUTIT_MUS,
 ULTIMATE_MUS,	/* Death Knight BOSS */

 XJAZNAZI_MUS,	/* Secret level */
 XFUNKIE_MUS,	/* Secret level */

 XEVIL_MUS	/* Angel of Death BOSS */

};

/*
=================
=
= StartMusic
=
=================
*/

void StartMusic() {
  StartCPMusic ((GAME_WOLF ? songs_WOLF : songs_SPEAR) [GetMapIndex()]);
}

/*
=============================================================================

						  USER CONTROL

=============================================================================
*/


#define BASEMOVE		35
#define RUNMOVE			70

/*
===================
=
= PollKeyboardButtons
=
===================
*/

void PollKeyboardButtons()
{
	int i;

	for (i=0;i<NUMBUTTONS;i++)
		if (IN_KeyDown(buttonscan[i]))
			buttonstate[i] = true;
}


/*
===================
=
= PollMouseButtons
=
===================
*/

void PollMouseButtons()
{
	int	buttons;

	buttons = IN_MouseButtons();

	if (buttons&1)
		buttonstate[buttonmouse[0]] = true;
	if (buttons&2)
		buttonstate[buttonmouse[1]] = true;
	if (buttons&4)
		buttonstate[buttonmouse[2]] = true;
}


/*
===================
=
= PollJoystickButtons
=
===================
*/

void PollJoystickButtons()
{
	int	buttons;

	buttons = IN_JoyButtons();

	if (joystickport && !joypadenabled)
	{
		if (buttons&4)
			buttonstate[buttonjoy[0]] = true;
		if (buttons&8)
			buttonstate[buttonjoy[1]] = true;
	}
	else
	{
		if (buttons&1)
			buttonstate[buttonjoy[0]] = true;
		if (buttons&2)
			buttonstate[buttonjoy[1]] = true;
		if (joypadenabled)
		{
			if (buttons&4)
				buttonstate[buttonjoy[2]] = true;
			if (buttons&8)
				buttonstate[buttonjoy[3]] = true;
		}
	}
}

/*
===================
=
= PollKeyboardMove
=
===================
*/

void PollKeyboardMove()
{
	int shift = (buttonstate[bt_run] ? RUNMOVE : BASEMOVE) * tics;

	if (IN_KeyDown(dirscan[di_north]))
		controly -= shift;
	if (IN_KeyDown(dirscan[di_south]))
		controly += shift;
	if (IN_KeyDown(dirscan[di_west]))
		controlx -= shift;
	if (IN_KeyDown(dirscan[di_east]))
		controlx += shift;

// Process additional movement keys:

	if (IN_KeyDown(sc_A))
		controlx -= shift;
	if (IN_KeyDown(sc_D))
		controlx += shift;
	if (IN_KeyDown(sc_W))
		controly -= shift;
	if (IN_KeyDown(sc_S))
		controly += shift;
	if (IN_KeyDown(sc_Q))
		strafeoff -= shift;
	if (IN_KeyDown(sc_E))
		strafeoff += shift;

	if (IN_KeyDown (sc_LT))
		strafeoff -= shift;
	if (IN_KeyDown (sc_GT))
		strafeoff += shift;
}

/*
===================
=
= PollMouseMove
=
===================
*/

void PollMouseMove()
{
	int mousexmove = 0, mouseymove = 0;

	IN_GetMouseDelta(&mousexmove, &mouseymove);
	
	controlx += mousexmove*10/(13-mouseadjustment);
	controly += mouseymove*20/(13-mouseadjustment);
}

/*
===================
=
= PollJoystickMove
=
===================
*/

#define	JOYTHRESH	64

void PollJoystickMove()
{
	int joyx, joyy, shift;

	INL_GetJoyDelta(joystickport,&joyx,&joyy);

	shift = (buttonstate[bt_run] ? RUNMOVE : BASEMOVE) * tics;

	if (joyx > JOYTHRESH)
		controlx += shift;
	else if (joyx < -JOYTHRESH)
		controlx -= shift;
	if (joyy > JOYTHRESH)
		controly += shift;
	else if (joyy < -JOYTHRESH)
		controly -= shift;
}

void UpdateInput()
{
//
// get button states
//
	PollKeyboardButtons();

	if (mouseenabled)
		PollMouseButtons();

	if (joystickenabled)
		PollJoystickButtons();

//
// get movements
//
	PollKeyboardMove();

	if (mouseenabled)
		PollMouseMove();

	if (joystickenabled)
		PollJoystickMove();
}

/*
===================
=
= PollControls
=
= Gets user or demo input, call once each frame
=
= controlx		set between -100 and 100 per tic
= controly
= buttonheld[]	the state of the buttons LAST frame
= buttonstate[]	the state of the buttons THIS frame
=
===================
*/

void PollControls()
{
	int max, min, i;
	byte buttonbits;

	strafeoff = controlx = controly = 0;
	memcpy(buttonheld, buttonstate, sizeof(buttonstate));
	memset(buttonstate, 0, sizeof(buttonstate));

	if (demoplayback) {
	//
	// read commands from demo buffer
	//
		buttonbits = *demoptr++;
		for (i = 0; i < NUMBUTTONS; i++)
		{
			buttonstate[i] = buttonbits&1;
			buttonbits >>= 1;
		}

		controlx = (signed char)*demoptr++;
		controly = (signed char)*demoptr++;

		if (demoptr == lastdemoptr)
			playstate = ex_completed;		// demo is done

		controlx *= (int)tics;
		controly *= (int)tics;

		return;
	}

	/* Update keys */
	IN_CheckAck(); 
	
	UpdateInput();
	
//
// bound movement to a maximum
//
	min = - (max = 100*tics);

	if (controlx > max)
		controlx = max;
	else if (controlx < min)
		controlx = min;

	if (controly > max)
		controly = max;
	else if (controly < min)
		controly = min;

	if (demorecord)
	{
	//
	// save info out to demo buffer
	//
		controlx /= (int)tics;
		controly /= (int)tics;

		buttonbits = 0;

		for (i=NUMBUTTONS-1;i>=0;i--)
		{
			buttonbits <<= 1;
			if (buttonstate[i])
				buttonbits |= 1;
		}

		*demoptr++ = buttonbits;
		*demoptr++ = controlx;
		*demoptr++ = controly;

		if (demoptr >= lastdemoptr)
			Quit("Demo buffer overflowed!");

		controlx *= (int)tics;
		controly *= (int)tics;
	}
}

//==========================================================================



///////////////////////////////////////////////////////////////////////////
//
//	CenterWindow() - Generates a window of a given width & height in the
//		middle of the screen
//
///////////////////////////////////////////////////////////////////////////

#define MAXX	320
#define MAXY	160

void CenterWindow(word w,word h)
{
	US_DrawWindow(((MAXX / 8) - w) / 2,((MAXY / 8) - h) / 2,w,h);
}

//===========================================================================

#define	STR_DEBMENU	"Press tilde (~) for debug menu"

void CheatMessage (char *msg) {
 ClearMemory();
 CA_CacheGrChunk(STARTFONT+1);
 ClearSplitVWB();

 Message(msg);

 CA_UnCacheGrChunk(STARTFONT+1);
 IN_ClearKeysDown();
 IN_Ack();

 DrawPlayBorder();
}	// CheatMessage

// Debug menu
void DebugMenu ();

/*
=====================
=
= LevelProgress
=
=====================
*/

// Game level name
char *GameLevel () {
static char buffer[16];
unsigned level = gamestate.mapon+1;

if (GAME_WOLF)
	sprintf (buffer, "E%u/L%u", gamestate.episode+1, level);
else
	sprintf (buffer, "L%u", level);

return buffer;
}	// GameLevel

// Game difficulcy
char *GameDiff () {
    switch (gamestate.difficulty) {
	case gd_baby:			return "baby";
	case gd_easy:			return "easy";
	case gd_medium:		return "normal";
	case gd_hard:			return "hard";
	}

return "";
}	// GameDiff

// Game time
static char *GameTime() {
static char buffer[16];
unsigned secs=gamestate.TimeCount/70;

sprintf (buffer, "%u:%02u", secs/60, secs%60);

return buffer;
}	// GameTime

void LevelProgress() {
char buffer [256];

#define	PROGRESS_MSG				\
	"Level progress\n(%s, %s)\n\n"		\
						\
	"Kills: %u/%u (%u%%)\n"			\
	"Secrets: %u/%u (%u%%)\n"		\
	"Treasures: %u/%u (%u%%)\n"		\
						\
	"\nTime: %s"

#define	CALC_RATIO(count,total)			\
	count, total, (count*100)/total

sprintf (buffer, PROGRESS_MSG,
	GameLevel(), GameDiff(),
	CALC_RATIO (gamestate.killcount, gamestate.killtotal),
	CALC_RATIO (gamestate.secretcount, gamestate.secrettotal),
	CALC_RATIO (gamestate.treasurecount, gamestate.treasuretotal),
	GameTime());

CheatMessage (buffer);
}	// LevelProgress

void InfoWindow(int w, int h, char *str) {
CA_CacheGrChunk (STARTFONT);
fontnumber=0;
SETFONTCOLOR(2,13);

CenterWindow (w,h);
US_PrintCentered (str);
US_UpdateWindow();

IN_Ack();
DrawPlayBorder();

CA_UnCacheGrChunk(STARTFONT);
}		// InfoWindow

/*
=====================
=
= CheckKeys
=
=====================
*/

void CheckKeys()
{
	byte	scan;

	if (screenfaded || demoplayback) // don't do anything with a faded screen
		return;

	scan = LastScan;

	//
	// Pause mode
	//
	if (Paused) {
		VWB_DrawPicSync(128, 64, PAUSEDPIC);

		SD_MusicOff();
		IN_Ack();
		IN_ClearKeysDown();
		SD_MusicOn();

		Paused = false;
		return;
	}

	//
	// View size adjust
	//

	if (IN_KeyDown(sc_Plus) && viewsize != MAXVIEWSIZE) {
		NewViewSize (viewsize+1);
		SD_PlaySound(HITWALLSND);
		DrawPlayBorder();
		VW_UpdateScreen();
		}

	if (IN_KeyDown(sc_Minus) && viewsize != MINVIEWSIZE) {
		NewViewSize (viewsize-1);
		SD_PlaySound(HITWALLSND);
		DrawPlayBorder();
		VW_UpdateScreen();
		}

	//
	// Weapon cycle
	//

	if (IN_KeyDown(sc_LtBr)) {
		CycleWeapon (false);
		IN_ClearKeysDown();
		}

	if (IN_KeyDown(sc_RtBr)) {
		CycleWeapon (true);
		IN_ClearKeysDown();
		}

	//
	// F1-F7/ESC to enter control panel
	//
	if (
		scan == sc_F10 ||
		scan == sc_F9 ||
		scan == sc_F7 ||
		scan == sc_F8)			// pop up quit dialog
	{
		ClearMemory();
		ClearSplitVWB();
		US_ControlPanel(scan);
	
		DrawPlayBorder();
		VW_UpdateScreen();
		
		if (scan == sc_F9)
			StartMusic();

		SETFONTCOLOR(0,15);
		IN_ClearKeysDown();
		
		lasttimecount = get_TimeCount();
		
		return;
	}

	if ( (scan >= sc_F1 && scan <= sc_F9) || scan == sc_Escape)
	{
		StopMusic();
		ClearMemory();
		VW_FadeOut();

		US_ControlPanel(scan);

		SETFONTCOLOR(0,15);
		IN_ClearKeysDown();
		DrawPlayScreen();
		VW_UpdateScreen();
		
		if (!startgame && !loadedgame)
		{
			VW_FadeIn();
			StartMusic();
		}
		if (loadedgame)
			playstate = ex_abort;
		lasttimecount = get_TimeCount();
		
		return;
	}

	//
	// Toggle map mode
	//

	if (IN_KeyDown (sc_Tab)) {
		IN_ClearKeysDown();
		map_mode = !map_mode;
		}

	//
	// Map scroll/zoom
	//
	if (map_mode) {
	    if (IN_KeyDown (scan))
		switch (scan) {
		    case sc_Insert:	MapViewZoom (true, 1);
					break;
		    case sc_Delete:	MapViewZoom (true, -1);
					break;

		    case sc_Home:	MapViewScroll (false, -1, -1);
					break;
		    case sc_End:	MapViewScroll (false, -1,  1);
					break;
		    case sc_PgUp:	MapViewScroll (false,  1, -1);
					break;
		    case sc_PgDn:	MapViewScroll (false,  1,  1);
					break;

		    case sc_5:
		    case sc_6:
		    case sc_7:
		    case sc_8:
		    case sc_9:		MapViewZoom (false, scan - sc_1);
					break;
		    }	// switch

		}

	//
	// Debug menu
	//

	if (IN_KeyDown (sc_Tilde)) {
		DebugMenu ();
		lasttimecount = get_TimeCount();
		return;
		}

	//
	// Game stats
	//

	if (IN_KeyDown (sc_Ques)) {
		LevelProgress ();
		return;
		}

	//
	// Quick toggle keys
	//

	if (IN_KeyDown (sc_Alt)) {

		if (IN_KeyDown (sc_M)) {
			InfoWindow (9, 2, (mouseenabled ^= 1) ? "Mouse ON" : "Mouse OFF");
			}

		else if (IN_KeyDown (sc_J)) {
			InfoWindow (12, 2, (joystickenabled ^= 1) ? "Joystick ON" : "Joystick OFF");
			}

		else if (IN_KeyDown (sc_G)) {
			InfoWindow (16, 2, VL_ToggleMouseGrab() ? "Mouse capture ON" : "Mouse capture OFF");
			}

		else if (IN_KeyDown (sc_F)) {
			InfoWindow (14, 2, VL_ToggleFullScreen() ? "Fullscreen ON" : "Fullscreen OFF");
			}

		else if (IN_KeyDown (sc_S)) {
			SD_WaitSoundDone();
			SD_SetSoundMode(SoundMode == sdm_On ? sdm_Off : sdm_On);
			InfoWindow (14, 2, SoundMode == sdm_On ? "FM Sound ON" : "FM Sound OFF");
			}

		else if (IN_KeyDown (sc_D)) {
			SD_SetDigiDevice(DigiMode == sds_On ? sds_Off : sds_On);
			InfoWindow (18, 2, DigiMode == sds_On ? "Digi Sound ON" : "Digi Sound OFF");
			}

		else if (IN_KeyDown (sc_T)) {
			SD_SetMusicMode(MusicMode == smm_On ? smm_Off : smm_On);
			if (MusicMode == smm_On) StartMusic();
			InfoWindow (12, 2, MusicMode == smm_On ? "Music ON" : "Music OFF");
			}

		return;
		}

	//
	// SECRET CHEAT CODE: TAB-G-F10 (OBSOLETE!)
	//
	if (IN_KeyDown(sc_Tab) && IN_KeyDown(sc_G) && IN_KeyDown(sc_F10)) {
		CheatMessage(STR_DEBMENU);
		return;
	}

	//
	// SECRET CHEAT CODE: 'MLI' (OBSOLETE!)
	//
	if (IN_KeyDown(sc_M) &&	IN_KeyDown(sc_L) && IN_KeyDown(sc_I)) {
		CheatMessage(STR_DEBMENU);
		return;
	}

	//
	// OPEN UP DEBUG KEYS (OBSOLETE!)
	//
	if (IN_KeyDown(sc_BackSpace) &&
	    IN_KeyDown(sc_LShift) && IN_KeyDown(sc_Alt)) {
		CheatMessage(STR_DEBMENU);
		return;
	}

	//
	// TRYING THE KEEN CHEAT CODE!
	//
	if (IN_KeyDown(sc_B) && IN_KeyDown(sc_A) && IN_KeyDown(sc_T)) {
		CheatMessage (STR_KEEN);
		return;
	}

}

//===========================================================================

/*
#############################################################################

				  The objlist data structure

#############################################################################

objlist containt structures for every actor currently playing.  The structure
is accessed as a linked list starting at *player, ending when ob->next ==
NULL.  GetNewObj inserts a new object at the end of the list, meaning that
if an actor spawn another actor, the new one WILL get to think and react the
same frame.  RemoveObj unlinks the given object and returns it to the free
list, but does not damage the objects ->next pointer, so if the current object
removes itself, a linked list following loop can still safely get to the
next element.

<backwardly linked free list>

#############################################################################
*/


/*
=========================
=
= InitActorList
=
= Call to clear out the actor object lists returning them all to the free
= list.  Allocates a special spot for the player.
=
=========================
*/

void InitActorList()
{
	int	i;

//
// init the actor lists
//
	for (i = 0; i != MAXACTORS; i++)
	{
		objlist[i].id = i;
		objlist[i].prev = &objlist[i+1];
		objlist[i].next = NULL;
	}

	objlist[MAXACTORS-1].prev = NULL;

	objfreelist = &objlist[0];
	lastobj = NULL;

/* give the player the first free spots */
	GetNewActor();
	player = new;

}

//===========================================================================

/*
=========================
=
= GetNewActor
=
= Sets the global variable new to point to a free spot in objlist.
= The free spot is inserted at the end of the liked list
=
=========================
*/

void GetNewActor()
{
	int id;
	
	if (!objfreelist)
		Quit("GetNewActor: No free spots in objlist!");
	
	new = objfreelist;
	id = new->id;
	objfreelist = new->prev;
	
	memset(new, 0, sizeof(*new));
	new->id = id;
	
	if (lastobj)
		lastobj->next = new;
	new->prev = lastobj;	// new->next is already NULL from memset

	new->active = false;
	lastobj = new;
}

//===========================================================================

/*
=========================
=
= RemoveObj
=
= Add the given object back into the free list, and unlink it from its
= neighbors
=
=========================
*/

static void RemoveObj(objtype *gone)
{
	if (gone == player)
		Quit("RemoveObj: Tried to remove the player!");

	gone->state = s_none;

//
// fix the next object's back link
//
	if (gone == lastobj)
		lastobj = gone->prev;
	else
		gone->next->prev = gone->prev;

//
// fix the previous object's forward link
//
	gone->prev->next = gone->next;

//
// add it back in to the free list
//
	gone->prev = objfreelist;
	objfreelist = gone;

}

/*
=============================================================================

						MUSIC STUFF

=============================================================================
*/


/*
=================
=
= StopMusic
=
=================
*/

void StopMusic()
{
	FreeMusic();
}

/*
=============================================================================

					PALETTE SHIFTING STUFF

=============================================================================
*/

#define NUMREDSHIFTS		6
#define REDSTEPS		8

#define NUMWHITESHIFTS		3
#define WHITESTEPS		20
#define WHITETICS		6

byte	redshifts[NUMREDSHIFTS][768];
byte	whiteshifts[NUMREDSHIFTS][768];

int	damagecount,bonuscount;
boolean	palshifted;

/*
=====================
=
= InitRedShifts
=
=====================
*/

void InitRedShifts()
{
	byte *workptr;
	const byte *baseptr;

	int	i,j,delta;


//
// fade through intermediate frames
//
	for (i=1;i<=NUMREDSHIFTS;i++)
	{
		workptr = (byte *)&redshifts[i-1][0];
		baseptr = gamepal;

		for (j=0;j<=255;j++)
		{
			delta = 64-*baseptr;
			*workptr++ = *baseptr++ + delta * i / REDSTEPS;
			delta = -*baseptr;
			*workptr++ = *baseptr++ + delta * i / REDSTEPS;
			delta = -*baseptr;
			*workptr++ = *baseptr++ + delta * i / REDSTEPS;
		}
	}

	for (i=1;i<=NUMWHITESHIFTS;i++)
	{
		workptr = (byte *)&whiteshifts[i-1][0];
		baseptr = gamepal;

		for (j=0;j<=255;j++)
		{
			delta = 64-*baseptr;
			*workptr++ = *baseptr++ + delta * i / WHITESTEPS;
			delta = 62-*baseptr;
			*workptr++ = *baseptr++ + delta * i / WHITESTEPS;
			delta = 0-*baseptr;
			*workptr++ = *baseptr++ + delta * i / WHITESTEPS;
		}
	}
}


/*
=====================
=
= ClearPaletteShifts
=
=====================
*/

void ClearPaletteShifts()
{
	bonuscount = damagecount = 0;
}


/*
=====================
=
= StartBonusFlash
=
=====================
*/

void StartBonusFlash()
{
	bonuscount = NUMWHITESHIFTS*WHITETICS;		// white shift palette
}

/*
=====================
=
= StartDamageFlash
=
=====================
*/

void StartDamageFlash(int damage)
{
	damagecount += damage;
}

/*
=====================
=
= UpdatePaletteShifts
=
=====================
*/

void UpdatePaletteShifts()
{
	int red, white;

	if (bonuscount)
	{
		white = bonuscount/WHITETICS + 1;
		if (white>NUMWHITESHIFTS)
			white = NUMWHITESHIFTS;
		bonuscount -= tics;
		if (bonuscount < 0)
			bonuscount = 0;
	}
	else
		white = 0;


	if (damagecount)
	{
		red = damagecount/10 +1;
		if (red>NUMREDSHIFTS)
			red = NUMREDSHIFTS;

		damagecount -= tics;
		if (damagecount < 0)
			damagecount = 0;
	}
	else
		red = 0;

	if (red)
	{
		VL_SetPalette(redshifts[red-1]);
		palshifted = true;
	}
	else if (white)
	{
		VL_SetPalette(whiteshifts[white-1]);
		palshifted = true;
	}
	else if (palshifted)
	{
		VL_SetPalette(gamepal);		// back to normal
		palshifted = false;
	}
}


/*
=====================
=
= FinishPaletteShifts
=
= Resets palette to normal if needed
=
=====================
*/

void FinishPaletteShifts()
{
	if (palshifted)
	{
		palshifted = 0;
		VL_SetPalette(gamepal);
	}
}


/*
=============================================================================

						CORE PLAYLOOP

=============================================================================
*/


/*
=====================
=
= DoActor
=
=====================
*/

void DoActor(objtype *ob)
{
	void (*think)(objtype *);

	if (!ob->active && !areabyplayer[ob->areanumber])
		return;

	if (!(ob->flags & (FL_NONMARK|FL_NEVERMARK)))
		actorat[ob->tilex][ob->tiley] = 0;

//
// non transitional object
//

	if (!ob->ticcount)
	{
		think =	gamestates[ob->state].think;
		if (think)
		{
			think(ob);
			if (ob->state == s_none)
			{
				RemoveObj (ob);
				return;
			}
		}

		if (ob->flags&FL_NEVERMARK)
			return;

		if ((ob->flags&FL_NONMARK) && actorat[ob->tilex][ob->tiley])
			return;

		actorat[ob->tilex][ob->tiley] = ob->id | AM_OBJID;
		return;
	}

//
// transitional object
//
	ob->ticcount-=tics;
	while (ob->ticcount <= 0)
	{
		think = gamestates[ob->state].action;	// end of state action
		if (think)
		{
			think(ob);
			if (ob->state == s_none)
			{
				RemoveObj(ob);
				return;
			}
		}

		ob->state = gamestates[ob->state].next;

		if (ob->state == s_none)
		{
			RemoveObj(ob);
			return;
		}

		if (!gamestates[ob->state].tictime)
		{
			ob->ticcount = 0;
			goto think;
		}

		ob->ticcount += gamestates[ob->state].tictime;
	}

think:
	//
	// think
	//
	think =	gamestates[ob->state].think;
	if (think)
	{
		think(ob);
		if (ob->state == s_none)
		{
			RemoveObj(ob);
			return;
		}
	}

	if (ob->flags&FL_NEVERMARK)
		return;

	if ((ob->flags&FL_NONMARK) && actorat[ob->tilex][ob->tiley])
		return;

	actorat[ob->tilex][ob->tiley] = ob->id | AM_OBJID;
}

//==========================================================================


/*
===================
=
= PlayLoop
=
===================
*/

// (2 second sampling)
#define	RATE_INTERVAL	TickBase*2

// (last framerate measured)
unsigned frame_rate = 0;

long funnyticount;

void PlayLoop()
{
	long last_time, last_frame;

	playstate = lasttimecount = 0;
	
	last_frame = frameon = 0;
	anglefrac = 0;
	facecount = 0;
	funnyticount = 0;
	
	memset (buttonstate,0,sizeof(buttonstate));
	ClearPaletteShifts();
	
	if (demoplayback)
		IN_StartAck();

	set_TimeCount(0);
	last_time = 0;
	
	do
	{
		/* get timing info for last frame */
		CalcTics();
		
		/* handle input */
		PollControls();

//
// actor thinking
//
		madenoise = false;

		MoveDoors();
		MovePWalls();

		for (obj = player; obj; obj = obj->next)
			DoActor(obj);

		UpdatePaletteShifts();

		ThreeDRefresh();

		//
		// MAKE FUNNY FACE IF BJ DOESN'T MOVE FOR AWHILE
		//

		if (GAME_SPEAR) {
		funnyticount += tics;
		if (funnyticount > 30l*70) {
			face_funny = 1;
			funnyticount = 0;
			facecount = 0;
			DrawFace ();
		}
		}

		gamestate.TimeCount += tics;

 		UpdateSoundLoc(player->x, player->y, player->angle);

		if (screenfaded)
			VW_FadeIn();

		CheckKeys();

//
// debug aids
//
		{	long timecount = get_TimeCount();
			if (timecount - last_time > RATE_INTERVAL) {
			frame_rate =
				(unsigned)(frameon - last_frame) * TickBase /
					(timecount - last_time);

			last_time = timecount;
			last_frame = frameon;
			}
		}

		if (singlestep)
		{
			wait_Ticks(14);
			lasttimecount = get_TimeCount();
		}

		if (demoplayback)
		{
			if (IN_CheckAck())
			{
				IN_ClearKeysDown();
				playstate = ex_abort;
			}
		}

	} while (!playstate && !startgame);

	if (playstate != ex_died)
		FinishPaletteShifts();
}
