
/*
 * +==========================================================
 * |	'wl_resmap.c':
 * |	Pagefile resources mapping (sprites/digi sounds)
 * |
 * |	LoneWolf (Wolf3D/Spear of Destiny engine) source code
 * |    by D.G.Gayev (dgaev@mail.ru), 2004-2005
 * +==========================================================
 *
 */

#include "wl_def.h"

unsigned sprite_map [SPR_TOTAL];
unsigned reverse_sprite_map [SPR_TOTAL];

unsigned spr_count = 0;

static void MapSpriteRange (int start, int end) {
while (start <= end)
	reverse_sprite_map [spr_count ++] = start ++;
}	// MapSpriteRange

void MapSpritePages () {

MapSpriteRange (SPR_DEMO, SPR_DEATHCAM);

MapSpriteRange (SPR_STAT_0, SPR_STAT_47);

if (GAME_SPEAR)
	MapSpriteRange (SPR_STAT_48, SPR_STAT_51);

MapSpriteRange (SPR_GRD_S_1, SPR_OFC_SHOOT3);

if (GAME_WOLF)
	MapSpriteRange (SPR_BLINKY_W1, SPR_GIFT_DEAD);

MapSpriteRange (SPR_ROCKET_1, SPR_BOOM_3);

if (GAME_SPEAR)
	MapSpriteRange (SPR_HROCKET_1, SPR_SPARK4);

if (GAME_WOLF)
	MapSpriteRange (SPR_GRETEL_W1, SPR_BJ_JUMP4);

else	MapSpriteRange (SPR_TRANS_W1, SPR_ANGEL_DEAD);

MapSpriteRange (SPR_KNIFEREADY, SPR_CHAINATK4);

MapSpriteRange (SPR_NULLSPRITE, SPR_NULLSPRITE);

//
// Calculate normal sprite mapping
//

{ int i;

for (i = 0; i != SPR_TOTAL; ++ i)
	sprite_map [i] = -1;

for (i = 0; i != spr_count; ++ i)
	sprite_map [reverse_sprite_map [i]] = i;

}

}	// MapSpritePages

//
// (Sounds map)
//

unsigned sound_map [SND_END - SND_BEG];

static void MapSound (int sound, int val) {
if (SND_BEG <= sound && sound < SND_END)
    sound_map [sound - SND_BEG] = val;
}	// MapSound

int LocateSound (int val) {
int i;
for (i = 0; i != SND_END-SND_BEG; ++ i)
    if (sound_map [i] == val)
	return i+SND_BEG;

return -1;
}	// LocateSound

void MapSoundPages () {

int i;

for (i = SND_BEG; i != SND_END; ++ i)
	sound_map [i - SND_BEG] = -1;

if (GAME_WOLF) {
	MapSound (HALTSND,                0);
	MapSound (DOGBARKSND,             1);
	MapSound (CLOSEDOORSND,           2);
	MapSound (OPENDOORSND,            3);
	MapSound (ATKMACHINEGUNSND,       4);
	MapSound (ATKPISTOLSND,           5);
	MapSound (ATKGATLINGSND,          6);
	MapSound (SCHUTZADSND,            7);
	MapSound (GUTENTAGSND,            8);
	MapSound (MUTTISND,               9);
	MapSound (BOSSFIRESND,            10);
	MapSound (SSFIRESND,              11);
	MapSound (DEATHSCREAM1SND,        12);
	MapSound (DEATHSCREAM2SND,        13);
	MapSound (DEATHSCREAM3SND,        13);
	MapSound (TAKEDAMAGESND,          14);
	MapSound (PUSHWALLSND,            15);
	MapSound (LEBENSND,               20);
	MapSound (NAZIFIRESND,            21);
	MapSound (SLURPIESND,             22);
	MapSound (YEAHSND,                32);

	if (is_game == GAME_WL6) {
	MapSound (DOGDEATHSND,            16);
	MapSound (AHHHGSND,               17);
	MapSound (DIESND,                 18);
	MapSound (EVASND,                 19);
	MapSound (TOT_HUNDSND,            23);
	MapSound (MEINGOTTSND,            24);
	MapSound (SCHABBSHASND,           25);
	MapSound (HITLERHASND,            26);
	MapSound (SPIONSND,               27);
	MapSound (NEINSOVASSND,           28);
	MapSound (DOGATTACKSND,           29);
	MapSound (LEVELDONESND,           30);
	MapSound (MECHSTEPSND,            31);

	MapSound (SCHEISTSND,             33);
	MapSound (DEATHSCREAM4SND,        34);	/* AIIEEE */
	MapSound (DEATHSCREAM5SND,	  35);	/* DEE-DEE */
	MapSound (DONNERSND,		  36);	/* EPISODE 4 BOSS DIE */
	MapSound (EINESND,		  37);	/* EPISODE 4 BOSS SIGHTING */
	MapSound (ERLAUBENSND,		  38);	/* EPISODE 6 BOSS SIGHTING */
	MapSound (DEATHSCREAM6SND,	  39);	/* FART */
	MapSound (DEATHSCREAM7SND,	  40);	/* GASP */
	MapSound (DEATHSCREAM8SND,	  41);	/* GUH-BOY! */
	MapSound (DEATHSCREAM9SND,	  42);	/* AH GEEZ! */
	MapSound (KEINSND,		  43);	/* EPISODE 5 BOSS SIGHTING */
	MapSound (MEINSND,		  44);	/* EPISODE 6 BOSS DIE */
	MapSound (ROSESND,		  45);	/* EPISODE 5 BOSS DIE */
	}	// (GAME_WL6)
	}	// (GAME_WOLF)

else {		// (GAME_SPEAR)
	MapSound (HALTSND,                0);
	MapSound (CLOSEDOORSND,           2);
	MapSound (OPENDOORSND,            3);
	MapSound (ATKMACHINEGUNSND,       4);
	MapSound (ATKPISTOLSND,           5);
	MapSound (ATKGATLINGSND,          6);
	MapSound (SCHUTZADSND,            7);
	MapSound (BOSSFIRESND,            8);
	MapSound (SSFIRESND,              9);
	MapSound (DEATHSCREAM1SND,        10);
	MapSound (DEATHSCREAM2SND,        11);
	MapSound (TAKEDAMAGESND,          12);
	MapSound (PUSHWALLSND,            13);
	MapSound (AHHHGSND,               15);
	MapSound (LEBENSND,               16);
	MapSound (NAZIFIRESND,            17);
	MapSound (SLURPIESND,             18);
	MapSound (LEVELDONESND,           22);
	MapSound (DEATHSCREAM4SND,        23);	// AIIEEE
	MapSound (DEATHSCREAM3SND,        23);	// DOUBLY-MAPPED!!!
	MapSound (DEATHSCREAM5SND,        24);	// DEE-DEE
	MapSound (DEATHSCREAM6SND,        25);	// FART
	MapSound (DEATHSCREAM7SND,        26);	// GASP
	MapSound (DEATHSCREAM8SND,        27);	// GUH-BOY!
	MapSound (DEATHSCREAM9SND,        28);	// AH GEEZ!
	MapSound (GETGATLINGSND,          38);	// Got Gat replacement

	if (is_game == GAME_SOD) {
	MapSound (DOGBARKSND,             1);
	MapSound (DOGDEATHSND,            14);
	MapSound (SPIONSND,               19);
	MapSound (NEINSOVASSND,           20);
	MapSound (DOGATTACKSND,           21);
	MapSound (TRANSSIGHTSND,	  29);	// Trans Sight
	MapSound (TRANSDEATHSND,	  30);	// Trans Death
	MapSound (WILHELMSIGHTSND,	  31);	// Wilhelm Sight
	MapSound (WILHELMDEATHSND,	  32);	// Wilhelm Death
	MapSound (UBERDEATHSND,		  33);	// Uber Death
	MapSound (KNIGHTSIGHTSND,	  34);	// Death Knight Sight
	MapSound (KNIGHTDEATHSND,	  35);	// Death Knight Death
	MapSound (ANGELSIGHTSND,	  36);	// Angel Sight
	MapSound (ANGELDEATHSND,	  37);	// Angel Death
	MapSound (GETSPEARSND,		  39);	// Got Spear replacement
	}	// (GAME_SOD)
	}	// (GAME_SPEAR)

}	// MapSoundPages

